
#
# BioPerl module for Bio::Pfam
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam - Object factory that provides Pfam based objects

=head1 SYNOPSIS

    $db = Bio::Pfam::default_db(); # gets a Bio::Pfam::DB object from
                                   # defualts

=head1 DESCRIPTION

This is really a package, not an object, providing the defaults for a system
for the Pfam databases. By using only the functionality in this and the
Bio::Pfam::DB module you should be insulated from the changes of the database
layout.

This module allows you to instantiate database objects. It is here where the
defaults of the database system are held. You should have a look at 
Bio::Pfam::DB for what methods you are allowed to use.

=head1 CONTACT

Describe contact details here

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal 
methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam;
use vars qw($AUTOLOAD @ISA);
use strict;
no strict 'vars';          # mmmm - this could be tightened up
                           # specifically export what we want to

use Bio::Pfam::DB;
use Bio::Pfam::DB_RCS;
use Bio::Pfam::DB_Web;
use Bio::Pfam::DB_RDB;

use UpdateRDB;

$pfam_root_dir  = $ENV{'PFAM_ROOT_DIR'};
if( ! $pfam_root_dir ) {
    $pfam_root_dir  = '/nfs/disk100/pubseq/Pfam';
}

# first, general srs database names for all underlying implementations

$srs_pfamseq = 'pfamseq';
$srs_pfamseq_label = 'PFAMSEQ';
$srs_pfamseqnew_label = 'PFAMSEQNEW';
$srs_pfam = 'pfamnew';
$srs_pfamb = 'pfambnew';
$srs_swisspfam = 'swisspfamnew';

# now rcs specific variables

my $name_of_current_dir = "CURRENT";
my $name_of_master_dir = "RCS_MASTER";
my $name_of_attic_dir = "RCS_ATTIC";
my $name_of_accession_dir = "ACCESSION";
my $name_of_accession_attic_dir = "$name_of_accession_dir/ATTIC";
my $name_of_acc_map_file = "accmap.dat";
my $name_of_acc_map_lock = "accmap.lock";

# globals, for convenient reading from other modules

$rcs_current_dir = "$pfam_root_dir/$name_of_current_dir";
$rcs_master_dir  = "$pfam_root_dir/$name_of_master_dir";
$rcs_attic_dir   = "$pfam_root_dir/$name_of_attic_dir";
$rcs_accession_dir           = "$pfam_root_dir/$name_of_accession_dir";
$rcs_accession_attic_dir = "$pfam_root_dir/$name_of_accession_attic_dir";

$fasta_db        = '/nfs/disk100/pubseq/pfamseqnew/pfamseq';

# rcs file lists - not used here but in PfamRCS and PfamQC

# the order of the files in each set is important as checkouts occur
# in order, and checkins check all the timestamps!

@annotation_file_set = ("DESC");
@family_file_set     = ("SEED","HMM_ls","HMM_fs","OUTPUT_ls","OUTPUT_fs","ALIGN","scores");
@view_file_set       = ("SEED.ann","HMM_ls.ann","HMM_fs.ann","ALIGN.ann", "SEED.tree", "ALIGN.tree");
@optional_file_set   = ("trues","notes","BAIT","PRIOR");

# now web-specific variables

$web_data_root = '/nfs/WWW/htdocs/Software/Pfam/data/';

#now RDB specific variables

$rdb_host = "webdbsrv1";
$rdb_driver = "mysql";
$rdb_user = "pfam";
$rdb_pass = "mafp1";

$live_rdb_name = "pfamlive";
$mirror_rdb_name = "pfam";
$mirror_new_rdb_name = "pfam2";
$development_rdb_name = "pfamdev";


=head2 default_db

 Title   : default_db
 Usage   : $db = &Bio::Pfam->default_db();
 Function: returns the default Pfam::DB object for this system
           This could be an RCS db or a flat-file based db -
           you shouldnt care
 Example :
 Returns : new Bio::Pfam::DB object (probably something that inheriets it in fact).
 Args    :


=cut

sub default_db{
    return Bio::Pfam::DB_RCS->new('-current' => $rcs_current_dir, 
				  '-attic' => $rcs_attic_dir,
				  '-index' => "$rcs_accession_dir/$name_of_acc_map_file", 
				  '-lock_file' => "$rcs_accession_dir/$name_of_acc_map_lock", 
				  '-fastadb' => $fasta_db, 
				  '-srsdb' => $srs_pfamseq,
				  '-srspfamb' => $srs_pfamb, 
				  '-srsswisspfam' => $srs_swisspfam 
				  );
} 



=head2 local_rcs_db

 Title   : local_rcs_db
 Usage   : $db = &Bio::Pfam->local_rcs_db( $database_root_directory )
 Function: returns the default Pfam::DB object for a database sitting
    under the specified directory. This database should have the same
    structure as the core RCS database. Certain database operations
    will be missing due to lack of information (for example an underlying
    srs pfamseq database)						
 Example :
 Returns : new Bio::Pfam::DB object (probably something that inheriets it in fact).
 Args    :


=cut

sub local_rcs_db{
    my $self = shift;
    my $local_root = shift;

    return Bio::Pfam::DB_RCS->new('-current' => "$local_root/$name_of_current_dir", 
				  '-attic' => "$local_root/$name_of_attic_dir",
				  '-index' => "$local_root/$name_of_accession_dir/$name_of_acc_map_file", 
				  '-lock_file' => "$local_root/$name_of_accession_dir/$name_of_acc_map_lock", 
				  ); 
} 



=head2 latest_release

 Title   : latest_release
 Usage   : $db = &Bio::Pfam::latest_release();
 Function: returns the latest fixed release of Pfam
           as Pfam::DB object
 Returns : a new Bio::Pfam::DB object (probably something that inheriets it in fact)
 Args    :

=cut

sub latest_release{
   my ($self) = @_;

    return Bio::Pfam::DB_Web->new('-datadir' => $web_data_root,
				  '-srsdb' => $srs_pfamseq, 
				  '-srspfam' => $srs_pfam,
				  '-srspfamb' => $srs_pfamb,
				  '-srsswisspfam' => $srs_swisspfam,
				  '-db_name' => $mirror_rdb_name,
				  '-db_host' => $rdb_host,
				  '-db_driver' => $rdb_driver,
				  '-db_user' => $rdb_user,
				  '-db_password' => $rdb_pass);
}




=head2 live_rdb

 Title   : live_rdb
 Usage   : $db = &Bio::Pfam->live_rdb();
 Function: Returns a PfamRDB object which contains methods for update and 
    query if the live Pfam relational database. This is NOT an implementation
    of DB.pm, rather an interface for writing to the RDB.
 Returns : a new Bio::Pfam::PfamRDB object
 Args    :

=cut

sub live_rdb {
   my ($self) = @_;

   return Bio::Pfam::DB_RDB->new('-db_name' => $live_rdb_name,
				 '-db_driver' => $rdb_driver, 
				 '-db_host' => $rdb_host,
				 '-db_user' => $rdb_user,
				 '-db_password' => $rdb_pass);

}



=head2 release_rdb

 Title   : release_rdb
 Usage   : $db = &Bio::Pfam->release_rdb();
 Function: Returns a PfamRDB object which contains methods for update and 
    query if the live Pfam relational database. This is NOT an implementation
    of DB.pm, rather an interface for writing to the RDB.
 Returns : a new Bio::Pfam::PfamRDB object
 Args    :

=cut

sub release_rdb {
   my ($self) = @_;

   return Bio::Pfam::DB_RDB->new('-db_name' => $mirror_rdb_name,
				 '-db_driver' => $rdb_driver, 
				 '-db_host' => $rdb_host,
				 '-db_user' => $rdb_user,
				 '-db_password' => $rdb_pass);

}


=head2 new_release_rdb

 Title   : new_release_rdb
 Usage   : $db = &Bio::Pfam->new_release_rdb();
 Function: Returns a PfamRDB object which contains methods for update and 
    query if the mirror ('new') RDB used to store the new data before it
    goes live
 Args    :

=cut

sub new_release_rdb {
   my ($self) = @_;

   return Bio::Pfam::DB_RDB->new('-db_name' => $mirror_new_rdb_name,
				 '-db_driver' => $rdb_driver, 
				 '-db_host' => $rdb_host,
				 '-db_user' => $rdb_user,
				 '-db_password' => $rdb_pass);

}


=head2 development_rdb

 Title   : development_rdb
 Usage   : $db = &Bio::Pfam->development_rdb();
 Function: Returns a PfamRDB object which contains methods for update and 
    query if the mirror ('new') RDB used to store the new data before it
    goes live
 Args    :

=cut

sub development_rdb {
   my ($self) = @_;

   return Bio::Pfam::DB_RDB->new('-db_name' => $development_rdb_name,
				 '-db_driver' => $rdb_driver, 
				 '-db_host' => $rdb_host,
				 '-db_user' => $rdb_user,
				 '-db_password' => $rdb_pass);

}



=head2 live_rdb_update

 Title   : live_rdb_update
 Usage   :
 Function:
 Returns : 
 Args    :

=cut

sub live_rdb_update{
    my ($self) = @_;

    my $dont = $ENV{'DONT_UPDATE_PFAM_RDB'};

    if (defined $dont) {
	if ($dont =~ /true/i) {
	    return undef;
	}
	else {
	    my $mess = "UpdateRDB - ";
	    $mess .= "expecting DONT_UPDATE_PFAM_RDB to be true or undefined; ";
	    $mess .= "Found it to be $dont";
	    $self->throw( $mess );
       }
    }
    else {
	return UpdateRDB->new('-db_name' => $live_rdb_name,
			      '-db_driver' => $rdb_driver, 
			      '-db_host' => $rdb_host,
			      '-db_user' => $rdb_user,
			      '-db_password' => $rdb_pass);
    }    

}



=head2 release_rdb_update

 Title   : release_rdb_update
 Usage   :
 Function:
 Returns : 
 Args    :

=cut

sub release_rdb_update{
    my ($self) = @_;

    my $dont = $ENV{'DONT_UPDATE_PFAM_RDB'};

    if (defined $dont) {
	if ($dont =~ /true/i) {
	    return undef;
	}
	else {
	    my $mess = "UpdateRDB - ";
	    $mess .= "expecting DONT_UPDATE_PFAM_RDB to be true or undefined; ";
	    $mess .= "Found it to be $dont";
	    $self->throw( $mess );
       }
    }
    else {
	return UpdateRDB->new('-db_name' => $mirror_rdb_name,
			      '-db_driver' => $rdb_driver, 
			      '-db_host' => $rdb_host,
			      '-db_user' => $rdb_user,
			      '-db_password' => $rdb_pass);
    }    

}




=head2 switchover_rdb_update

 Title   : switcover_rdb_update
 Usage   :
 Function:
 Returns : 
 Args    :

=cut

sub switchover_rdb_update{
    my ($self) = @_;

    my $dont = $ENV{'DONT_UPDATE_PFAM_RDB'};

    if (defined $dont) {
	if ($dont =~ /true/i) {
	    return undef;
	}
	else {
	    my $mess = "UpdateRDB - ";
	    $mess .= "expecting DONT_UPDATE_PFAM_RDB to be true or undefined; ";
	    $mess .= "Found it to be $dont";
	    $self->throw( $mess );
       }
    }
    else {
	return UpdateRDB->new('-db_name' => $mirror_new_rdb_name,
			      '-db_driver' => $rdb_driver, 
			      '-db_host' => $rdb_host,
			      '-db_user' => $rdb_user,
			      '-db_password' => $rdb_pass);
    }    

}



=head2 development_rdb_update

 Title   : development_rdb_update
 Usage   :
 Function:
 Returns : 
 Args    :

=cut

sub development_rdb_update{
    my ($self) = @_;

    my $dont = $ENV{'DONT_UPDATE_PFAM_RDB'};

    if (defined $dont) {
	if ($dont =~ /true/i) {
	    return undef;
	}
	else {
	    my $mess = "UpdateRDB - ";
	    $mess .= "expecting DONT_UPDATE_PFAM_RDB to be true or undefined; ";
	    $mess .= "Found it to be $dont";
	    $self->throw( $mess );
       }
    }
    else {
	return UpdateRDB->new('-db_name' => $development_rdb_name,
			      '-db_driver' => $rdb_driver, 
			      '-db_host' => $rdb_host,
			      '-db_user' => $rdb_user,
			      '-db_password' => $rdb_pass);
    }    

}


1;

