
#
# BioPerl module for Bio::Pfam::Bait::BaitObject
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::Bait::BaitObject - Sub-Family Representative object; 
Internal representation of a bait file in the bait system.

=head1 SYNOPSIS

    $bait = new Bio::Pfam::Bait::BaitObject('-PFAM_ACCESSION' => $domAccession,
					    '-PFAM_ID' => $id,
					    '-AUTHOR' => $origin,
					    '-METHOD' => $method,
					    '-DOMAIN_SEARCH' => $trueOrFalse);

=head1 DESCRIPTION

This object encapsulates the workings of the bait system. Its structure models that of a
bait file.

There is tight coupling between the method field and the threshold fields of each member
of the list of BaitUnits in that the thresholds are only valid when the method of comparison
is used.

The -DOMAIN_SEARCH field is a boolean, which should be true iff a domain-only search
is required against this BaitUnit (i.e. if the domain of interest should be cut out
of the query sequence and the search performed between the query sequence segment
and the complete BaitUnit sequence). If false, the complete query sequence will be
used in the search against the BaitUnit sequence. 

This object makes no assumptions about the structure of the bait files themselves. It
leaves such issues to the BaitEngine object, which acts as an intermediary between
the bait files / pfam database and the BaitObject, andalso does the work

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...

package Bio::Pfam::Bait::BaitObject;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

# Object preamble - inherits from Bio::Root::Object

use Bio::Root::Object;
use Bio::Pfam::Bait::BaitUnit;
use Bio::Pfam::Bait::BaitRegion;

@ISA = qw(Bio::Root::Object Exporter);
#@EXPORT_OK = qw();
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params) = @_;
  my($accession, $id, $author, $method, $domainSearch) = 
      (($params{'-PFAM_ACCESSION'}||$params{'-pfam_accession'}), ($params{'-PFAM_ID'}||$params{'-pfam_id'}), 
       ($params{'-AUTHOR'}||$params{'-author'}), ($params{'-METHOD'}||$params{'-method'}), 
       ($params{'-DOMAIN_SEARCH'}||$params{'domain_search'}));

# the Object stores UniqueID (?), Author, list of BaitUnits, and list of delegates.
# All of this information will be read in from the bait file.

  my $make = $self->SUPER::_initialize;

  # Note: for now, I am using the Pfam accesion numnber as a unique identifier for
  # the bait object. This assumes a one family : one bait file set-up. It may be
  # necessaey in the future to add an extra unique id field to the object, and let
  # the bait database take care of how to fill in the object appropriately

  $self->{'baitObj_accession'} = $accession;
  $self->{'baitObj_id'} = $id;
  $self->{'baitObj_author'} = $author;
  $self->{'baitObj_method'} = $method;
  $self->{'baitObj_isDomainSearchNeeded'} = $domainSearch;
  $self->{'baitObj_unitList'} = {};
  $self->{'baitObj_delegateList'} = [];
  $self->{'baitObj_indexedSequences'}  = 

  return $make; # success - we hope!
}




=head2 accession

 Title   : accession
 Usage   : 
    $baitObj->accession(); $baitObj->accession( $pfam_acc );
 Function: sets or returns this BaitObject objects id field
 Returns : The pfam accession number as id string
 Args    : The pfam accession number as an id string (optional)

=cut

sub accession{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'baitObj_accession'} = $value;
   }
   return $self->{'baitObj_accession'};
}




=head2 addDelegate

 Title   : addDelegate
 Usage   : 
    $baitObj->addDelegate( $pfam_accession );
 Function:
    The list of delegates records to which BaitUnits this BaitUnit should
    yield in the annotation process. This is a way of adding to that
    list
 Returns : 
 Args    :A pfam accession number.

=cut

sub addDelegate{
   my ($self, $accession) = @_;

   push @{$self->{ 'baitObj_delegateList'}}, $accession;
}




=head2 addBaitUnit

 Title   : addBaitUnit
 Usage   : $baitObj->addBaitUnit( Bio::Pfam::Bait::BaitUnit->new('-ID' => $id, 
								 '-DB_NAME' => $dbName, 
								 '-FROM' => $start, 
								 '-TO' => $end, 
								 '-THRESHOLD' => $threshold,
								 '-SEQUENCE' => $sequenceRef,  	
								 '-ANNOTATION' => $annotationRef) );
 Function: The object is basically a holder for a list of BaitUnit objects. This provides 
    a method of adding to this list. The method checks for duplicate accession-keyed entries, 
    to allow for the storage of possibly several (threshold, annotation) pairs for an entry
 Returns : 
 Args    : A reference to an BaitUnit

=cut

sub addBaitUnit{
   my ($self, $unitRef) = @_;

   my $newid = $unitRef->id();
   while ( defined $self->{ 'baitObj_unitList'}{ $newid } ) {
       # The list of pairs in the given unitRef is bound to contain a single
       # element. Even it it does not, we just extend the list in the matched
       # entry in the BaitObject with the first element of the list
       
       # First though, check that the sequence for the duplicate is the same

       if (not $unitRef->sequence()->out_raw() eq 
	   $self->{'baitObj_unitList'}{$newid}->sequence()->out_raw() ) {
	   # Right, we have a different sequence with the same id as an existing
	   # one. It may be a segment, so give it a unique id

	   # does the id contain an _number at the end?
	   if ($newid =~ /^(.*)_(\d+)$/ ) { 
	       $newid = $1.'_'.($2+1);
	   }
	   else {
	       $newid .= "_1";
	   }

	   $unitRef->id( $newid );

       }
       else {
	   # we need to add the th/an pairs from the given unit to the match we have found
	   my ($pair) = $unitRef->eachThresholdAnnotationPair();
	   $self->{ 'baitObj_unitList'}{ $unitRef->id()}->addThresholdAnnotationPair( $pair );

	   # we have done everything necessary, so just return now
	   return;
       }
   }
   $self->{ 'baitObj_unitList'}{ $unitRef->id() } = $unitRef;

}




=head2 annotate

 Title   : annotate
 Usage   : 
    baitObj->annotate( $annSeqRef, $annRegionRef );
 Function:
    Attempts to construct an BaitRegion for the given AnnotatedSequence
    based on this BaitObject. The given AnnotatedRegion allows the
    extent of the bait annotation to be determined and specified 
 Returns : 
    An BaitRegion reference
 Args    :
    An AnnotatedSequence (ref),
    Am AnnotatedRegion or derivative (ref)

=cut

sub annotate{
   my ($self, $seq, $domain, @allDomains ) = @_;

   my ($theSequence, $annotationStart, $annotationEnd );

   # Firstly, check for delegates

   foreach my $del ($self->eachDelegate()) {
       foreach my $region (@allDomains ) {
	   if ( $del eq $region->accession() ) {
	       return Bio::Pfam::Bait::BaitRegion->new('-TYPE' => $Bio::Pfam::Bait::BaitRegion::_DELEGATE);
	   }
       }
   }

   if ($self->eachBaitUnit()) {
       # Okay. First, get the indexed file for this BaitObject
       my $baitSeqs = $self->getIndexedSequences();

       my $querySeq = "/tmp/".$seq->id().".$$";
       open QUERYSEQ, ">$querySeq" or $self->throw("Could not create temporary file $querySeq\n");
       # now extract the correct part of the query sequence and put it into the file.
       if (not $self->isDomainSearchNeeded() ) {
	   $theSequence = $seq;
	   $annotationStart = 1;
	   $annotationEnd = $theSequence->seq_len();
       }
       else {
	   $theSequence = Bio::Seq->new('-id' => $seq->id(),
					'-seq' => $seq->str($domain->from(),$domain->to())
					);
	   $annotationStart = $domain->from();
	   $annotationEnd = $domain->to();
       }
       print QUERYSEQ $theSequence->out_fasta();
       close QUERYSEQ;
       
       my ($highestScore, $entryWithHighestScore, $theAnnotation);
       
       open( SEARCH, $self->method()." $baitSeqs $querySeq B=1 V=1 |" ) or 
	   $self->throw("Search command initiated unsuccessfully");
       while( <SEARCH> ) {
	   if (/^>(\S+)/) {
	       # we have found an id line
	       $entryWithHighestScore = $1;
	   }
	   if (/^\s+Score\s*=\s*(\d+)/) {
	       if ($1 > $highestScore ) {
		   $highestScore = $1;
	       }
	   }
       }
       close SEARCH or $self->throw("Search command terminated unsuccessfully");
       unlink $querySeq;
       
       # Now check whether the score is significant
       
       if ($entryWithHighestScore ){
	   # i.e if there actually was a high scoring entry
	   my $unit = $self->{'baitObj_unitList'}->{$entryWithHighestScore};   
	   
	   foreach my $pair (sort { $b->threshold() <=> $a->threshold() } 
			     $unit->eachThresholdAnnotationPair() ) {
	       if ($highestScore > $pair->threshold()) {

		   $theAnnotation = Bio::Annotation->new('-DESCRIPTION' => $pair->annotation()->description() );
		   last;
	       }
	   }
       }
       
       # The following line will remove the db file and query file, but not the index files,
       # hence the unlink call
       
       if (not defined $theAnnotation ) {

	   return Bio::Pfam::Bait::BaitRegion->new('-TYPE' => $Bio::Pfam::Bait::BaitRegion::_DEFAULT,
						   '-HIGHEST_BAIT' => $entryWithHighestScore,
						   '-SCORE_OF_HIGHEST_BAIT' => $highestScore,
						   '-FROM' => $annotationStart,
						   '-TO' => $annotationEnd,
						   '-ANNOTATION' => Bio::Annotation->new()
					    );
       }
       else {
	   return Bio::Pfam::Bait::BaitRegion->new('-TYPE' => $Bio::Pfam::Bait::BaitRegion::_BAIT,
						   '-HIGHEST_BAIT' => $entryWithHighestScore,
						   '-SCORE_OF_HIGHEST_BAIT' => $highestScore,
						   '-FROM' => $annotationStart,
						   '-TO' => $annotationEnd,
						   '-ANNOTATION' => $theAnnotation
					     );      
       }
   }
   else {
       return  Bio::Pfam::Bait::BaitRegion->new('-TYPE' => $Bio::Pfam::Bait::BaitRegion::_DEFAULT,
						'-FROM' => $domain->from(),
						'-TO' => $domain->to(),
						'-ANNOTATION' => Bio::Annotation->new()
						);
   }
}





=head2 author

 Title   : author
 Usage   : 
    $baitObj->author(); $baitObj->author( $theAuthor );
 Function: sets or returns this BaitObjects Author field
 Returns : A string
 Args    : A string (optional)

=cut

sub author{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'baitObj_author'} = $value;
   }
   return $self->{'baitObj_author'};
}




=head2 eachDelegate

 Title   : eachDelegate
 Usage   : 
    @delegateList = $baitObj->eachDelegate();
 Function:
    The list of delegates records to which BaitUnits this BaitUnit should
    yield in the annotation process. This is a way of adding to that
    list
 Returns : A list of accession number strings
 Args    :

=cut

sub eachDelegate{
   my ($self ) = @_;

   return @{$self->{'baitObj_delegateList'}};
}




=head2 eachBaitUnit

 Title   : eachBaitUnit
 Usage   : 
    @unitList = $baitObj->eachBaitUnit();
 Function:
    The list of BaitUnit records that make up this BaitObject
 Returns : A list of BaitUnit references
 Args    :

=cut

sub eachBaitUnit{
   my ($self ) = @_;

   return values %{$self->{'baitObj_unitList'}};
}



=head2 getIndexedSequences

 Title   : getIndexedSequences
 Usage   :
    $baitObj->getIndexedSequences()
 Function:
    This function looks in the tmp directory for an indexed bait sequences file for this BaitObject
 Returns : An absolute file name (which is what blast needs)
 Args    : 

=cut

sub getIndexedSequences{
   my ($self,@args) = @_;
   my ($name);

   if (defined $self->indexedSequences() ) {
       $name = $self->indexedSequences();
   }
   else {
       $name = "/tmp/".$self->accession().$self->author().".baitfa$$";
       open BAITSEQS, ">$name" or $self->throw("Could not create temporary indexed file $name");
       # extract the Bait sequences and put them in the temp file
       foreach my $unit ( $self->eachBaitUnit() ) {
	   print BAITSEQS $unit->sequence()->out_fasta();
       }
       close BAITSEQS;
       
       # supress stdout while we do it
       open SAVEOUT, ">&STDOUT" or $self->throw("could not save stdout");
       open STDOUT, "/dev/null" or $self->throw("Could not redirect sdtout"); 
       
       system( "setdb", "$name") == 0 or $self->throw("Could not index the sequence file $!"); 
       
       close STDOUT;
       open STDOUT, ">&SAVEOUT" or $self->throw("Could not restore stdout");

       # now store the name of the indexed file inside the object so that we know what to delete later
       $self->indexedSequences( $name );
   }
   return $name;
}




=head2 id

 Title   : id
 Usage   : 
    $baitObj->id(); $baitObj->id( $pfam_acc );
 Function: sets or returns this BaitObject objects id field
 Returns : id string
 Args    : id string (optional)

=cut

sub id{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'baitObj_id'} = $value;
   }
   return $self->{'baitObj_id'};
}




=head2 indexedSequences

 Title   : indexedSequences
 Usage   :
    $baitObj->indexedSequences(); $baitObj->indexedSequences( $name );
 Function:
    Stores the full name of the indexed sequences file
 Returns : the name of the indexed sequence file
 Args    : an indexed sequence filename (optional)

=cut

sub indexedSequences{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'baitObj_indexedSequences'} = $value;
   }
   return $self->{'baitObj_indexedSequences'};
}




=head2 isDomainSearchNeeded

 Title   : isDomainSearchNeeded()
 Usage   : 
    if ($baitObj->isDomainSearchNeeded() ) { ...
 Function: Returns a boolean indicating whether a domain only search is needed
     for the records in this BaitObject (see description)
 Returns : A boolean: True iff a domain-only search is desired
 Args    : None

=cut

sub isDomainSearchNeeded{
   my ($self, $value ) = @_;

   if (defined $value) {
       $self->{'baitObj_isDomainSearchNeeded'} = $value;
   }
   return $self->{'baitObj_isDomainSearchNeeded'};
}




=head2 method

 Title   : method
 Usage   : 
    $baitObj->method(); $baitObj->method( $theMethod );
 Function: sets or returns this BaitObjects method field
 Returns : A string which should be parsable as a search command string
 Args    : A string which should be parsable as a search command (optional)

=cut

sub method{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'baitObj_method'} = $value;
   }
   return $self->{'baitObj_method'};
}


=head2 putIndexedSequences

 Title   : putIndexedSequences
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :

=cut

sub putIndexedSequences{
   my ($self,@args) = @_;


}




=head2 read

 Title   : read
 Usage   :
    $emptyBaitObj->read( $fh );
 Function:
    The BaitObject fills itself via given bait file
 Returns : 
 Args    : A file handle for the bait file

=cut

sub read{
   my ($self, $baitFile, $author) = @_;

   my ($sequenceAccumulator, $lastRecord, $shouldRead);

   # the following are per object fields
   my ($objectAuthor, $objectMethod, $objectSearchType, @delegates );
    
   # the following are per unit fields
   my ($unitId, $unitFromTo, $unitThreshold, $unitAnnotationString);


   while (<$baitFile>) {
       if (/^#/) {
	   # comment line; ignore
	   next;
       }
       elsif (/^ACCESSION:\s*(\S+)/i) {
	   $self->accession($1);
       }
       elsif (/^ID:\s*(\S+)/i) {
	   $self->id($1);
       }
       elsif (/^AUTHOR:\s*(\S+)/i) {
	   $objectAuthor = $1;
	   if ($author eq $objectAuthor) {
	       $shouldRead = 1;
	       $self->author( $objectAuthor );
	   }
	   else {
	       $shouldRead = 0;
	   }
       }
       elsif (/^METHOD:\s*(\S+)/i ) {
	   if ($shouldRead) {
	       $self->method($1);
	   }
       }
       elsif (/^SEARCHTYPE:\s*(\S+)/i) {
	   if ($shouldRead) {
	       my $objectSearchType = $1;
	       $objectSearchType =~ tr/A-Z/a-z/;
	       $self->isDomainSearchNeeded( ($objectSearchType eq "domain")?1:0 );
	   }
       } 	
       elsif (/^DELEGATE:\s*/i) {
	   if ($shouldRead) {
	       @delegates =  split /\s+/, $';
	       foreach my $del (@delegates) {
		   $self->addDelegate( $del );
	       }
	   }
       }
       elsif ( ($unitId, $unitFromTo, $unitThreshold, $unitAnnotationString )
	       = /^>(.+?);(.+?);(.+?);(.*)$/ )  {
	   if ($shouldRead) {
	       # We may be here after processing header information. Lets check and create object if necessary
		   
	       if (! defined($self->accession() ) ) {
		   $self->throw("An ACCESSION number could not be extracted from the BAIT file");
	       }
	       if (! defined($self->id() ) ) {
		   $self->throw("An ID could not be extracted from the BAIT file");
	       }
	       if (! defined($self->author()) ) {
		   $self->throw("An AUTHOR could not be extracted from the BAIT file");
	       }
	       if (! defined($self->method()) ) {
		   $self->throw("A METHOD could not be extracted from the BAIT file");
	       }
	   
	       
	       # Now we have processed all of the sequence from the previous record, add it
	       if ( defined $lastRecord ) {
		   $sequenceAccumulator =~ s/\s+//g;
		   $lastRecord->sequence( Bio::Seq->new('-id' => $lastRecord->id(),
							'-seq'=> $sequenceAccumulator 
							 ) 
					  );
		   $self->addBaitUnit( $lastRecord );
		   
		   $sequenceAccumulator = "";
	       }
	       my ($from, $to) = ($unitFromTo =~ /(\S+)-(\S+)/);
	       $lastRecord = 
		 Bio::Pfam::Bait::BaitUnit->new('-ID' => $unitId,
						'-FROM' => $from,
						'-TO' => $to,
						'-THRESHOLD' => $unitThreshold,
						'-ANNOTATION' => 
				              Bio::Annotation->new( '-DESCRIPTION' => $unitAnnotationString )
						);
    	   }
       }
       else {
	   # we must have a sequence line, I hope
	   if ($shouldRead) {
	       $sequenceAccumulator .= $_;
	   }
       }
   }
   if ( defined $lastRecord ) {
       $sequenceAccumulator =~ s/\s+//g;
       $lastRecord->sequence( Bio::Seq->new('-id' => $lastRecord->id(),
					    '-seq' => $sequenceAccumulator
					     ) 
			      );
       $self->addBaitUnit( $lastRecord );
   }
   # finally, reset the file pinter to the start, justin case we need to read again
   seek $baitFile, 0, 0;
}



=head2 write

 Title   : write
 Usage   : 
    $baitObj->write( /*HANDLE, $hraderNeeded );
 Function: Spits out the contents of the BaitObject to the given file handle
 Returns : 
 Args    : The destination file handle

=cut

sub write{
   my ($self, $file, $headerNotNeeded) = @_;

   if (! $headerNotNeeded) {
       print $file "ACCESSION: ".$self->accession()."\n";
       print $file "ID: ".$self->id()."\n";
   }
   print $file "AUTHOR: ".$self->author()."\n";
   print $file "METHOD: ".$self->method()."\n";
   print $file "SEARCHTYPE: ".($self->isDomainSearchNeeded()?"domain":"all")."\n";
   print $file "DELEGATE:";
   foreach my $del ($self->eachDelegate() ) {
       print $file " $del";
   }
   print $file "\n";
   foreach my $unit ($self->eachBaitUnit() ) {
       $unit->write( $file );
   }   
}
