
#
# BioPerl module for Bio::Pfam::Bait::ThresholdAnnotationPair
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::Bait::ThresholdAnnotationPair - Encapsulates the concept of an annotation being 
associated with a comparison achieving a certain threshold or above (in some 
scoring system).

=head1 SYNOPSIS

Another object will likely use this class by maintaining a list of instances
sorted by threshold for example.

    use Bio::Pfam::Bait::ThresholdAnnotationPair;

    $holder->addThresholdAnnotationPair( Bio::Pfam::Bait::ThresholdAnnotationPair->
					 new('-THRESHOLD' => $thres,
					     '-ANNOTATION' => $annotRef ) );

=head1 DESCRIPTION

When deciding the annotation for a sequence in some annotation system
that uses scores, it may the case that different annotations are appropriate
for different scores. This class enforces the 1-1 relationship between 
threshold and annotation  

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::Bait::ThresholdAnnotationPair;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

# Object preamble - inherits from Bio::Root::Object

use Bio::Root::Object;


@ISA = qw(Bio::Root::Object Exporter);

# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params) = @_;
  my($threshold, $annotRef) = 
      ( ($params{'-THRESHOLD'}||$params{'-threshold'}), ($params{'-ANNOTATION'}||$params{'-annotation'}) );

  my $make = $self->SUPER::_initialize;

  $self->{'pair_threshold'} = $threshold;
  $self->{'pair_annotation'} = $annotRef;

 return $make; # success - we hope!
}



=head2 annotation

 Title   : annotation
 Usage   : 
    $pair->annotation(); # or ...
    $pair->annotation( $annotRef );
 Function: For setting and getting the ANNOTATION field in the object

=cut

sub annotation{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'pair_annotation'} = $value;
   }
   return $self->{'pair_annotation'};
}



=head2 threshold

 Title   : threshold
 Usage   : 
    $pair->threshold(); $pair->threshold( $theThreshold );
 Function: sets or returns this object's threshold field
 Returns : A number (the threshold)
 Args    : A number (the threshold) (optional)

=cut

sub threshold{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'pair_threshold'} = $value;
   }
   return $self->{'pair_threshold'};
}


