
#
# BioPerl module for Bio::Pfam::DB_Web
#
# Cared for by Mhairi Marshall <mm1@sanger.ac.uk>
#
# Copyright Pfam
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::DB_Web - The Web version of Pfam::DB

=head1 SYNOPSIS    A pfamseq identifier (ref to a list of)
    A list of region types needed
 Notes   : 
    1. The AnnotatedSequence returned has no enclosed sequence,
    just annotation. If the sequence is required, use 
    get_Seq_pfamseq to get a Bio::Pfam::SeqPfam and insert it into
    the AnnotatedSequence method using 
    AnnotatedSequence->sequence().

    2. Region-type items can be:
        'seed'
        'full'
        'pfamb'
        'other'
    When the list is empty, all regions are returned. If not, then 
    if the specified region types are availalble they will be 
    returned, but the types specified will not be returned


This is a concrete implementation of the abstract database object
found in Bio::Pfam::DB

=head1 DESCRIPTION

Describe the object here

=head1 CONTACT

pfam@sanger.ac.uk

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::DB_Web;

use vars qw(@ISA);
use strict;
use Bio::Pfam::SeqPfam;
use FileHandle;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Pfam::DB;
use Bio::Pfam::EntryA_Web;
use Bio::Pfam::EntryB_Impl;
use Bio::Pfam::PfamAnnSeqFactory;

## RDB VARS
use DBI;
use Bio::Pfam::PfamRDB;

@ISA = qw(Bio::Pfam::DB);
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called


sub _initialize {
  my($self, %params) = @_;

  my $make = $self->SUPER::_initialize(%params);

  my ( $datadir, $srspfamseq, $srspfam, $srspfamb, $srsswisspfam, $getz, $nordb) = 
      (($params{'-datadir'} || $params{'-DATADIR'}),
       ($params{'-srsdb'}||$params{'-SRSDB'}),
       ($params{'-srspfam'}||$params{'-SRSPFAM'}) ,
       ($params{'-srspfamb'} || $params{'-SRSPFAMB'}),
       ($params{'-srsswisspfam'} || $params{'-SRSSWISSPFAM'}),
       ($params{'-getz'} || $params{'-GETZ'}),
       ($params{'-no_rdb'} || $params{'-NO_RDB'})
       );


  ## Add parameters for the web site !
  $self->_the_RDB( Bio::Pfam::PfamRDB->new( %params ));

  if( ! -d $datadir ) {
      $self->throw("$datadir is not a directory for web pfam access");
  }
  
  $self->{'dbweb_srspfamseq'} = $srspfamseq;
  $self->{'dbweb_srspfam'} = $srspfam;
  $self->{'dbweb_srspfamb'} = $srspfamb;
  $self->{'dbweb_srsswisspfam'} = $srsswisspfam;
  $self->{'dbweb_getz'} = $getz;

  $self->{'dbweb_datadir'} = $datadir;

  $self->{'no_rdb'} = $nordb;

  # set stuff in self from @args
  return $make; # success - we hope!
}



=head2 id2acc

 Title   : id2acc
 Usage   : $acc = $db->id2acc("pkinase"); 
 Function: converts an id to an accession. Only works for
	   current ids - does not work for previous ids
 Returns : string of the accession, or undef if no accession
 Throws  : Exceptions thrown if this $id does not exist or due to RCS misconfiguration
 Args    :

=cut

sub id2acc{
   my ($self,$id) = @_;
   my ($dbh, $st, $acc, $len);

   eval {
       if (not $self->{'no_rdb'}) {
	   $dbh = $self->_the_RDB->connect();
	   
	   if(defined($dbh)) {
	       $st = $dbh->prepare("select pfamA_acc, model_length from pfamA where pfamA_id = '$id'");
	       $st->execute();
	       ($acc, $len) = $st->fetchrow;
	       $st->finish();
	       $self->_the_RDB->disconnect;
	   }
       }
   };
   
   if ($self->{'no_rdb'} or $@) {
       $acc =  $self->_file_id2acc($id);
   } 
   else {
       if (not defined $acc) {
	   $self->throw("DB_RDB::id2acc - $id is not a valid id for this pfam database");
       }
       elsif (not defined $len) {
	   $self->throw("DB_RDB::id2acc - $id is the id of a dead family");
       }
       
   }  
   return $acc;
}


=head2 _file_id2acc

 Title   : _file_id2acc
 Usage   : $acc = $db->id2acc("pkinase"); 
 Function: converts an id to an accession. Only works for
	   current ids - does not work for previous ids
 Returns : string of the accession, or undef if no accession
 Throws  : Exceptions thrown if this $id does not exist or due to RCS misconfiguration
 Args    :
 Notes   : 

=cut

sub _file_id2acc{
   my ($self,$id) = @_;
   my ($srspfam, $fh, $acc, $temp);

   my $srsdb = $self->{'dbweb_srspfam'};
   $fh = new FileHandle;

   my $getz = $self->{'dbweb_getz'};

   if( $fh->open("$getz -f 'acc' '[$srsdb-id:$id]' |") == 0) {
       $self->throw("Cannot fork getz process off");
   }
   $temp = <$fh>;

   if( $temp =~ /^AC\s+(\S+)/ ) {
       $acc = $1;
   }
   else {
       $self->throw("No Pfam family with that id $id");
   }
   if( $fh->close() == 0 ) {
       $self->throw("Getz pipe failed to close");
   }

   return $acc;
}



=head2 acc2id

 Title   : acc2id
 Usage   : $id = $db->acc2id("PF00001");
 Function: converts an accession to an id
 Returns : the id, throws an exception if no accession, or dead accession
 Args    :

=cut

sub acc2id{
   my ($self,$acc) = @_;
   my ($dbh, $st, $id, $len);
   
   eval {
       if (not $self->{'no_rdb'}) {
	   $dbh = $self->_the_RDB->connect();
	   
	   if(defined($dbh)) {
	       $dbh = $self->_the_RDB->connect();
	       $st = $dbh->prepare("select pfamA_id, model_length from pfamA where pfamA_acc = '$acc'");
	       $st->execute;
	       ($id, $len) = $st->fetchrow;
	       $st->finish;
	       $self->_the_RDB->disconnect;
	   }
       }
   };
   
   if($self->{'no_rdb'} or $@) {
       $id =  $self->_file_acc2id($acc);
   }
   else {
       if (not defined $id) {
	   $self->throw("DB_RDB::acc2id - $acc is not a valid id for this pfam database");
       }
       elsif (not defined $len) {
	   $self->throw("DB_RDB::acc2id - $acc is the id of a dead family");
       }
       
   }
 
   return $id;
}


=head2 _file_acc2id

 Title   : _file_acc2id
 Usage   : $id = $db->acc2id("PF00001");
 Function: converts an accession to an id
 Returns : the id, or undef if no id for that accession
 Args    :
 Notes   : 

=cut

sub _file_acc2id{
   my ($self,$acc) = @_;
   my ($srspfam, $fh, $id, $temp);

   $srspfam = $self->{'dbweb_srspfam'};

  my $getz = $self->{'dbweb_getz'};

   $fh = new FileHandle;

   if( $fh->open("$getz -f 'id' '[$srspfam-acc:$acc]' |") == 0) {
       $self->throw("Cannot fork getz process off");
   }
   $temp = <$fh>;

   if( $temp =~ /^ID\s+(\S+)/ ) {
       $id = $1;
   }
   else {
       $self->throw("No Pfam family with accession $acc");
   }
   if( $fh->close() == 0 ) {
       $self->throw("Getz pipe failed to close");
   }

   return $id;


}


=head2 get_allacc

 Title   : get_allacc
 Usage   : @list = $db->getallacc('ALPHA');
 Function: Gets a list of all accession numbers in the current db
 Returns : a list of accession numbers
 Args    : Either 'ALPHA' or 'DATE' for the sort
           by alphabetical order of id or date.

=cut

sub get_allacc{
   my ($self,$type) = @_;
   my ($dbh, $st, $acc, $id, @acclist, %map);

   eval {
       if (not $self->{'no_rdb'}) {
	   $dbh = $self->_the_RDB->connect();
	   $dbh->prepare("select pfamA_acc, pfamA_id from pfamA where model_length is not null");
	   $st = $dbh->execute();
	   while ( ($acc, $id) = $st->fetchrow) {
	       $map{ $acc } = $id;
	   }
	   $st->finish;
	   $self->_the_RDB->disconnect;
       }
   };
   
   if ($self->{'no_rdb'} or $@) {
       @acclist = $self->_file_get_allacc($type);
   }
   else {  
       if ($type =~ /ALPHA/) {
	   @acclist = sort { lc($map{$a}) cmp lc($map{$b}) } keys %map;
       }
       elsif ($type =~ /DATE/) {
	   @acclist = sort {my ($aa,$bb); $a =~ /^PF+0+(\d+)$/; $aa = $1; $b =~ /^PF+0+(\d+)$/; $bb = $1; return $aa <=> $bb; } keys %map;
       }
       else {
	   $self->throw("Bio::Pfam::DB_RDB - get_allaccs - cannot sort into $type order");
       }
   }

   return @acclist;
}



=head2 _file_get_allacc

 Title   : _file_get_allacc
 Usage   : @list = $db->getallacc('ALPHA');
 Function: Gets a list of all accession numbers in the current db
 Returns : a list of accession numbers
 Args    : Either 'ALPHA' or 'DATE' for the sort
           by alphabetical order of id or date.
=cut

sub _file_get_allacc{
    my ($self, @args) = @_;
    my ($dir,@files,@out,$acc);

    $dir = $self->{'dbweb_datadir'};
    opendir(_DIR,"$dir/desc") || $self->throw("Could not open data directory $dir/desc $!");
    @files = readdir(_DIR);
    closedir(_DIR);

    foreach $acc ( @files ) {
	$acc =~ /^PF/ || next;
	$acc =~ s/\..*$//g;
	push(@out,$acc);
    }

    return @out;

}






=head2 get_AnnotSeqs

 Title   : get_AnnotSeqs
 Usage   : ($annot) = $db->get_annotSeqs( $id );
 Function: gets an Annotated sequence from underlying swisspfam db
 Returns : Bio::Pfam::AnnotatedSequence 
 Args    : A ref to a list of swisspfam (pfamseq) identifier
 Notes   : 
    1. The AnnotatedSequence returned has no enclosed sequence,
    just annotation. If the sequence is required, use 
    get_Seq_pfamseq to get a Bio::Pfam::SeqPfam and insert it into
    the AnnotatedSequence method using 
    AnnotatedSequence->sequence().

    2. Region-type items can be:
        'seed'
        'full'
        'pfamb'
        'other'
    When the list is empty, all regions are returned. If not, then 
    if the specified region types are availalble they will be 
    returned, but the types specified will not be returned

  3. This sub acesses the RDB, if it cant get a connection then it gets the sequences from the flat file version.

=cut


sub get_AnnotSeqs {
   my ($self, $id_list, $type_list) = @_;
   my ($dbh, $st_pfamA_reg_seed, $st_pfamA_reg_full,  $st_pfamB_reg, $st_other_reg, $st_length, $length,
       $seq_id, $st, $en, $score, $mo_st, $mo_en, $bits, $ev, $acc, $id, $desc, $orien,
       $annot, $sou, $all_regs, $mod_len, $tree_order,
       @annseqlist,
       %temp);

 
   my $fac = Bio::Pfam::PfamAnnSeqFactory->instance();
   eval {
       $dbh = $self->_the_RDB->connect();
   };
   if ($@ or not defined( $dbh ) or $self->{'no_rdb'}) { 
       ## Connection to the database failed - get pfama & pfamb from the flatfile
       my @type_list = ("full", "pfamb");
       
       @annseqlist =  $self->_file_get_AnnotSeqs( $id_list, \@type_list);
       
   }
   else {
       if (not $type_list) {
	   $all_regs = "true";	 
       }
       else {
	   foreach my $info_type (@{$type_list}) {
	       if ($info_type =~ /seed/i) {
		   $temp{ "seed" } = 1;
	       }
	       elsif ($info_type =~ /full/i) {
		   $temp{ "full" } = 1;
	       }
	       elsif ($info_type =~ /pfamb/i) {
		   $temp{ "pfamb" } = 1;
	       }
	       elsif ($info_type =~ /other/i) {
		   $temp{ "other" } = 1;
		       
	       }
	       else {
		   $self->throw("Bio::Pfam::DB_RDB->get_AnnotSeqs does not supprto region type $info_type");
	       }
	   }
       }
       
       
       if ($temp{"full"} or $all_regs) {

#	   my $stat = "select seq_start, seq_end, model_start, model_end, ";
#	   $stat .= "bits_score, evalue_score, pfamA_reg_full.pfamA_acc, pfamA_id, description ";
#	   $stat .= "from pfamA_reg_full natural left join pfamA  where pfamseq_id = ?";
	
	  my $stat = "select seq_start, seq_end, model_start, model_end, ";
	  $stat .= "domain_bits_score, domain_evalue_score, pfamA.pfamA_acc, pfamA_id, pfamA.description , pfamA_reg_full.tree_order ";
	  $stat .= "from pfamseq, pfamA, pfamA_reg_full ";
	  $stat .= "where pfamseq.pfamseq_id =  ? and pfamseq.auto_pfamseq = pfamA_reg_full.auto_pfamseq and pfamA_reg_full.auto_pfamA = pfamA.auto_pfamA  and in_full = '1'";
	  ##and significant = '1'  #### BUG FIX !! 


	   $st_pfamA_reg_full = $dbh->prepare($stat);
	   
	   
	   my $q = "select length ";     
	   $q .= "from pfamseq  where pfamseq_id = ?";
	   
	   
	   $st_length = $dbh->prepare($q);
       }
	   
       ### STILL TO BE FIXED
       if ($temp{"seed"}) {
#	   my $stat = "select seq_start, seq_end, ";
#	   $stat .= "pfamA_reg_seed.pfamA_acc, pfamA_id, description ";
#	   $stat .= "from pfamA_reg_seed natural left join pfamA where pfamseq_id = ?";
	   
	 my $stat = "select seq_start, seq_end, ";
           $stat .= "pfamA.pfamA_acc, pfamA_id, pfamA.description ";
           $stat .= "from pfamA_reg_seed, pfamA, pfamseq  where pfamseq.pfamseq_id  = ? and pfamseq.auto_pfamseq = pfamA_reg_seed.auto_pfamseq and pfamA_reg_seed.auto_pfamA = pfamA.auto_pfamA";



	   $st_pfamA_reg_seed = $dbh->prepare($stat);
       }

        ### STILL TO BE FIXED
       if ($temp{"pfamb"} or $all_regs) {
#	   my $stat = "select seq_start, seq_end, pfamB_reg.pfamB_acc, pfamB_id ";
#	   $stat .= "from pfamB_reg natural left join pfamB where pfamseq_id = ?";

	  my $stat = "select distinct seq_start, seq_end, pfamB.pfamB_acc, pfamB_id ";
	#  my $stat = "select distinct * ";
	  $stat .= "from pfamB_reg, pfamB, pfamseq ";
	  $stat .= "where pfamseq_id = ? and pfamB_reg.auto_pfamseq = pfamseq.auto_pfamseq and pfamB_reg.auto_pfamB = pfamB.auto_pfamB;";

	


	   $st_pfamB_reg = $dbh->prepare($stat);
       }

        ### STILL TO BE FIXED
       if ($temp{"other"} or $all_regs) {

#	   my $stat = "select seq_start, seq_end, type_id, source_id, score, orientation ";
#	   $stat .= "from other_reg, pfamseq ";
#	   $stat .= "where pfamseq_id = ? and other_reg.auto_pfamseq = pfamseq.auto_pfamseq";

	   my $stat = "select seq_start, seq_end, type_id, source_id, score, orientation ";
           $stat .= "from other_reg, pfamseq ";
           $stat .= "where pfamseq.pfamseq_id = ? and other_reg.auto_pfamseq = pfamseq.auto_pfamseq";

	   $st_other_reg = $dbh->prepare($stat);
       }
       
       foreach my $in_id (@{$id_list}) {
	   my $annSeq = $fac->createAnnotatedSequence();
	   $annSeq->id( $in_id );
	 
	   if (defined $st_pfamA_reg_full) {
	       $st_pfamA_reg_full->execute($in_id);

	       while ( ($st, $en, $mo_st, $mo_en, $bits, $ev, $acc, $id, $desc, $tree_order)
		       = $st_pfamA_reg_full->fetchrow) {

		   $annot = Bio::Annotation->new('-DESCRIPTION' => $desc );
		   $annSeq->addAnnotatedRegion( Bio::Pfam::PfamRegion->new('-PFAM_ACCESSION' => $acc,
									   '-PFAM_ID' => $id,
									   '-SEQ_ID' => $in_id,
									   '-FROM' => $st,
									   '-TO' => $en,
									   '-MODEL_FROM' => $mo_st,
									   '-MODEL_TO' => $mo_en,
									   '-BITS' => $bits,
									   '-EVALUE' => $ev,
									   '-ANNOTATION' => $annot));
		   
	       }
	       
	       
	       $st_pfamA_reg_full->finish;
	   }
	   
	   
	   
	   if (defined $st_pfamA_reg_seed) {
	       $st_pfamA_reg_seed->execute($in_id);

	       while ( ($st, $en, $acc, $id, $desc)
		       = $st_pfamA_reg_seed->fetchrow) {
	
		   $annot = Bio::Annotation->new('-DESCRIPTION' => $desc );
		   $annSeq->addAnnotatedRegion( Bio::Pfam::PfamRegion->new('-PFAM_ACCESSION' => $acc,
									   '-PFAM_ID' => $id,
									   '-SEQ_ID' => $in_id,
									   '-FROM' => $st,
									   '-TO' => $en,
									   '-ANNOTATION' => $annot));
	       }
	       $st_pfamA_reg_seed->finish;
	   }
	   
	   if (defined $st_pfamB_reg) {
	       $st_pfamB_reg->execute($in_id);
	       
	       while ( ($st, $en, $acc, $id) = $st_pfamB_reg->fetchrow) {
		   $annSeq->addAnnotatedRegion( Bio::Pfam::PfamRegion->new('-PFAM_ACCESSION' => $acc,
									   '-PFAM_ID' => $id,
									   '-SEQ_ID' => $in_id,
									   '-FROM' => $st,
									   '-TO' => $en));
	       }
	       $st_pfamB_reg->finish;
	   }
	   
	   
	   ## Get the length and add to the annotated sequence object
	   my($the_length);
	   if (defined $st_length) {
	       $st_length->execute($in_id);
	       while ( ($length) = $st_length->fetchrow) {
		   
		   $the_length = $length if ($length =~ /[0-9]/);
		   
	       }
	       
	       $annSeq->length( $the_length);
	       $st_length->finish;
	       
	   }

	   
	   if (defined $st_other_reg) {
	       $st_other_reg->execute($in_id);
	       while ( ($st, $en, $id, $sou,$score, $orien ) = $st_other_reg->fetchrow) {
		   $annSeq->addAnnotatedRegion( Bio::Pfam::OtherRegion->new('-SEQ_ID' => $in_id,
									    '-FROM' => $st,
									    '-TO' => $en,
									    '-TYPE' => $id,
									    '-SOURCE' => $sou,
									    '-SCORE' => $score,
									    '-ORIENTATION' => $orien
									    )); 
		   
	       }
	       $st_other_reg->finish;
	   }
	   push @annseqlist, $annSeq;
       }
       
       $self->_the_RDB->disconnect;
   }
   
   return @annseqlist;
}




=head2 _file_get_AnnotSeqs

 Title   : _file_get_AnnotSeqs
 Usage   : ($annot1, $annot2) = $db->get_AnnotSeqs( [$id1, $id2], $type1, $type2 );
 Function: gets Annotated sequencea from underlying swisspfam database
 Returns : Bio::Pfam::AnnotatedSequence (list of) 
 Args    : 
    A pfamseq identifier (ref to a list of)
    A list of region types needed
 Notes   : 
    1. The AnnotatedSequence returned has no enclosed sequence,
    just annotation. If the sequence is required, use 
    get_Seq_pfamseq to get a Bio::Pfam::SeqPfam and insert it into
    the AnnotatedSequence method using 
    AnnotatedSequence->sequence().

    2. Region-type items can be:
        'full'
        'pfamb'
    When the list is empty, all regions are returned. If not, then 
    if the specified region types are availalble they will be 
    returned, but the types specified will not be returned

=cut

sub _file_get_AnnotSeqs{
   my ($self, $input_ids, $type_list) = @_;

   my ($fh, $annotseq, $fac, $swisspfamdb, @retlist, $getz);
   $swisspfamdb = $self->{'dbweb_srsswisspfam'};
   $getz = $self->{'dbweb_getz'};

   $fac = Bio::Pfam::PfamAnnSeqFactory->instance();
 
   $annotseq = $fac->createAnnotatedSequence();

   foreach my $in_id (@{$input_ids}) {
     
       $fh = new FileHandle;
       
       if( $fh->open("$getz -e '[$swisspfamdb-id:$in_id]' |") == 0) {
	   $self->throw("Cannot fork getz process off");
       }

       
       $fac->addSwissPfamToAnnSeq( $annotseq, $fh, $type_list);

       if( $fh->close() == 0 ) {
	   $self->throw("Getz pipe failed to close");
       }
     
   }
   push @retlist, $annotseq;
   return @retlist;
}



=head2 get_Seq_pfamseq

 Title   : get_Seq_pfamseq($id)
 Usage   : $seq = $db->get_Seq_pfamseq($id)
 Function: gets a Bio::Pfam::SeqPfam object from the underlying database
 Returns : a newly made Bio::Pfam::SeqPfam object
 Args    :
    1. name of sequence
    2. Whether the name is an accession number or id ('id' or 'acc')
    3. Whether the actual sequence itself is required
 Notes   :
    Arg 3 is ignored in this database implementation, since the sequences themselves
    are not strored in the pfam relational database

=cut

sub get_Seq_pfamseq{
   my ($self,$input_id, $lookup, $seqrequired) = @_;
   my ($dbh, $st, $id, $acc, $desc, $len, $org, $out, $seq);
   eval {
       if (not $self->{'no_rdb'}) {
	   $dbh = $self->_the_RDB->connect();
	   if(defined($dbh)) {
	       #   if ($lookup =! /acc/i) {
	       if ($lookup =~  /acc/i) {
		   $st = "select pfamseq_id, pfamseq_acc, description, length, species from pfamseq where pfamseq_acc = '$input_id'";
	       }
	       else {
		   $st = "select pfamseq_id, pfamseq_acc, description, length, species  from pfamseq where pfamseq_id = '$input_id'";
	       }
	       my($stat) = $dbh->prepare($st);
	       $stat->execute();
	       ($id, $acc, $desc, $len, $org) = $stat->fetchrow;
	       $stat->finish;
	       $self->_the_RDB->disconnect;
	   }
       }
   };
   if ($@ or $self->{'no_rdb'}) {
       $out = $self->_file_get_Seq_pfamseq($input_id, $lookup, $seqrequired);   
   }
   else {
       if ($seqrequired) {
	   my $srspfamseq = $self->{'dbweb_srspfamseq'};
	   my $fh = new FileHandle;
   	  my $getz = $self->{'dbweb_getz'};
	   if( $fh->open("$getz -f seq -sf fasta '[$srspfamseq-$lookup:$input_id]' |") == 0) {
	       $self->throw("Cannot fork getz process off (DB_Web::get_filehandle_pfamseq)");
	   }

	   $seq = "";
	   my $id = undef;
	   my $readingseq = 0;
	   
	   while (<$fh>) {
	       if ($readingseq) {
		   /^>/ && $self->throw("Request $id gave multiple sequences in db. Problem!");	   
		   s/\W//g;
		   $seq .= $_;
	       }

	       elsif ( /^>(\S+)\s/ ) {
		   $readingseq = 1;
	       }
	   }
	   if( $fh->close() == 0 ) {
	       $self->throw("Getz pipe failed to close");
	   }
       }

       $org =~ s/\.$//; $org = lc $org;
       $desc =~ s/\.$//; $desc = lc $desc;
       
       ($acc and $id and $desc and $org and $len) or
	   $self->throw("DB_RDB::get_Seq_pfamseq - Information missing");   
       $out = Bio::Pfam::SeqPfam->new('-seq' => $seq,
				      '-start' => '1',
				      '-end' => $len,
				      '-id'=> $id,
				      '-acc' => $acc,
				      '-organism' => $org,
				      '-desc' => $desc);
   
   } 

   return $out;
}




=head2 _file_get_Seq_pfamseq

 Title   : _file_get_Seq_pfamseq($id)
 Usage   : $seq = $db->get_Seq_pfamseq($id)
 Function: gets a Bio::Pfam::SeqPfam object from the underlying database
 Returns : a newly made Bio::Pfam::SeqPfam object
 Args    :


=cut

sub _file_get_Seq_pfamseq{
   my ($self,$input_id, $lookup, $seqrequired) = @_;
   my ($temp,$id,$seq,$fh,$out, $srspfamseq, $id, $acc, $readingseq, $desc, $org, $tax, $seqcmd);
   if (not defined $lookup) { 
       $lookup = 'id';
   }
   $seqcmd = ($seqrequired)?"-f 'seq' -sf fasta":"";
   $srspfamseq = $self->{'dbweb_srspfamseq'};

   $fh = new FileHandle;
   	  my $getz = $self->{'dbweb_getz'};
   if( $fh->open("$getz -f 'id' -f 'acc' -f 'des' -f 'org' -f 'tax' $seqcmd '[$srspfamseq-$lookup:$input_id]' |") == 0) {
       $self->throw("Cannot fork getz process off (DB_Web::get_filehandle_pfamseq)");
   }

   $seq = "";
   $desc = "";
   $id = undef;
   $readingseq = 0;

   while (<$fh>) {
       /^\/\// and last;

       if ($readingseq) {
	   /^>/ && $self->throw("Request $id gave multiple sequences in db. Problem!");	   
	   s/\W//g;
	   $seq .= $_;
       }
       elsif (/^AC\s+(\S+);/) {
	   $acc = $1;
       }
       elsif (/^ID\s+(\S+)/) {
	   $id = $1;
       }
       elsif (/^DE\s+(.*)/) {
	   if ($desc) {
	       $desc .= " $1";
	   }
	   else {
	       $desc = $1;
	   }
       }
       elsif (/^OS\s+(.*)/) {
	   if ($org) {
	       $org .= " $1";
	   }
	   else {
	       $org = $1;
	   }
       }
       elsif (/^OC\s+(.*)/) {
	   if ($tax) {
	       $tax .= " $1";
	   }
	   else {
	       $tax = $1;
	   }
       }
       elsif ( /^>(\S+)\s/ ) {
	   $readingseq = 1;
       }
   }

   if( $fh->close() == 0 ) {
       $self->throw("Getz pipe failed to close");
   }

   $org =~ s/\.$//; $org = lc $org;
   $tax = lc $tax;
   $desc =~ s/\.$//; $desc = lc $desc;
   ($acc and $id and $desc and $org and $tax) or
       $self->throw("DB_RCS::get_Seq_pfamseq - Information missing");

   $out = Bio::Pfam::SeqPfam->new('-seq' => $seq,
				  '-start' => '1',
				  '-end' => length($seq),
				  '-id'=>$id,
				  '-acc' => $acc, 
				  '-organism' => $org,
				  '-taxonomy' => $tax,
				  '-desc' => $desc);

   return $out;
}



=head2 get_EntryA_by_id

 Title   : get_EntryA_by_id
 Usage   : $en = $db->get_EntryA_by_id('pkinase');
 Function: get a Bio::Pfam::Entry object by id
 Returns : Bio::Pfam::Entry object
 Args    : string of the id of the family


=cut

sub get_EntryA_by_id{
   my ($self,$id) = @_;
   my ($dir,$out,$acc);
   
  
   $acc = $self->id2acc($id);

   $dir = $self->{'dbweb_datadir'};
   open(_DESC,"$dir/desc/$acc.desc") || $self->throw("Could not open [$dir/desc/$acc.desc] as a web site description");
   $out = Bio::Pfam::EntryA_Web->new();
   $out->_datadir($dir);
   $out->acc($acc);
   close(DESC);
   return $out;
}



=head2 get_EntryB_by_id

 Title   : get_EntryB_by_id
 Usage   : $en = $db->get_EntryB_by_id('Pfam-B_101');
 Function: get a Bio::Pfam::Entry object by id
 Returns : Bio::Pfam::Entry object
 Args    : string of the id of the family


=cut

sub get_EntryB_by_id{
    my ($self,$id) = @_;
    my ($out, $pfambdb);
    
    $pfambdb = $self->{'dbweb_srspfamb'};
    
   my $getz = $self->{'dbweb_getz'};
    open TEST, "$getz -e '[$pfambdb-id:$id]' |";
    while (<TEST>) {
	if (/^AC\s+(\S+)/) {
	    $out = Bio::Pfam::EntryB_Impl->new();
	    $out->_datasource("$getz -e '[$pfambdb-id:$id]' |");
	    $out->id($id);
	    $out->acc($1);
	    last;
	}
    }
    close TEST;

    if (not defined $out) {
	$self->throw("There is no entry for id $id");
    }
    else {
	return $out;
    }
}




=head2 rdb_get_EntryA_by_acc

 Title   : get_EntryA_by_acc
 Usage   : $en = $db->get_EntryA_by_acc('PF00076');
 Function: get a Bio::Pfam::Entry object by acc
 Returns : Bio::Pfam::Entry object
 Args    : string of the accession number of the family


=cut

#sub _rdb_get_EntryA_by_acc{
  sub get_EntryA_by_acc{
    my ($self,$acc) = @_;
    my ($dir, $out , $dbh, $st);

    ##### PARAMS FOR pfamA table ########
    my ($auto_pfamA , $pfamA_acc , $pfamA_id , $description , $model_length , $author  , $seed_source , $alignment_method , $type   , $ls_sequence_GA , $ls_domain_GA , $fs_sequence_GA , $fs_domain_GA , $ls_sequence_TC , $ls_domain_TC , $fs_sequence_TC , $fs_domain_TC , $ls_sequence_NC , $ls_domain_NC , $fs_sequence_NC , $fs_domain_NC , $ls_mu , $ls_kappa , $fs_mu , $fs_kappa , $comment , $previous_id, $hmmbuild_ls, $hmmcalibrate_ls, $hmmbuild_fs, $hmmcalibrate_fs, $num_seed, $num_full);

    if (not $self->{'no_rdb'}) {
      
      
      eval {
	# if (not $self->{'no_rdb'}) {
	$dbh = $self->_the_RDB->connect();
	if(defined($dbh)) {
	  $st = "select * from pfamA where pfamA_acc = '$acc'";

	  my($stat) = $dbh->prepare($st);
	  $stat->execute();
	  ($auto_pfamA , $pfamA_acc , $pfamA_id , $description , $model_length , $author  , $seed_source , $alignment_method , $type   , $ls_sequence_GA , $ls_domain_GA , $fs_sequence_GA , $fs_domain_GA , $ls_sequence_TC , $ls_domain_TC , $fs_sequence_TC , $fs_domain_TC , $ls_sequence_NC , $ls_domain_NC , $fs_sequence_NC , $fs_domain_NC , $ls_mu , $ls_kappa , $fs_mu , $fs_kappa , $comment , $previous_id, $hmmbuild_ls, $hmmcalibrate_ls, $hmmbuild_fs, $hmmcalibrate_fs, $num_seed, $num_full) = $stat->fetchrow;
	  $stat->finish;
	  
	  $self->_the_RDB->disconnect;
	}
	# }
      };
    
      if ($@ or not defined( $dbh ) or $self->{'no_rdb'} or (!$auto_pfamA)) { 
	## Connection to the database failed - get pfama from the flatfile

	$out = $self->_file_get_EntryA_by_acc($acc);
	
	
      } else {

	
	$out = Bio::Pfam::EntryA_Web->new();
	$out->from_rdb(1);
	$out->acc($acc);
	
	
	$out->entry_type($type );
	$out->id($pfamA_id );
	$out->model_length($model_length);
	$out->author($author );
	$out->source($seed_source );
	$out->alignmethod($alignment_method );
	
	$out->ls_sequence_gathering_cutoff($ls_sequence_GA );
	$out->ls_domain_gathering_cutoff($ls_domain_GA );
	$out->fs_sequence_gathering_cutoff( $fs_sequence_GA );
	$out->fs_domain_gathering_cutoff($fs_domain_GA );
	
	$out->ls_sequence_trusted_cutoff($ls_sequence_TC );
	$out->ls_domain_trusted_cutoff($ls_domain_TC );
	$out->fs_sequence_trusted_cutoff($fs_sequence_TC );
	$out->fs_domain_trusted_cutoff( $fs_domain_TC);
	
	$out->ls_sequence_noise_cutoff($ls_sequence_NC );
	$out->ls_domain_noise_cutoff($ls_domain_NC );
	$out->fs_sequence_noise_cutoff($fs_sequence_NC );
	$out->fs_domain_noise_cutoff($fs_domain_NC );
	
	$out->previous_ids($previous_id );
	
	$out->auto_pfamA($auto_pfamA );
	
	$out->ls_kappa($ls_kappa );
	$out->ls_mu($ls_mu );
	$out->fs_kappa($fs_kappa );
	$out->fs_mu($fs_mu );
	
	$out->comment($comment );
	$out->description( $description);
	
	### KNOWS WHAT THE DATA DIR IS !
	my $dir = $self->{'dbweb_datadir'};
	$out->_datadir($dir);
	
	$out->num_seqs_in_full($num_full);
	$out->num_seqs_in_seed($num_seed);

	$out->add_build_line($hmmbuild_ls);
	$out->add_build_line($hmmcalibrate_ls);
	$out->add_build_line($hmmbuild_fs);
	$out->add_build_line($hmmcalibrate_fs);
	
	
	############# FILL UP THE ANNOTATION OBJECT FROM THE RDB ################
	# GET PDB, database_links & literature_refs and add then to the object!
	################################################################################
	
	my @refs;
	
	##### GET LITERATURE REFERENCES
	#### PARAMS FOR LITERATURE REFS ########
	my($lit_comment, $order_added, $medline, $title, $author , $journal);
	
	eval {
	  if (not $self->{'no_rdb'}) {
	    $dbh = $self->_the_RDB->connect();
	    if(defined($dbh)) {
	      $st = "select comment, order_added, medline, title, author , journal from pfamA_literature_references, literature_references where auto_pfamA = '$auto_pfamA' and pfamA_literature_references.auto_lit = literature_references.auto_lit";
	      
	      my($stat) = $dbh->prepare($st);
	      $stat->execute();
	      #  while ( ($st, $en, $acc, $id) = $st_pfamB_reg->fetchrow) {
	      
	      while ( ($lit_comment, $order_added, $medline, $title, $author , $journal) = $stat->fetchrow) {
		push @refs, "LIT:$lit_comment~$order_added~$medline~$title~$author~$journal";
	
	      }
	      $stat->finish;
	      
	      $self->_the_RDB->disconnect;
	    }
	  }
	};
	
	
	
	#### PARAMS FOR DATABASE LINKS ########
	my($db_id, $db_comment, $db_link, $other_params);
	
	eval {
	  if (not $self->{'no_rdb'}) {
	    $dbh = $self->_the_RDB->connect();
	    if(defined($dbh)) {
	      $st = "select db_id, comment , db_link, other_params from pfamA_database_links where auto_pfamA = '$auto_pfamA'";
	      
	      my($stat) = $dbh->prepare($st);
	      $stat->execute();
	     
	      my %db_links;
	      while ( ($db_id, $db_comment, $db_link, $other_params) = $stat->fetchrow) { #### ADD OTHER STUFF
		
		if ( ($db_comment || $other_params)  ||  ($db_id !~ /HOMSTRAD/i) && ($db_id !~ /PFAMB/i) && ($db_id !~ /COGS/i) ) {
		  push @refs, "DATA:$db_id~$db_comment~$db_link~$other_params";
		  
		} else {
		  $db_links{$db_id} = $db_links{$db_id} . " " . $db_link;

		} 


		
	      }

	      foreach my $db_id (sort keys %db_links) {

		push @refs, "DATA:" . $db_id . "~ ~" . $db_links{$db_id} ."~ ";
		
	      }

#	      my $pfam_db_link;
#	      while ( ($db_id, $db_comment, $db_link, $other_params) = $stat->fetchrow) { #### ADD OTHER STUFF
		
#		if ($db_id !~ /PFAMB/i) {
#		  push @refs, "DATA:$db_id~$db_comment~$db_link~$other_params";
#		} else {
#		  $pfam_db_link = $pfam_db_link . " " . $db_link;	
#		}
		
#	      }
	      
#	      push @refs, "DATA:PFAMB~ ~$pfam_db_link~ " if ($pfam_db_link);
	     
	      
	      $stat->finish;
	      
	      $self->_the_RDB->disconnect;
	    }
	  }
	};
	
	
	#### ADD PDBMAP !!!! ##########
	my($pdb_id, $chain, $start, $end);

	eval {
	  if (not $self->{'no_rdb'}) {
	    $dbh = $self->_the_RDB->connect();
	    if(defined($dbh)) {
	      $st = "select pdb_id, chain, pdb_start_res, pdb_end_res from pdb, pdbmap, pfamA where pfamA_acc = '$acc' and pfamA.auto_pfamA = pdbmap.auto_pfam and pfam_region = '1' and pdbmap.auto_pdb = pdb.auto_pdb";
	      
	      my($stat) = $dbh->prepare($st);
	      $stat->execute();
	     
	      while ( ($pdb_id, $chain, $start, $end) = $stat->fetchrow) { #### ADD OTHER STUFF
	
		push @refs, "DATA:PDB~~$pdb_id $chain~$start;$end";
	
	      }
	      $stat->finish;
	      
	      $self->_the_RDB->disconnect;
	    }
	  }
	};



	$out->_load_refs_from_rdb($comment, $description, @refs);

      } ### end if RDB CONNECTION FAILED
      
      
    } else {
      
      $out = $self->_file_get_EntryA_by_acc($acc);
      
      
    } # END IF SWITCHED OFF RDB CONNECTION WITH NORDB FILE
    
    return $out;
    
}



sub _file_get_EntryA_by_acc{
   my ($self,$acc) = @_;
   my ($dir,$out);
   $dir = $self->{'dbweb_datadir'};
  
   open(_DESC,"$dir/desc/$acc.desc") || &PfamWWWConfig::user_error("This is not a valid Pfam accession number", 1);

   $out = Bio::Pfam::EntryA_Web->new();
   $out->from_rdb(0);
   $out->_datadir($dir);
   $out->acc($acc);
   close(DESC);
   return $out;

}



sub get_pfamseq_ids_for_pfam_regions {
  
  my($self, $aln_type, $acc) = @_;
  
  my(@ids, $dbh, $st, $stat, %pfamseq, @store, @final_seqs);
  
  eval {
    if (not $self->{'no_rdb'}) {
      $dbh = $self->_the_RDB->connect();
      if(defined($dbh)) {
	if ($aln_type =~ /FULL/i) {
	  
	  #### UPDATE SQL - IN FOR TESTING 
	  $st = "select  pfamseq_id, tree_order from pfamA_reg_full, pfamseq, pfamA where pfamA_acc = '$acc' and pfamA.auto_pfamA = pfamA_reg_full.auto_pfamA and pfamA_reg_full.auto_pfamseq = pfamseq.auto_pfamseq and in_full = '1' ";

	} else {
	  $st = "select distinct pfamseq_id from pfamA_reg_seed, pfamseq, pfamA where pfamA_acc = '$acc' and pfamA.auto_pfamA = pfamA_reg_seed.auto_pfamA and pfamA_reg_seed.auto_pfamseq = pfamseq.auto_pfamseq ";
	}
	my($stat) = $dbh->prepare($st);
	$stat->execute();
	my ($pfamseq_id, $tree_order);
	while (($pfamseq_id, $tree_order) = $stat->fetchrow) {
	  if (defined($pfamseq{$pfamseq_id})) {
	    if ($tree_order < $pfamseq{$pfamseq_id}) {
	      $pfamseq{$pfamseq_id} = $tree_order if ($tree_order ne 0);
	    }
	  } else {
	    $pfamseq{$pfamseq_id} = $tree_order if ($tree_order ne 0);
	  }
		      
	  ###push @ids, $pfamseq_id;
	}
	      
	foreach my $pfam_id (sort keys %pfamseq) {
	  my %temp = ('id' => $pfam_id,
		      'count' => $pfamseq{$pfam_id}
		      );
	  push @store, \%temp;

	}
	      
	 @store = sort { $a->{'count'} <=> $b->{'count'} } @store;
	
	my($outer_loop) = 0;
	while ($outer_loop <= $#store ) {
	  

	  push @final_seqs, $store[$outer_loop]->{'id'};
	  $outer_loop++;
	}

 
      
	      
	$stat->finish;
	      
	      


	$self->_the_RDB->disconnect;
      }
    }
  };

  return @final_seqs;
}



=head2 get_EntryB_by_acc

Title   : get_EntryB_by_acc
 Usage   : $en = $db->get_EntryB_by_acc('PB000076');
 Function: get a Bio::Pfam::Entry object by acc
 Returns : Bio::Pfam::Entry object
 Args    : string of the accession number of the family


=cut

sub get_EntryB_by_acc{
   my ($self,$acc) = @_;
   my ($out, $pfambdb);

   $pfambdb = $self->{'dbweb_srspfamb'};

   my $getz = $self->{'dbweb_getz'};
   open (TEST, "$getz -e '[$pfambdb-acc:$acc]' |") 
       or $self->throw("Could not open the getz test command for $acc");

   while (<TEST>) {
       if (/^ID\s+(\S+)/) {
	    $out = Bio::Pfam::EntryB_Impl->new();
	    $out->_datasource("$getz -e '[$pfambdb-acc:$acc]' |");
	    $out->acc($acc);
	    $out->id($1);
	    last;
       }
   }
   close TEST;

   if (not defined $out) {
     &PfamWWWConfig::user_error("This is not a valid Pfam accession number", 1);
       $self->throw("There is no entry for acc $acc");
   }
   else {
       return $out;
   }
}


######################################################################################################
#  MODEL LENGTH
######################################################################################################


=head2 _model_length

 Title   : _model_length
 Usage   : $id = $db->acc2id("PF00001");
 Function: converts an accession to an id
 Returns : the id, throws an exception if no accession, or dead accession
 Args    :

=cut

sub _model_length {
   my ($self,$acc) = @_;
   my ($dbh, $st, $len, $id);
   eval {
       if (not $self->{'no_rdb'}) {
           $dbh = $self->_the_RDB->connect();

           if(defined($dbh)) {
               $dbh = $self->_the_RDB->connect();
               $st = $dbh->prepare("select pfamA_id, model_length from pfamA where pfamA_acc = '$acc'");
               $st->execute;
               ($id, $len) = $st->fetchrow;
               $st->finish;
               $self->_the_RDB->disconnect;
           }
       }
   };

   if($self->{'no_rdb'} or $@) {
  
   }
   else {
       if (not defined $id) {
           $self->throw("DB_RDB::model_length - $acc is not a valid id for this pfam database");
       }
       elsif (not defined $len) {
           $self->throw("DB_RDB::model_length - $acc is the id of a dead family");
       }

   }

   return $len;
}

######################################################################################################
## FAMILY TYPE
######################################################################################################

sub get_family_type {

  my($self, $acc) = @_;

  my($dbh, $st, $type);

  eval {
       if (not $self->{'no_rdb'}) {
           $dbh = $self->_the_RDB->connect();

           if(defined($dbh)) {
               $dbh = $self->_the_RDB->connect();
               $st = $dbh->prepare("select type from pfamA where pfamA_acc = '$acc'");
               $st->execute;
               ($type) = $st->fetchrow;
               $st->finish;
               $self->_the_RDB->disconnect;
           }
       }
   };

  if($self->{'no_rdb'} or $@) {
	$type = "family";
    # $self->throw("Error getting type from RDB \n");
   }
  $type = lc($type);
     return $type;
}




######################################################################################################
# COMPLEXES
######################################################################################################



sub get_complexes{




  my ($self,$acc, $complex_exists) = @_;
  my ($dbh, $st, $id,  $desc, $len, $org, $out, $seq);

  my %arr;
  my $complex = 0;
  eval {
    if (not $self->{'no_rdb'}) {
      $dbh = $self->_the_RDB->connect();
      if(defined($dbh)) {
	
	my $pfamA = 0;

	if ($acc =~ /^PF/i) {
	  
	   ### UPDATE FOR PFAMA & B
	  $st = "select distinct auto_pdb from pdbmap,  pfamA where pfamA_acc = '$acc' and pfamA.auto_pfamA = pdbmap.auto_pfam and pfam_region = '1' and complex = '1'";

	  $pfamA = 1;
	} else {
	  $st = "select distinct auto_pdb from pdbmap,  pfamB where pfamB_acc = '$acc' and pfamB.auto_pfamB = pdbmap.auto_pfam and pfam_region = '0' and complex = '1'";

	}

        my($stat) = $dbh->prepare($st);
        $stat->execute();
        my( $auto_pdb, $pdb, $domain, $pfamseq);

	
	my @the_pfam_type = ("pfamA", "pfamB");
        while ( ($auto_pdb  ) = $stat->fetchrow) {
	if ($complex_exists) {
	   $complex = 1;
	   return;
	 }
	  my @all;

	  
	  foreach my $the_pfam_type (@the_pfam_type) {
	    
	    $complex = 1;
	    
	    ### UPDATE FOR PFAMA & B
	    my $sql;
	    my $id = "pfamB_id";
	    my $the_auto = "auto_pfamB";
	    my $the_pfamregion = 0;
	    if ($the_pfam_type =~ /pfamA/i) {
	      $id = "pfamA_id";
	      $the_auto = "auto_pfamA";
	      $the_pfamregion = 1;
	    }

	
	      $sql =  "select pdb_id, chain,pdb_start_res, pdb_end_res, " .$id . " , pfam_region, pfamseq_id, pfam_start_res , pfam_end_res , complex  from pdb, pdbmap, $the_pfam_type, pfamseq where pdb.auto_pdb = '$auto_pdb' and pdb.auto_pdb = pdbmap.auto_pdb and pfam_region = " . $the_pfamregion. "  and pdbmap.auto_pfam = $the_pfam_type." . $the_auto . " and pdbmap.auto_pfamseq = pfamseq.auto_pfamseq order by " . $id ." ";
	      

	    
	    my($int_stat) = $dbh->prepare($sql);
	    $int_stat->execute();
	    my ($int_pdb, $int_chain, $pdb_start_atom, $pdb_end_atom,  $int_domain, $int_pfam_type,  $int_pfamseq, $int_pfam_start, $int_pfam_end,  $int_complex);
	    
	    
	    while ( ($int_pdb, $int_chain, $pdb_start_atom, $pdb_end_atom,  $int_domain, $int_pfam_type,  $int_pfamseq, $int_pfam_start, $int_pfam_end,  $int_complex  ) = $int_stat->fetchrow) {
	      
	      
	      $pdb = $int_pdb;
	      push @all, "$int_pdb~$int_chain~$pdb_start_atom~$pdb_end_atom~$int_domain~$int_pfamseq~$int_pfam_start~$int_pfam_end~$int_complex";
	    }

	    
          }			#/ end WHILE
	  


	  my @refs;
	  
	  my %store_ids;

	  foreach (@all) {

	    my($int_pdb, $int_chain, $pdb_start_atom, $pdb_end_atom,  $int_domain,  $int_pfamseq, $int_pfam_start, $int_pfam_end,  $int_complex) = split(/~/, $_);
	   
	    my $id = "$int_domain~$int_pfamseq";
	    
	    $store_ids{$id} = $store_ids{$id} . "*" . "$int_pfam_start~$int_pfam_end~$int_chain"; 
	    push @refs, $id;
	  }
	  
	  my ($temp_name, $temp_value);
          $temp_value = $pdb. ";";
	  
	  
	  my $first = 1;
	  
	  my %check_ids;
	  my @new_refs;
	  foreach (@refs) {
	    if(defined($check_ids{$_})) {

	    } else {
	      push @new_refs, $_;
	      $check_ids{$_} = $_;
	    }
	  }
	  
	  foreach my $key (@new_refs) {
	    my $min = "999999999999999";
	    my $max = 0;
	    my(@vals) = split(/\*/, $store_ids{$key} );
	    my %chains;
	    foreach my $val  (@vals) {
	      if ($val =~ /[A-Z]|[0-9]/i) {
		my($pfam_start, $pfam_end, $chain) = split(/~/, $val);
		$min = $pfam_start if ($pfam_start < $min);
		$max = $pfam_end if ($pfam_end > $max);
		$chains{$chain} = $chain;
	      }
	    }
	    
	    my ($the_domain, $the_pfamseq) = split(/~/, $key);
	    
	    if ($first) {
	      $temp_name = $the_domain;
	      $first = 0;
	    } else {
	      
	      
	      $temp_name = $temp_name . "~" . $the_domain;
	      
	    }
	    
	    $temp_value = $temp_value . $the_domain . "~" . $min . "~" . $max . "~" . $the_pfamseq . ":";
	    
	  }
	  



          if(defined($arr{$temp_name})) {
		my(@existing) = split(/\*/, $arr{$temp_name});
		my $not_equals = 1;
	
		foreach (@existing) {
	          $not_equals = 0 if ($temp_value eq $_);				
		}

	    $arr{$temp_name} = $arr{$temp_name} . "*" . $temp_value if($not_equals);
          } else {
            $arr{$temp_name} = $temp_value;
          }


        }

      }


    }


	
  };
  if ($complex_exists) {
    $complex
  } else {
    return %arr;
    
 }
}




######################################################################################################
# CRC 64
######################################################################################################


sub _crc64_to_protein {

  my($self, $crc64, $pfam_type) = @_;
  my($dbh, $st);
  my($pfam_id, %all_pfam_ids,  $seq_start, $seq_end, $pfamseq_id, $pfamB_acc, $pfamB_id);


	if (!$pfam_type) {
  eval {
    if (not $self->{'no_rdb'}) {
      $dbh = $self->_the_RDB->connect();
      if(defined($dbh)) {

     
	$st = "select distinct pfamA_id from pfamA_reg_full, pfamA, pfamseq  where crc64 = '$crc64' and pfamseq.auto_pfamseq = pfamA_reg_full.auto_pfamseq and pfamA_reg_full.auto_pfamA = pfamA.auto_pfamA ";

        my($stat) = $dbh->prepare($st);
        $stat->execute();
        
	my($pfam_id);

        while ( ($pfam_id) = $stat->fetchrow) {

	$all_pfam_ids{$pfam_id} = $pfam_id;
	}
      }


    }

  };
	} else {
	   eval {
      if (not $self->{'no_rdb'}) {
	$dbh = $self->_the_RDB->connect();
	if(defined($dbh)) {
	  
	  
	  $st = "select  pfamseq_id, seq_start, seq_end, pfamB_acc, pfamB_id from pfamB_reg, pfamseq, pfamB  where crc64 = '$crc64' and pfamseq.auto_pfamseq = pfamB_reg.auto_pfamseq and pfamB_reg.auto_pfamB = pfamB.auto_pfamB";

	  my($stat) = $dbh->prepare($st);
	  $stat->execute();
	  
	  my($pfam_id);
	  
	  while ( ($pfamseq_id, $seq_start, $seq_end, $pfamB_acc, $pfamB_id) = $stat->fetchrow) {
	 
	    my $fasta_search = $pfamseq_id . "~" . $seq_start . "~" . $seq_end . "~" .  $pfamB_acc . "~" . $pfamB_id;

	    $all_pfam_ids{$seq_start . "~" . $seq_end} = $fasta_search;
	  }
	}
	
	
      }
      
    };


	}

  return %all_pfam_ids;
  



}


######################################################################################################
# INTERPRO & QuickGO information
######################################################################################################


sub get_interpro_information {

  my($self, $pfamA_acc) = @_;
   my($dbh, $st, $interpro_id, $interpro_abstract, $process, $component, $function, %interpro_info);
 
  eval {

      $dbh = $self->_the_RDB->connect();
      if ( (defined($dbh))  or (not $self->{'no_rdb'})   ) {


	$st = "select interpro_and_go.interpro_id, interpro_abstract, go_function , go_component, go_process, pfamA.auto_pfamA from pfamA, interpro_and_go, pfam_to_interpro where pfamA_acc = '$pfamA_acc' and pfamA.auto_pfamA = pfam_to_interpro.auto_pfamA and pfam_to_interpro.interpro_id = interpro_and_go.interpro_id ";


        my($stat) = $dbh->prepare($st);

        $stat->execute();
        
	my($pfam_id);
	
        my ($interpro_id, $interpro_abstract, $go_function , $go_component, $go_process, $auto_pfamA) = $stat->fetchrow;

	
	if (!$auto_pfamA) {

	} else {
	  $interpro_info{'INTERPRO'} = $interpro_id ;
	  $interpro_info{'ABSTRACT'} = $interpro_abstract ;
	  $interpro_info{'FUNCTION'} = $go_function ;
	  $interpro_info{'PROCESS'} = $go_process ;
	  $interpro_info{'COMPONENT'} = $go_component ;
	  
	}

      } else {
	
	%interpro_info = $self->_file_get_interpro_information($pfamA_acc);
	
      }
      

      
      
  };
  

  return %interpro_info;
  

}


#### CANT GET INTERPRO FROM RDB - USE FILE INSTEAD !

sub _file_get_interpro_information {
  
  my($self, $acc) = @_;
  
  my($interpro_id, $interpro_abstract, $function, $process, $component, %interpro_info);
  my $interpro_file = $PfamWWWConfig::data_root . "/interpro/" . $acc . ".interpro";
 
  open(_INTERPRO, $interpro_file) or die "canna open $interpro_file $! <P>";;
  while(<_INTERPRO>) {

    if ($_ =~ /^ABSTRACT\s+(.*)/) {
      $interpro_abstract .= $1;
    } elsif ($_ =~  /^INTERPRO\s+(\S+)/){
      $interpro_id = $1;
     
    } elsif ($_ =~  /^PROCESS\s+(.*)/) {
      $process = $1;
    } elsif ($_ =~  /^COMPONENT\s+(.*)/){
      $component = $1;
    } elsif ($_ =~  /^FUNCTION\s+(.*)/) {
      $function = $1;
    } else {
      $interpro_abstract .= $_;
    }

  }
  $interpro_info{'INTERPRO'} = $interpro_id ;
  $interpro_info{'ABSTRACT'} = $interpro_abstract ;
  $interpro_info{'FUNCTION'} = $function ;
  $interpro_info{'PROCESS'} = $process ;
  $interpro_info{'COMPONENT'} = $component ;
  
close(_INTERPRO);

  return %interpro_info;

}

######################################################################################################
# PDB DETAILS 
######################################################################################################


=head2 get_pdb_details

 Title   : get_pdb_details
 Usage   : 
 Function:
 Returns : 
 Args    : 


=cut

sub get_pdb_details {

  my($self, $pdb_id) = @_;
  my($dbh, $st);
  my(  $header, $title);
  eval {
    if (not $self->{'no_rdb'}) {
      $dbh = $self->_the_RDB->connect();
      if(defined($dbh)) {

        $st = "select header, title from pdb where pdb_id = '$pdb_id' ";
        my($stat) = $dbh->prepare($st);
        $stat->execute();
        
	my($tmp_header, $tmp_title);

        while ( ($tmp_header, $tmp_title) = $stat->fetchrow) {
	 $header = $tmp_header;
	 $title = $tmp_title;
	}
      }


    }

  };

  return $header , $title;
  


}


sub species_distribution_for_family {

  my ($self, $family_id) = @_;

  my($dbh, $st);

  my (@list);

  eval {
    if (not $self->{'no_rdb'}) {
      $dbh = $self->_the_RDB->connect();
      if(defined($dbh)) {

    $st = "select distinct pfamseq_id, species, taxonomy from pfamA, pfamA_reg_full, pfamseq where pfamA_id = '$family_id' and pfamA.auto_pfamA = pfamA_reg_full.auto_pfamA and in_full = '1' and pfamA_reg_full.auto_pfamseq = pfamseq.auto_pfamseq";
        my($stat) = $dbh->prepare($st);
        $stat->execute();
        
	my ($tmp_seq, $tmp_species, $tmp_taxonomy);

        while ( ( $tmp_seq, $tmp_species, $tmp_taxonomy) = $stat->fetchrow) {
	  push @list, "$tmp_seq~$tmp_species~$tmp_taxonomy";

	}
      }


    } else {
      return 0;
    }

  };

 
  return @list;






}


######################################################################################################
############################ Internal methods
######################################################################################################


=head2 _the_RDB

 Title   : _the_RDB
 Usage   : $rdb = $self->_the_RDB();
 Function:
    Gets/sets the underlying database object for this query layer
 Returns : A object that provides basic connection facilities to the RDB
 Args    : A object that provides basic connection facilities to the RDB (optional)

=cut

sub _the_RDB {
   my ($self,$value) = @_;

   if (defined $value) {
       $self->{'_the_rdb'} = $value;
   }
   return $self->{'_the_rdb'};
}

