
#
# BioPerl module for Bio::Pfam::OtherRegion
#
# Cared for by Mhairi Marshall <mm1@sanger.ac.uk>
#
# Copyright Kevin Howe & Mhairi Marshall
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::OtherRegion - Object for the annotation of a specified region of a sequence

=head1 SYNOPSIS

This is a concrete implementation of AnnotatedRegion. It is for those
sequence regions that Pfam knows something about, but are not Pfam
domains. At time of writing, the allowed types of region are:

1. low_complexity
2. coiled_coil
3  sec_struct
4. trans_membrane
5. surface_accessibility
6. ligand_binding
7. posterior_prob
8. Active site

    use Bio::Pfam::OtherRegion;

    $reg = Bio::Pfam::OtherRegion->new('-SEQ_ID' => $name,
				       '-FROM' => $from,
				       '-TO' => $to,
				       '-TYPE' => 'low_complexity',
				       '-DISPLAY' => $string,
				       '-SOURCE' => 'seg (version 1)',
				       '-ORIENTATION' => 'I-O',
				       '-ANNOTATION' => Bio:Annotation->new(
					   '-DESCRIPTION' => "Any further information"))
                                      );


=head1 DESCRIPTION

This is a concrete implementation of AnnotatedRegion. It is for those
sequence regions that Pfam knows something about, but are not Pfam
domains. At time of writing, the allowed types of region are:

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::OtherRegion;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;
# Object preamble - inherits from Bio::Root::Object

use Bio::Pfam::AnnotatedRegion;


@ISA = qw(Bio::Pfam::AnnotatedRegion);
#@EXPORT_OK = qw();
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params) = @_;
  my( $type, $source, $display, $score, $orientation) = 
      (
       ($params{'-TYPE'}||$params{'-type'}), 
       ($params{'-SOURCE'}||$params{'-source'}),
       ($params{'-DISPLAY'}||$params{'-display'}),
       ($params{'-SCORE'}||$params{'-score'}),
       ($params{'-ORIENTATION'}||$params{'-orientation'}),	
       );

  my $make = $self->SUPER::_initialize( %params );

  $self->type( $type );
  $self->source( $source );
  $self->display( $display );
  $self->score($score);
  $self->orientation($orientation);

  return $make; # success - we hope!
}




=head2 display

 Title   : display
 Usage   :
    $reg->display("SSSSS----SSS");
 Function:
    This function allows you to get/set a textual represenation of the region.\
    The length of the string shouold be consistent with the start and end of
    the region (start=5, end=10 => stringlength =6
 Returns :
    A textual representation of the region
 Args    :
    A textual representation of the region (optional)

=cut

sub display{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'OtherReg_display'} = $value;
   }
   return $self->{'OtherReg_display'};
}





=head2 type

 Title   : type
 Usage   : 
    $annReg->type(); # or ...
    $annReg->type( 123 );
 Function: For setting and getting the 'type' field in the object

=cut

sub type{
   my ($self, $value) = @_;


   if (defined $value) {
       $self->{'OtherReg_type'} = $value;
   }
   return $self->{'OtherReg_type'};
}



sub source{
   my ($self, $value) = @_;


   if (defined $value) {
       $self->{'OtherReg_source'} = $value;
   }
   return $self->{'OtherReg_source'};
}




sub score {

   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'OtherReg_score'} = $value;
   }
   return $self->{'OtherReg_score'};

}



sub orientation {
    my ($self, $value) = @_;

   if (defined $value) {
       $self->{'OtherReg_orientation'} = $value;
   }
   return $self->{'OtherReg_orientation'};
}    
