
#
# BioPerl module for Bio::ReSeed
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::ReSeed - Converts one multiple alignment to another database

=head1 SYNOPSIS

  $re = Bio::ReSeed->new( -olddb => 'pfamseq', -newdb => 'pfamseq4' );

  # or

  $re = Bio::ReSeed->new ( -blast => 'pfamseq', -blastindex => $pfamseq_bioperl_index);
  # $old is a Bio::SimpleAlign object

  $new = $re->convert($old);


=head1 DESCRIPTION

This object factory converts Bio::SimpleAlign objects into new SimpleAlign objects
which have a new underlying database

It is quite specific to Pfam. In particular, it expects the old and new databases
to be pfamseq style databases, where you want to follow the sequences by id, but
have to use the accession number to follow the database. This requires the databases
to be SRS indexed (it makes calls to getz).

However, if you want to use it on a different database mapping system, you can always
subclass it and replace the functions _get_old_seq and _convert_seq which provide
this logic.

Finally a note about the conversion process. If the old and new sequences are identical,
then it maps them without changing the sequence. If the old subsequence can be found 
identically inside the new sequence, that subsequence is used. Otherwise an HMM alignment
step is used to make the new sequence.

=head1 CONTACT

pfam@sanger.ac.uk

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::ReSeed;
use vars qw($AUTOLOAD @ISA);
use strict;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Root::Object;
use Bio::Seq;
use Bio::Tools::pSW;
use Bio::Tools::Blast;


@ISA = qw(Bio::Root::Object Exporter);

# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self,@args) = @_;
  my($olddb,$newdb,$blast,$blastindex) = $self->_rearrange([qw(OLDDB NEWDB BLAST BLASTINDEX)],@args);
  my $make = $self->SUPER::_initialize(@args);

  $self->_olddb($olddb);
  $self->_newdb($newdb);

  if( defined $blast ) {
      if( ! defined $blastindex ) {
	  $self->throw("If you are using blast, you must have both blast and blastindex provided");
      }
      $self->use_blast(1);
      $self->_blast_db($blast);
      $self->_bpindex_blast_db($blastindex);
  }
  else {
      $self->use_blast(0);
  }

  $self->{'newseqcache'} = {};
  $self->{'oldseqcache'} = {};
  $self->{'idhash'} = {};

  # set stuff in self from @args
  return $make; # success - we hope!
}

=head2 convert

 Title   : convert
 Usage   : $new = $reseed->convert($old,\*ERROR_LOG,\*LOG);
 Function: Actually does the conversion of the old to the new
           database
 Returns : New Bio::SimpleAlign object
 Args    : old Bio::SimpleAlign object, and then two optional filehandles
           The first is only written to for subs/indel errors: the second
           records everything which is done

=cut

sub convert{
   my ($self,$old,$error,$log) = @_;
   my ($new,$seq,$newseq,$oldseq,$strippedseq,@toalign,$start,$end,$newsubseq,$psw,$name,$oldid,$newid,$oldstart,$oldend, @newseqs, @goodnewseqs);


   #
   # flush sequence caches
   #

   $self->_flush_newseqcache();
   $self->_flush_oldseqcache();

   $new = Bio::SimpleAlign->new();

   # in case we need some alignments, we need the pSW alignment
   # factory

   $psw = Bio::Tools::pSW->new(); # use defaults. Should be fine!

   SEQ: foreach $seq ( $old->eachSeq ) {
     # so we can report easily

     $name = $seq->get_nse();

     # get old and new underlying sequences

     my $tmp;
     if( $self->use_blast != 1 ) {
	 $oldseq = $self->_get_old_seq($seq);
	 
	 if (! defined $oldseq || length($oldseq->seq()) <=0) {
	     if ( defined $error ) {
		 print $error "$name could not be retrieved from old database\n";
		 print $log "$name could not be retrieved from old database\n";  
		 next SEQ;
	     } 
	     else {
		 $self->throw("Could not convert $name");
	     }	 
	 }
	 $tmp = $self->_convert_seq($seq, $error, $log);
     } else {
	 $tmp = $self->_blast_seq_to_new($seq, $error, $log);
     }

     @newseqs = ($tmp)?@{$tmp}:();
    
     if (not @newseqs) {
	 if ( defined $error ) {
	     print $error "$name could not be retrieved from new database\n";
	     print $log "$name could not be retrieved from new database\n";
	     next SEQ;
	 } 
	 else {
	     $self->throw("Could not convert $name");
	 }
     }
     
     @goodnewseqs = ();
     foreach $newseq (@newseqs) {
	 if( length($newseq->seq()) > 0 ) {
	     push @goodnewseqs, $newseq;
	 }
     }

     if (not @goodnewseqs) {
	 if ( defined $error ) {
	     print $error "$name could not be retrieved from new database\n";
	     print $log "$name could not be retrieved from new database\n";
	     next;
	 } 
	 else {
	     $self->throw("Could not convert $name");
	 }
     }
	 
     $oldstart = $seq->start();
     $oldend   = $seq->end();
     $oldid = $oldseq->id();

     $strippedseq = $seq->seq();
	     
     $strippedseq =~ s/[^A-Za-z]//g;
     $strippedseq =~ tr/[a-z]/[A-Z]/;

     if( $self->use_blast != 1 ) {
	 foreach $newseq (@goodnewseqs) {
	     
	     # get fields out for convience
	     # so we can report stuff to log files etc
	     
	     $newid = $newseq->id();
	     
	     # print STDERR $oldseq->out_fasta();
	     # print STDERR $newseq->out_fasta();
	     
	     # if they are the same - yippee!
	     # oldseq might be undefined if we are in blast mode
	     if( defined $oldseq && $oldseq->seq() eq $newseq->seq() ) {
		 
		 # id's might be different
		 
		 $seq->id($newseq->id());
		 
		 # add the old sequence in
		 $new->addSeq($seq);
		 
		 $log && print $log "$name maps without any changes $oldid:$newid\n";
		 next SEQ;
	     }
	     
	     # check to see if the subsequence is unique somewhere
	     # we need to take the old subsequence, remove the gaps and
	     # see if it matches to anything in the new subsequence
	     
	     
	     if( $newseq->seq() =~ /^([A-Z]*)$strippedseq([A-Z]*)$/ ) {
		 
		 # we have an identical subsequence
		 
		 my $pre = $1;
		 my $post = $2;
		 $start = length($pre)+1;
		 $end = length($strippedseq)-1+$start;
		 
		 $newsubseq = Bio::Seq->new( -seq=> scalar($seq->seq()), 
					     -id => $newseq->id(), 
					     -start => $start, 
					     -end => $end , 
					     -type => 'aligned');
		 
		 # add it to the alignment
		 
		 $new->addSeq($newsubseq);
		 $log && print $log "$name sequence has changed but subsequence ok $oldid:$newid, old[$oldstart:$oldend] vs new[$start:$end]\n";
		 
		 next SEQ; # back to main loop
	     }
	 }
     }
	 
     # yuk! We have a subs/indel problem.
     # we need to chop out the sequence as best we can using bp_sw
     # to get start/end points. Then we place it on the @toalign
     # array. At the end of processing all the sequences, we realign
     # these by making a temporary hmm...

     my $tempseq = Bio::Seq->new ( -seq => $strippedseq, -id => 'tempseq');

     # At this point, we have decided that the sequence has changed, so we 
     # are going to do a smith-waterman alignment of the subsequence in the 
     # old seed to the new sequence. If we are dealing with more than one
     # possible new sequence, we need to try the alignment with all of
     # them, and pick the one with the best score.

     my $bestscore;
     foreach $newseq (@goodnewseqs) {

	 my ($tempaln, $score) = $psw->pairwise_alignment($tempseq,$newseq);

	 if (defined $bestscore and $bestscore > $score) {
	     # do nothing
	 }
	 else {
	     my @tarray = $tempaln->eachSeq();
	     
	     $tempseq = $tarray[1];
	     
	     $newsubseq = Bio::Seq->new ( -seq => scalar($newseq->seq($tempseq->start,$tempseq->end+1)),
					  -id => $newseq->id(), 
					  -start => $tempseq->start, 
					  -end => $tempseq->end+1,
					  );

	     $bestscore = $score;
	 }
     }

     # push it on to the array

     push(@toalign,$newsubseq);
     $log && print $log "$name has changes\n";
     $error && print $error "$name has changes\n";
     if (@goodnewseqs > 1) {
	 $log && print $log "$name was replaced by aligning ".$newsubseq->id()."/".$newsubseq->start."-".$newsubseq->end." from a selection\n";
	 $error && print $error "$name was replaced by aligning ".$newsubseq->id()."/".$newsubseq->start."-".$newsubseq->end." from a selection\n";
     }
   } # end of loop over all sequences.


   # now we need to align the remaining sequences

   if( @toalign >= 1 ) { 
     # we have some sequences to align

     # build a temporary alignment file

     open(_TEMP_,">reseed.temp.$$") || $self->throw("Could not open temp file $!");
     $new->write_Pfam(\*_TEMP_);
     close(_TEMP_);
     # build an HMM

     open(_T_,"hmmbuild reseed.HMM.$$ reseed.temp.$$ |");
     while(<_T_>) {
       ;
     }

     # write out all the other sequences to be aligned

     open(_TEMP_,">reseed.unaln.$$") || $self->throw("Could not open temporary unaligned file $!");
     foreach my $tseq ( @toalign ) {
       my $name = $tseq->get_nse();
       print _TEMP_ ">$name\n";
       print _TEMP_ $tseq->seq() , "\n";
     }

     close(_TEMP_);

     # make the alignment

     open(_TEMP2_,"hmmalign --mapali reseed.temp.$$ reseed.HMM.$$ reseed.unaln.$$ |");

     # read it in

     my $newaln = Bio::SimpleAlign->new();
     while(<_TEMP2_>) {
       /^#/ && last;
     }

     
     $newaln->read_selex(\*_TEMP2_);

     close(_TEMP2_) || $self->throw("Could not close hmmalign ");

     # remove temporary files

     unlink("reseed.temp.$$");
     unlink("reseed.HMM.$$");
     unlink("ressed.unaln.$$");

     # reset new to newaln

     $new = $newaln;
     $new->map_chars('\-','.');
   }

   return $new;
  
}

=head2 _get_old_seq

 Title   : _get_old_seq
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _get_old_seq{
  my ($self,$subseq) = @_;
  my ($new,$olddb,$id);

  $olddb = $self->_olddb();
  $id = $subseq->id();

  $new = $self->_is_oldseqcache($id);

  if( !defined $new ) {
      open _TEMP_, "getz -sf fasta -d '[$olddb-id:$id]' |";
      my ($temp_id, $temp_seq);
      while (<_TEMP_>) {
	  if (/^>(\S+)/) {
	      $temp_id = $1;
	  }
	  elsif (/^(\S+)/) {
	      $temp_seq .= $1;
	  }
      }
      if ($temp_id) {	      
	  $new = Bio::Seq->new('-id' => $temp_id, '-seq' =>$temp_seq );
      }
  } else {
    return $new;
  }

  $self->_add_to_oldseqcache($id,$new);
  return $new;
  
}


=head2 _convert_seq

 Title   : _convert_seq
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _convert_seq{
  my ($self,$subseq) = @_;
  my ($new,$olddb,$newdb,$id,$acc);

  $olddb = $self->_olddb();
  $newdb = $self->_newdb();

  $id = $subseq->id();
  $acc = $self->{'idhash'}->{$id};


  if( !defined $acc) {
    open(_TEMP_,"getz -f acc '[$olddb-id:$id]' |");
    while(<_TEMP_>) {
      /^AC\s+(\S+);/ && do { $acc = $1; last; };
    }
    

    if( !defined $acc ) {
      $self->throw("Unable to find $acc for $id");
    }
    $self->{'idhash'}->{$id} = $acc;

    close(_TEMP_) || $self->throw("Unable to close filehandle for reading _TEMP_");
  }


  $new = $self->_is_newseqcache($acc);


  if( ! defined $new ) {
      my ($temp_id, $temp_seq);
      $new = [];
      open _TEMP_, "getz -sf fasta -d '[$newdb-acc:$acc]' |";
      while (<_TEMP_>) {
	  if (/^>(\S+)/) {
	      if ($temp_id) {
		  # this accession returned more than one sequence.
		  push @$new, Bio::Seq->new('-seq' => $temp_seq, '-id' => $temp_id);
		  $temp_seq = "";
	      }
	      $temp_id = $1;      
	  }
	  elsif (/^(\S+)/) {
	      $temp_seq .= $1;
	  }
      }
      if ($temp_id) {
	  push @$new, Bio::Seq->new('-seq' => $temp_seq, '-id' => $temp_id);
      }
  } 
  else {
      # print STDERR "Using cache...\n";
      return $new;
  }


  $self->_add_to_newseqcache($acc,$new);

  return $new;
}

=head2 _blast_seq_to_new

 Title   : _blast_seq_to_new
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _blast_seq_to_new {
   my ($self,$seq,$error,$log) = @_;
   
   if( $self->use_blast != 1 ) {
       $self->warn("Using blast_seq_to_new when use_blast is not 1. Weird!");
   }
   if( ! defined $self->_blast_db ) {
       $self->throw("No blast database! (_blast_db) Bombing out!");
   }
   if( ! defined $self->_bpindex_blast_db ) {
       $self->throw("You have not provided a bpindexed blast db (_bpindex_blast_db) for the blast");
   }

   if( ! $seq->isa('Bio::Seq') ) {
       $self->throw(" $seq is not a Bio::Seq, but something else. Yuk!");
   }
   my $actualseq = $seq->seq();
   $actualseq = uc($actualseq);
   $actualseq =~ s/[^A-Z]//g;
   my $tempseq = Bio::Seq->new( -seq => $actualseq , -id => $seq->id() );

   open(TEMP,">reseed.$$.tmp") || $self->throw("Could not open reseed.$$.tmp. $!");
   print TEMP $tempseq->out_fasta;
   close(TEMP);

   my $blastdb = $self->_blast_db();


   system("wublastp $blastdb reseed.$$.tmp  > reseed.$$.bl");

   my $blast;

   eval {
       $blast = Bio::Tools::Blast->new ( -file => "reseed.$$.bl",
					 -parse => 1,
					 -signif => '1e-3',
					 -strict => 1,
					 );
   };
   if( $@ ) {
       print ERROR "Unable to parse Blast report. Possibly no hits found! Possibly an error.\n$@\n";
       unlink("reseed.$$.tmp");
       unlink("reseed.$$.bl");
   }

   my @hits;
   @hits = $blast->hits;
   my $hit  = shift @hits;

   $seq = $self->_bpindex_blast_db($hit->id());

   # we need to return a reference to hash to keep things happy.
   my $ret = ();
   push(@{$ret},$seq);
   return $ret;
}

=head2 use_blast

 Title   : use_blast
 Usage   : $obj->use_blast($newval)
 Function: 
 Returns : value of use_blast
 Args    : newvalue (optional)


=cut

sub use_blast{
   my $obj = shift;
   if( @_ ) {
      my $value = shift;
      $obj->{'use_blast'} = $value;
    }
    return $obj->{'use_blast'};

}


=head2 _blast_db

 Title   : _blast_db
 Usage   : $obj->_blast_db($newval)
 Function: 
 Returns : value of _blast_db
 Args    : newvalue (optional)


=cut

sub _blast_db{
   my $obj = shift;
   if( @_ ) {
      my $value = shift;
      $obj->{'_blast_db'} = $value;
    }
    return $obj->{'_blast_db'};

}

=head2 _bpindex_blast_db

 Title   : _bpindex_blast_db
 Usage   : $obj->_bpindex_blast_db($newval)
 Function: 
 Returns : value of _bpindex_blast_db
 Args    : newvalue (optional)


=cut

sub _bpindex_blast_db{
   my $obj = shift;
   if( @_ ) {
      my $value = shift;
      if( ! $value->isa('Bio::DB::BioSeqI') ) {
	  $obj->warn("bioperl index must be a Bio::DB::BioSeqI compliant module. $value is not. Continuing anyway");
      }

      $obj->{'_bpindex_blastdb'} = $value;
    }
    return $obj->{'_bpindex_blastdb'};

}


=head2 _olddb

 Title   : _olddb
 Usage   : $obj->_olddb($newval)
 Function: 
 Example : 
 Returns : value of _olddb
 Args    : newvalue (optional)


=cut

sub _olddb{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'_olddb'} = $value;
    }
    return $self->{'_olddb'};

}

=head2 _newdb

 Title   : _newdb
 Usage   : $self->_newdb($newval)
 Function: 
 Example : 
 Returns : value of _newdb
 Args    : newvalue (optional)


=cut

sub _newdb{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'_newdb'} = $value;
    }
    return $self->{'_newdb'};

}

=head2 _add_to_newseqcache

 Title   : _add_to_newseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _add_to_newseqcache{
   my ($self,$hashval,$seq) = @_;
   
   $self->{'newseqcache'}->{$hashval} = $seq;

}

=head2 _flush_newseqcache

 Title   : _flush_newseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _flush_newseqcache{
   my ($self) = @_;

   $self->{'newseqcache'} = {};
}

=head2 _is_newseqcache

 Title   : _is_newseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _is_newseqcache{
   my ($self,$hashval) = @_;

   return $self->{'newseqcache'}->{$hashval};

}


=head2 _add_to_oldseqcache

 Title   : _add_to_oldseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _add_to_oldseqcache{
   my ($self,$hashval,$seq) = @_;
   
   $self->{'oldseqcache'}->{$hashval} = $seq;

}

=head2 _flush_oldseqcache

 Title   : _flush_oldseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _flush_oldseqcache{
   my ($self) = @_;

   $self->{'oldseqcache'} = {};
}

=head2 _is_oldseqcache

 Title   : _is_oldseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _is_oldseqcache{
   my ($self,$hashval) = @_;

   return $self->{'oldseqcache'}->{$hashval};

}
