
#
# Perl module for UpdateRDB
#
# Cared for by Mhairi Marshall <mm1@sanger.ac.uk>
#
# Copyright Mhairi Marshall & Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

UpdateRDB

=head1 DESCRIPTION

This module contains the basic functionality for updating the Pfam 
relational database. It relies upon the Bio::Pfam::PfamRDB module
to provide basic connection facilities


=cut


# Let the code begin...
    

package UpdateRDB;

use vars qw($AUTOLOAD 
	    @ISA 
	    @EXPORT_OK ); 

use strict;
use Bio::Pfam::PfamRDB;

@ISA = qw(Bio::Pfam::PfamRDB Exporter);
@EXPORT_OK = qw();

sub _initialize {
    my ($self, %params) = @_;

    my $make = $self->SUPER::_initialize( %params );
    # insert fields here

    return $make;
}


############## The following mewthods can be used to update the database
############## and should beused with extreme caution


=head2 check_in_EntryA

 Title   : check_in_entry
 Usage   : $updatedb->check_in_EntryA( $entry );
 Function:
    This function updates the underlying relational database
    by loading the given family through the middleware layer
    and inserting all relevant records into the correct tables,
    deleting records if necesary
 Returns :
 Args    : Bio::Pfam::EntryA

=cut

sub check_in_EntryA {
  my ($self, @en ) = @_;
  
  my ($dbh,
      @regions );
  
 
  $dbh = $self->open_transaction('pfamA', 'pfamA_reg_seed', 'pfamA_reg_full', 'pfamseq', 'pfamA_literature_references', 'literature_references' , 'pfamA_database_links');
  
    eval {
      foreach my $ent (@en) {
	$self->update_pfamA( [$ent] );
	
	$self->update_pfamA_reg_seed( [$ent] );
	
	$self->update_pfamA_reg_full( [$ent] );
	
	$self->update_literature_references( [$ent] );
	$self->update_pfamA_database_links( [$ent] );
	 
      }
    };

  $self->close_transaction( $@);
  $@ and $self->throw($@);
  
}







=head2 delete_EntryA

 Title   : delete_entryA
 Usage   : $updatedb->delete_EntryA( @entry_list );
 Function:
    This function deletes all of the region information
    associated with a pfamA enttry. Note that even the entry
    itself is deleted from pfamA. Even though dead entries 
    are not deleted from the pfamA table, this occurs so
    that if we wish to replace the entry by using the file
    entry mechanism, everything works.
 Returns :
 Args    : Bio::Pfam::EntryA (list of)

=cut

sub delete_EntryA {
   my ($self, @ents) = @_;

   my ($dbh,
       @regions );
   
   $dbh = $self->open_transaction( 'pfamA', 'pfamA_reg_seed', 'pfamA_reg_full' , 'pfamA_database_links', 'pfamA_literature_references', 'pdbmap', 'pfam_to_interpro');
   
   eval {
       foreach my $en (@ents) {
	   my $acc = $en->acc;

	   ####### AUTO NUM 
	   my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_acc = '$acc'");
	   $st->execute();
	   my($temp_auto) = $st->fetchrow;
	   $st->finish();
	   
	   my $rdb_auto_num = $temp_auto if(defined($temp_auto)); 

	   $dbh->do("delete from pfamA where auto_pfamA = '$rdb_auto_num'");
	   $dbh->do("delete from pfamA_reg_seed where auto_pfamA = '$rdb_auto_num'");
	   $dbh->do("delete from pfamA_reg_full where auto_pfamA = '$rdb_auto_num'");
	   $dbh->do("delete from pfamA_database_links where auto_pfamA = '$rdb_auto_num'");
	   $dbh->do("delete from pfamA_literature_references where auto_pfamA = '$rdb_auto_num'");
	   $dbh->do("delete from pdbmap where auto_pfam = '$rdb_auto_num'");
	   $dbh->do("delete from pfam_to_interpro where auto_pfamA = '$rdb_auto_num'");
       }
   };
   


   $self->close_transaction( $@);
   $@ and $self->throw($@);

}




=head2 empty_tables

 Title   : empty_tables
 Usage   :
 Function:
    This function empties the given tables as a single recoverable transaction.
    If a transaction handle is given, it is assumed that the operation is part
    of a larger internal transaction
 Returns :
    The number of rows deleted from each table (a list);
    Throws an exception if something went wrong
 Args    :
    1. A ref to a list of table names
 Notes   :
    1. If the transaction handle is undefined, a transaction is created and committed
    for just this update.

=cut

sub empty_tables {
   my ($self, $table_list) = @_;
   my (@rows, $dbh, $tab, $error, $rows);

   $dbh = $self->open_transaction( @{$table_list} );

   eval {
       foreach $tab (@{$table_list}) {
	   $self->report_mode and 
	       printf STDERR "Removing all records from %s.%s\n", $self->_database_name, $tab;
	   $dbh->do( $self->__empty_table_sql( $tab ) );
	   push @rows, $dbh->rows;
       }
   };

   $self->close_transaction( $@ );
   $@ and $self->throw($@);

   return $rows;
}





=head2 load_pfamA_from_file

 Title   : load_pfamA_from_file
 Usage   :
 Function:
    This function takes the name of a file, each line of which is a tab-delimited 
    list of:
    1. Pfam-A accession
    2. Pfam-A identifier
    3. Pfam-A desc.
    4. Length of HMM
    5. No. seqs in seed
    6. No seqs in full
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A file name

 Notes   :
    1. If the transaction handle is undefined, a transaction is created and committed
    for just this update. If a handle is given, then it is assumed that the update
    is part of a larger transaction.

=cut

sub load_pfamA_from_file{
   my ($self, $file) = @_;
   my ($rows,
       $dbh);

   $dbh = $self->open_transaction( 'pfamA' );
   
   if (! -e $file) {
       $self->throw("load_pfamA_from_file error: Could not find file $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.pfamA from file %s\n", $self->_database_name, $file;
   eval {
       $rows = $dbh->do( $self->__insert_from_file_sql( 'pfamA', $file ));
   };

   $self->close_transaction( $@ );
   $@ and $self->throw( $@ );

   return $rows;
}



sub load_generic_file_to_rdb {

  my ($self, $file, $table_name, $delete_from_table) = @_;
   my ($rows,
       $dbh);

 
   $dbh = $self->open_transaction( $table_name);
   
   if (! -e $file) {
       $self->throw("load_ $table_name _from_file error: Could not find file $file");
   }
  if (!$table_name) {
       $self->throw("load_ $table_name _from_file error: Did not pass the table_name");
   }
  
  if ($delete_from_table) {
    $dbh->do("delete from $table_name");
  }

    

   $self->report_mode and 
       printf STDERR "Inserting into %s.$table_name from file %s\n", $self->_database_name, $file;
   eval {
       $rows = $dbh->do( $self->__insert_from_file_sql( $table_name, $file ));
   };

   $self->close_transaction( $@ );
   $@ and $self->throw( $@ );

   return $rows;

}


=head2 load_pfamB_from_file

 Title   : load_pfamB_from_file
 Usage   :
 Function:
    This function takes the name of a file, each line of which is a tab-delimited 
    list of:
    1. Pfam-B accession
    2. Pfam-B identifier
    3. Number of sequences in the alignment/family
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A file name

 Notes   :
    1. If the transaction handle is undefined, a transaction is created and committed
    for just this update. If a handle is given, then it is assumed that the update
    is part of a larger transaction.

=cut

sub load_pfamB_from_file{
   my ($self, $file) = @_;
   my ($rows,
       $dbh,
	$stat,
	$error);

   $dbh = $self->open_transaction( 'pfamB' );
   
   if (! -e $file) {
       $self->throw("load_pfamB_from_file error: Could not find file $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.pfamB from file %s\n", $self->_database_name, $file;

   eval {

	open(_FILE, $file) or die "canna open $file as $! \n";
	while(<_FILE>) {
		chop($_);
	my($rdb_acc, $rdb_id) = split(/\s+/,$_);
	my $rdb_auto_pfamB; 	
           if (not defined $stat) {
               $stat = $dbh->prepare($self->__replace_sql('pfamB', 3));
           }

           $stat->execute( $rdb_auto_pfamB,
                          $rdb_acc,
                           $rdb_id
		);
	$rows += $stat->rows;


	}
	close(_FILE);

       #$rows = $dbh->do( $self->__insert_from_file_sql( 'pfamB', $file ));

	
   };



  if ($@) {
           $error = "Could not do the insertion/update on the pfamB table [$@]";
           last;
       }
 
   $self->close_transaction( $@ );
   $@ and $self->throw( $@ );

   return $rows;
}


####### THIS SUB ASSUMES THAT AUTO KEYS ADDED - copying between RDB's only  ########
=head2 load_pfamA_reg_full_from_file

 Title   : load_pfamA_reg_full_from_file
 Usage   :
 Function:
    This function populates the pfamA_reg_full table from a file,
    each line of which is tab-delimited as:
    1. pfamseq identifer
    2. seq start
    3. seq end
    4. model start
    5. model end
    6. score
    7. Pfam-A accession
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A file name (the name of a file that has already been filled).
    2. Transaction handle (optional). New transaction started if none given

 Notes   :

=cut

sub load_pfamA_reg_full_from_file{
   my ($self, $file) = @_;
   my ($dbh,
       $count,
       @regions);

   $dbh = $self->open_transaction( 'pfamA_reg_full' );

   if (! -e $file) {
       $self->throw("update_pfamA_reg_full_from_file error. Could not find $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.pfamA_reg_full from file %s\n", $self->_database_name, $file;

   eval {
       $count = $dbh->do( $self->__insert_from_file_sql( 'pfamA_reg_full', $file ));
   };
   
   $self->close_transaction($@ );
   $@ and $self->throw( $@ );

   return $count;
}



####### THIS SUB ASSUMES THAT AUTO KEYS ADDED - copying between RDB's only  ########
=head2 load_pfamA_reg_seed_from_file

 Title   : load_pfamA_reg_seed_from_file
 Usage   :
 Function:
    This function populates the pfamA_reg_seed table from a file,
    each line of which is tab-delimited as:
    1. pfamseq identifer
    2. seq start
    3. seq end
    7. Pfam-A accession
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A file name (the name of a file that has already been filled).

 Notes   :

=cut

sub load_pfamA_reg_seed_from_file{
   my ($self, $file) = @_;
   my ($dbh,
       $count,
       @regions);

   $dbh = $self->open_transaction( 'pfamA_reg_seed' );

   if (! -e $file) {
       $self->throw("update_pfamA_reg_full_from_file error. Could not find $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.pfamA_reg_seed from file %s\n", $self->_database_name, $file;

   eval {
       $count = $dbh->do( $self->__insert_from_file_sql( 'pfamA_reg_seed', $file ));
   };
   
   $self->close_transaction($@ );
   $@ and $self->throw( $@ );

   return $count;
}




######## UPDATE ############
=head2 load_pfamB_reg_from_file

 Title   : load_pfamB_reg_from_file
 Usage   :
 Function:
    This function populates the pfamB_reg table from a file,
    each line of which is tab-delimited as:
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A file name (the name of a file that has already been filled).

 Notes   :

=cut

sub load_pfamB_reg_from_file{
   my ($self, $file) = @_;
   my ($dbh,
       $count,
       @regions,
	$stat,
	$error);

   $dbh = $self->open_transaction( 'pfamB_reg' , 'pfamseq', 'pfamB');

   if (! -e $file) {
       $self->throw("update_pfamB_reg_from_file error. Could not find $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.pfamB_reg from file %s\n", $self->_database_name, $file;

   eval {
	open(_FILE, $file) or die "cant open $file as $! \n";
	while(<_FILE>) {
	chop($_);
	my($pfamseq, $start, $end, $pfamB_acc) = split(/\s+/, $_);
	my $st = $dbh->prepare("select auto_pfamseq from pfamseq where pfamseq_id = '$pfamseq'");
	$st->execute();
	my $rdb_seqid = $st->fetchrow;            
	$st->finish();
	
        my $b_st = $dbh->prepare("select auto_pfamB from pfamB where pfamB_acc = '$pfamB_acc'");
        $b_st->execute();
	my $rdb_auto_pfamB = $b_st->fetchrow;            
	$b_st->finish();
	

	if (not defined $stat) {
	  $stat = $dbh->prepare($self->__replace_sql('pfamB_reg', 4));
	}
	
	$stat->execute( $rdb_auto_pfamB,
			$rdb_seqid,
                        $start,
			$end 
		      );
	
	$count += $stat->rows;
	


		
      };

	if ($@) {
	  $error = "Could not do the insertion/update on the pfamB_reg table [$@]";
	  last;
	}
	

	close(_FILE);
	

	#$count = $dbh->do( $self->__insert_from_file_sql( 'pfamB_reg', $file ));
      };

   $self->close_transaction( $@ );
   $@ and $self->throw( $@ );

   return $count;
}




=head2 load_pfamseq_from_file

 Title   : load_pfamseq_table_from_file
 Usage   :
 Function:
    This function takes a file, each line of is a tab-delimited as follows:
    1. acc - accession number
    2. id - identifier
    3. desc - description for entry
    4. len - length of associated sequence
    5. org - host organism of sequence
    
    and loads the pfamseq table in the rdb with the info
 Returns :
    The number of rows inserted (should be the size of pfamseq);
    Throws an exception if something went wrong
 Args    :
    1. The file storing the data

=cut

sub load_pfamseq_from_file {
   my ($self, $file) = @_;
   my ($rows,
       $dbh);

   $dbh = $self->open_transaction( 'pfamseq' );

   if (! -e $file) {
       $self->throw("load_pfamseq_from_file error: Could not find $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.pfamseq from file %s\n", $self->_database_name, $file;

   eval {
       $rows = $dbh->do( $self->__insert_from_file_sql( 'pfamseq', $file ));
   };

   $self->close_transaction( $@ );
   $@ and $self->throw( $@ );
   
   $self->report_mode and 
       printf STDERR "Just added %d records to %s.pfamseq\n", $rows, $self->_database_name;
   
   return $rows;
}


##### SHOULD BE OK #########
=head2 load_other_reg_from_file

 Title   : load_other_reg_from_file
 Usage   :
 Function:
    This function takes the name of a file, each line of which is a tab-delimited 
    list of:
    1. pfamseq id
    2. start res.
    3. end res.
    4. region type
    5. region source
    6. score
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A file name

 Notes   :
    1. If the transaction handle is undefined, a transaction is created and committed
    for just this update. If a handle is given, then it is assumed that the update
    is part of a larger transaction.

=cut

sub load_other_reg_from_file {
   my ($self, $file, $temp_dir) = @_;
   my ($rows,
       $dbh);

   $dbh = $self->open_transaction( 'other_reg', 'pfamseq' );
   
   if (! -e $file) {
       $self->throw("load_other_regions_from_file error: Could not find file $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.other_reg from file %s\n", $self->_database_name, $file;
   
   ### If pfamseq not in correct format then do pfamseq lookup.
   my ($fname1 , $write_file);
   if ($temp_dir) {
     my $read_file = $file;
     open(_FILE, $read_file);
     $file = $temp_dir . "/output_$$";
     $write_file = FileHandle->new();
     $write_file->open(">$file") or die "Could not open file $fname1 for writing";

     while(<_FILE>) {
       my($id, $pfamseq, $start, $end, $region, $source) = split(/\s+/, $_);
       my $st = $dbh->prepare("select auto_pfamseq from pfamseq where pfamseq_id = '$pfamseq'");
       $st->execute();
       my $rdb_seqid = $st->fetchrow;		 
       $st->finish();
       print $write_file "\\N\t$rdb_seqid\t$start\t$end\t$region\t$source\n";
       if ($@) {
	 my $error = "Problem with getting the auto_pfamseq from pfamseq table, maybe a problem with the format";
	 last;
       }

     }
      

     close(_FILE);

   }
 
   eval {
     $write_file->close;

       $rows = $dbh->do( $self->__insert_from_file_sql( 'other_reg', $file ));
   };

   if ($temp_dir) {

    unlink $file;
   }

   $self->close_transaction( $@ );
   $@ and $self->throw( $@ );

   return $rows;
}
     


# FIX ?
sub load_complexes_from_file {
   my ($self, $file) = @_;
   my ($rows,
       $dbh);

   $dbh = $self->open_transaction( 'complexes' );
   
   if (! -e $file) {
       $self->throw("load_other_regions_from_file error: Could not find file $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.complexes from file %s\n", $self->_database_name, $file;

   eval {
       $rows = $dbh->do( $self->__insert_from_file_sql( 'complexes', $file ));
   };

   $self->close_transaction( $@ );
   $@ and $self->throw( $@ );

   return $rows;
 }



# FIXED
sub load_pdbmap_from_file {
   my ($self, $file) = @_;
   my ($rows,
       $dbh);


   $dbh = $self->open_transaction( 'pdbmap', 'pfamseq', 'pfamA', 'pfamB', 'pdb' );

   if (! -e $file) {
       $self->throw("load_other_regions_from_file error: Could not find file $file");
   }

   $self->report_mode and 
       printf STDERR "Inserting into %s.complexes from file %s\n", $self->_database_name, $file;


   my (%store_pfamseq, %store_pdb, %store_pfam);
     open(_FILE, $file) or die "Canna open $file as $! \n";
     while(<_FILE>) {
       eval { 
  
       chop($_);
       my($pdb_id, $chain, $pdb_start, $pdb_end, $pfam_region, $pfamseq, $start, $end, $complex) = split(/\t/, $_);
 
       my ($rdb_seqid,  $stat, $auto_pdb);
      
       if (defined($store_pfamseq{$pfamseq})) {
	 $rdb_seqid = $store_pfamseq{$pfamseq};
	 
       } else {
	 my $st = $dbh->prepare("select auto_pfamseq from pfamseq where pfamseq_id = '$pfamseq'");
	 $st->execute();
	 $rdb_seqid = $st->fetchrow;            
	 $st->finish();

	 $store_pfamseq{$pfamseq} = $rdb_seqid;
       }
       
       my ($domain_type, $auto_pfam);
 
       if (defined($store_pfam{$pfam_region})) {
	 $auto_pfam = $store_pfam{$pfam_region};
	 if ($pfam_region =~ /Pfam-B_/i) {
	   $domain_type = "0";
	 } else {
	   $domain_type = "1";
	 }

       } else {

	 if ($pfam_region =~ /Pfam-B_/i) {
	   $domain_type = "0";

	   
	   my $st = $dbh->prepare("select auto_pfamB from pfamB where pfamB_id = '$pfam_region'");
	   $st->execute();
	   $auto_pfam = $st->fetchrow;            
	   $st->finish();
	   $store_pfam{$pfam_region} = $auto_pfam;
	   
	 } else {
	   $domain_type = "1";

	   my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_id = '$pfam_region' ");
	 
	   $st->execute();
	   $auto_pfam = $st->fetchrow;            
	   $st->finish();

	    $store_pfam{$pfam_region} = $auto_pfam;
	 }

       }

       
 
       if (defined($store_pdb{$pdb_id})) { 
	 $auto_pdb = $store_pdb{$pdb_id};

       } else {
       	 my $st = $dbh->prepare("select auto_pdb from pdb where pdb_id = '$pdb_id' ");
	 
	 $st->execute();
	 $auto_pdb = $st->fetchrow;            
	 $st->finish();			
	 
	 $store_pdb{$pdb_id} =  $auto_pdb;
       }


           if (not defined $stat) {
               $stat = $dbh->prepare($self->__replace_sql('pdbmap', 10));
           }

       if ( (!$auto_pdb) || (!$auto_pfam) ||(!$rdb_seqid) ) {

#	 print  qq (CANT ADD: pdbid: $pdb_id :: AUTO: $auto_pdb, CHAIN: $chain, pdb_start: $pdb_start, pdb_end: $pdb_end, auto_pfam: $auto_pfam,  domain_type: $domain_type, rdb_seqid: $rdb_seqid, start: $start, end: $end, $complex \n);
       } else {


           $stat->execute( $auto_pdb,
			   $chain,
			   $pdb_start,
			   $pdb_end,
			   $auto_pfam,
			   $domain_type,
			   $rdb_seqid,
			   $start,
			   $end,
			   $complex);

	   $rows += $stat->rows;
	 
	 }

   
 
     };

       if ($@) {
	 print "Update of pdbmap failed as:  $@ \n";
       }

     }

   close(_FILE);

   $self->close_transaction( $@ );
   $@ and $self->throw( $@ );

   return $rows;
 }


sub check_in_dead_EntryA {
 my ($self, @en) = @_;
 
 my ($stat,
       $dbh,
       $rows, 
       $error, 
       $rdb_acc, 
       $rdb_id, 
     $rdb_comment,
     $rdb_auto_num,
     $rdb_forward
     );

 $dbh = $self->open_transaction( 'pfamA', 'dead_families' );

$self->report_mode();
 
 eval {
   foreach my $en (@en) {
 
     $rdb_acc = $en->acc;
    
     $rdb_id = $en->id();
     $rdb_comment = $en->comment();
     $rdb_forward = $en->each_forward_acc();
     foreach my $forward ($en->each_forward_acc()) {
        $rdb_forward = $rdb_forward . ";$forward";
    }

          
     ### GET COMMENT
     foreach my $comment ( $en->ann->flatcomment->each_flat() ) {
       $rdb_comment .= " " . $comment;
     }
     
     
     eval {
       if (not defined $stat) {
	 $stat = $dbh->prepare($self->__replace_sql('dead_families', 4));
       }
       
       $stat->execute( $rdb_acc, 
		       $rdb_id, 
		       $rdb_comment,
		       $rdb_forward
		     );
       $rows += $stat->rows;
     };

  
     if ($@) {
       $error = "Could not do the insertion/update on the dead_families table [$@]";
      last;
    }
    
  }

  };
 

 
 $self->close_transaction( $error );
 $error and $self->throw( $error );

 ### No errors so delete the pfamA entry from RDB !
 if(!$error) {
   $self->delete_EntryA(@en);
 }


}


=head2 update_pfamA

 Title   : update_pfamA
 Usage   :
 Function:
    This function takes a pfam EntryA and progresses the given
    tranaction by updating the corresponding pfamA table in the RDB
 Returns :
    The number of rows inserted/updated (should be 1);
    Throws an exception if something went wrong
 Args    :
    1. A reference to a list of entries


 Notes   :
    1. If the transaction handle is undefined, a transaction is created and committed
    for just this update. If a handle is given, then it is assumed that the update
    is part of a larger transaction.

    2. If a dead entry is given, it is assumed that we need to update the database
    for the KILLING of this family.

=cut

sub update_pfamA{
   my ($self, $entries) = @_;
   my ($stat,
       $dbh,
       $rows, 
       $error, 
       $rdb_acc, 
       $rdb_id,  
       $rdb_desc, 
       $rdb_modlen,
       
       $rdb_auto_num,
       $rdb_author,
       $rdb_comment,
       $rdb_entry_type,
       $rdb_align_method,
       $gathering_cutoff,
       $noise_cutoff,
       $rdb_previous_ids,
       $rdb_source,
       $trusted_cutoff,

       $rdb_ls_sequence_GA,
       $rdb_ls_domain_GA,
       $rdb_fs_sequence_GA, 
       $rdb_fs_domain_GA,

       $rdb_ls_sequence_TC,
       $rdb_ls_domain_TC,
       $rdb_fs_sequence_TC, 
       $rdb_fs_domain_TC,

       $rdb_ls_sequence_NC,
       $rdb_ls_domain_NC,
       $rdb_fs_sequence_NC, 
       $rdb_fs_domain_NC,

       $rdb_ls_mu,
       $rdb_ls_kappa,
       $rdb_fs_mu,
       $rdb_fs_kappa,
       $rdb_hmmcalibrate_ls,
       $rdb_hmmbuild_ls,
       $rdb_hmmcalibrate_fs,
       $rdb_hmmbuild_fs,
       $rdb_num_full,
       $rdb_num_seed,

      );
   
   $dbh = $self->open_transaction( 'pfamA' );

   foreach my $en (@{$entries}) {
    
       eval {

	   $rdb_acc = $en->acc;

	   $rdb_id = $en->id();
	   $rdb_auto_num = $en->auto_pfamA;

	   ### NEW AUTO NUMBER

 
	   ####### AUTO NUM 
	   my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_acc = '$rdb_acc'");
	   $st->execute();
	   my($temp_auto) = $st->fetchrow;
	   $st->finish();
	   
	  $rdb_auto_num = $temp_auto if(defined($temp_auto)); 



	   if (defined $en->ann) {
	    

	       $rdb_desc = $en->ann->description;

	     ### NEW RDB PARAMS

	       $rdb_author = $en->author;  # author

	      ### GET COMMENT
	       foreach my $comment ( $en->ann->flatcomment->each_flat() ) {
		$rdb_comment .= " " . $comment;
	       }
	
	       $rdb_entry_type = $en->entry_type(); 


	       $rdb_align_method = $en->alignmethod();

	       $rdb_ls_sequence_GA = $en->ls_sequence_gathering_cutoff();
	       $rdb_ls_domain_GA = $en->ls_domain_gathering_cutoff();
	       $rdb_fs_sequence_GA = $en->fs_sequence_gathering_cutoff(); 
	       $rdb_fs_domain_GA  = $en->fs_domain_gathering_cutoff();


	       $rdb_ls_sequence_NC = $en->ls_sequence_noise_cutoff();
	       $rdb_ls_domain_NC = $en->ls_domain_noise_cutoff();
	       $rdb_fs_sequence_NC = $en->fs_sequence_noise_cutoff(); 
	       $rdb_fs_domain_NC  = $en->fs_domain_noise_cutoff();

	       $rdb_previous_ids = $en->previous_ids();
	       $rdb_source = $en->source();

	       $rdb_ls_sequence_TC = $en->ls_sequence_trusted_cutoff();
	       $rdb_ls_domain_TC = $en->ls_domain_trusted_cutoff();
	       $rdb_fs_sequence_TC = $en->fs_sequence_trusted_cutoff(); 
	       $rdb_fs_domain_TC  = $en->fs_domain_trusted_cutoff();

	       
	       $rdb_ls_mu = $en->ls_mu();
	       $rdb_ls_kappa = $en->ls_kappa();
	       $rdb_fs_mu = $en->fs_mu();
	       $rdb_fs_kappa = $en->fs_kappa();

	     #  $en->load_desc_file_info();
	       $rdb_hmmcalibrate_ls = $en->hmmcalibrate_ls;
	       $rdb_hmmbuild_ls = $en->hmmbuild_ls;
	       $rdb_hmmcalibrate_fs = $en->hmmcalibrate_fs;
	       $rdb_hmmbuild_fs = $en->hmmbuild_fs;

	       $rdb_num_full = "";
	       $rdb_num_seed = "";
	       

	   }
	   if (not $en->is_dead) {
	       $rdb_modlen = $en->model_length;
	   }
       };
       if ($@) {
	   $error = "Could not fetch all the needed data from the pfamA entry [$@]";
	   last;
       }
    
       $self->report_mode and 
	   printf STDERR "Inserting into %s.pfamA from entry %s\n", $self->_database_name, $rdb_acc;
       

       eval {
	   if (not defined $stat) {
	       $stat = $dbh->prepare($self->__replace_sql('pfamA', 33));
	   }


	   $stat->execute( $rdb_auto_num, 
			  $rdb_acc, 
			   $rdb_id, 
			   $rdb_desc, 
			   $rdb_modlen,
			   $rdb_author,			  
			   $rdb_source,
			   $rdb_align_method,
			   $rdb_entry_type,			 			   
			   $rdb_ls_sequence_GA,
			   $rdb_ls_domain_GA,
			   $rdb_fs_sequence_GA, 
			   $rdb_fs_domain_GA,			   
			   $rdb_ls_sequence_TC,
			   $rdb_ls_domain_TC,
			   $rdb_fs_sequence_TC, 
			   $rdb_fs_domain_TC,			   
			   $rdb_ls_sequence_NC,
			   $rdb_ls_domain_NC,
			   $rdb_fs_sequence_NC, 
			   $rdb_fs_domain_NC,
			   $rdb_ls_mu,
			   $rdb_ls_kappa,
			   $rdb_fs_mu,
			   $rdb_fs_kappa,
			   $rdb_comment,
			   $rdb_previous_ids,
			   $rdb_hmmbuild_ls,
			   $rdb_hmmcalibrate_ls,
			   $rdb_hmmbuild_fs,
			   $rdb_hmmcalibrate_fs,
			   $rdb_num_seed, 
			   $rdb_num_full
			 );
	   $rows += $stat->rows;
       };
       
       if ($@) {
	 
	   $error = "Could not do the insertion/update on the pfamA table [$@]";
	   
	   last;
       }
   }
   


   $self->close_transaction( $error );
   $error and $self->throw( $error );

   return $rows;
}



##### UPDATE #
=head2 update_pfamB

 Title   : update_pfamB
 Usage   :
 Function:
    This function takes a list EntryB objects, and fills in the pfamB table from them
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A ref to a list of entries
    2. A boolean. If true, the data will be first written to an
       intermediate file, and read in from there
       (This ultimately uses the MySql-specific 'load data infile' so is
	not a portable solution).
    3. A transaction handle (optional)

 Notes   :
    1. If the transaction handle is undefined, a transaction is created and committed
    for just this update. If a handle is given, then it is assumed that the update
    is part of a larger transaction.

=cut

sub update_pfamB {
   my ($self, $entries) = @_;
   my ($stat,
       $dbh,
       $rows,
       $error, 
       $rdb_acc, 
       $rdb_id);

   $dbh = $self->open_transaction( 'pfamB' );

   foreach my $en (@{$entries}) {
       $rdb_acc = $en->acc;
       $rdb_id = $en->id();

       $self->report_mode and 
	   printf STDERR "Inserting into %s.pfamB from entry %s %s\n", $self->_database_name, $rdb_acc, $rdb_id;

       eval {
	   if (not defined $stat) {
	       $stat = $dbh->prepare($self->__replace_sql('pfamB', 2));
	   }
	   $stat->execute( $rdb_acc, 
			   $rdb_id);
	   $rows += $stat->rows;
       };
       if ($@) {
	   $error = "Could not do the insertion/update on the pfamB table [$@]";
	   last;
       }
   }

   $self->close_transaction( $error );
   $error and $self->throw( $error );

   return $rows;
}




=head2 update_pfamA_reg_full

 Title   : update_pfamA_reg_full
 Usage   :
 Function:
    This function takes a pfam EntryA and progresses the given
    tranaction by updating the corresponding pfamA_table table in the RDB
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A list of entries

 Notes   :
    If the given entry is dead, then we need to kill the family. As far 
    as the pfamA_reg_full table is concerned, this just means deleting
    all records concerning that family from the table

=cut

sub update_pfamA_reg_full {
   my ($self, $entries) = @_;

 
   my ($error, 
       $dbh,
       $stat,
       $rdb_acc, 
       $rdb_id, 
       $rdb_auto_num,
       $count,
       $rdb_mode,
       $rdb_significant,
       $rdb_in_full,       
       @regions,
      %store_pfamseq);

   $dbh = $self->open_transaction( 'pfamA_reg_full' );

   foreach my $en (@{$entries}) {
       eval {
	   $rdb_acc = $en->acc;
	   $rdb_id = $en->id();

	   #### Get what the auto_pfamA is from pfamA table
	   my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_acc = '$rdb_acc'");
	   $st->execute();
	   my($temp_auto) = $st->fetchrow;
	   $st->finish();

	   $rdb_auto_num = $temp_auto if(defined($temp_auto)); 


	   if (not $en->is_dead()) {
	    
	       @regions = $en->all_full_annotated_regions('FULL');
	   }
	  
       };
       if ($@) {
	   $error = "Could not fetch all the needed data from the pfamA entry [$@]";
	   last;
       }


       eval {
	   $self->report_mode and 
	       printf STDERR "Deleting from %s.pfamA_reg_full for entry %s\n", $self->_database_name, $rdb_auto_num;
	 
	   $dbh->do("delete from pfamA_reg_full where auto_pfamA = '$rdb_auto_num'");
	  
       };   
       if ($@) {
	   $error = "Could not delete from pfamA_reg_full rec. $rdb_auto_num [$@]";
	   last;
       }

     
       if (not $en->is_dead) {
	   $self->report_mode and 
	       printf STDERR "Inserting into %s.pfamA_reg_full from entry %s\n", $self->_database_name, $rdb_auto_num;
	
	   eval {
	       if (not defined $stat) {
		   $stat = $dbh->prepare( $self->__insert_sql( 'pfamA_reg_full', 14));
	       }
	     
	      
	       foreach my $reg (@regions) {

		 #### Get what the auto_pfamseq is from pfamseq table
		 my $pfamseq_id = $reg->seq_name;

		  my($rdb_auto_pfamseq);
		 if (defined($store_pfamseq{$pfamseq_id})) {
		   $rdb_auto_pfamseq = $store_pfamseq{$pfamseq_id};
		 } else {
		   my $st = $dbh->prepare("select auto_pfamseq from pfamseq where pfamseq_id = '$pfamseq_id'");
		   $st->execute();
		   $rdb_auto_pfamseq = $st->fetchrow;
		   print  "ERROR: $pfamseq_id is NOT in the pfamseq table \n" if (!$rdb_auto_pfamseq);
		   $st->finish();
		   $store_pfamseq{$pfamseq_id} = $rdb_auto_pfamseq;
		 }
 



		 
		 ## Fix is_sig so that will be 0 and not null
		 my $temp_sig = $reg->is_significant;		 
		 $rdb_significant = $temp_sig;		 
		 $rdb_significant = "0" if ($temp_sig ne 1 );

		 ## Fix in full so that 


		 my $temp = $reg->in_full;
		 $rdb_in_full = $temp;
		 $rdb_in_full = "0" if ($temp ne 1 );
		 my $tree = "";


		 eval {
		   $stat->execute($rdb_auto_num,
				  $rdb_auto_pfamseq, 
				  $reg->from, 
				  $reg->to, 
				  $reg->model_from, 
				  $reg->model_to,
				  $reg->domain_bits_score,
				  $reg->domain_evalue_score,	
				  $reg->sequence_bits_score,
				  $reg->sequence_evalue_score,				  	
				  $reg->mode,
				  $rdb_significant ,
				  $rdb_in_full,
				  $tree
				 );
		   
		   $count += $stat->rows; 
		  
		 };
		  if ($@) {
		   my  $error = "Could not insert data into pfamA_reg_full table [$@]";
		    print "ERROR: $error \n";

		  }

		 

	       }
	   };
	  
	   if ($@) {
	       $error = "Could not insert data into pfamA_reg_full table [$@]";
	       print "ERROR: $error \n";
	       last;
	   }
	   
       }
   }
  
   $self->close_transaction( $error );
   $error and $self->throw( $error );

   return $count;
}




####### OK ? 
=head2 update_pfamA_reg_seed

 Title   : update_pfamA_reg_seed
 Usage   :
 Function:
    This function takes a pfam EntryA and progresses the given
    tranaction by updating the corresponding pfamA_table table in the RDB
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A list of entries

 Notes   :
    If the given entry is dead, then we need to kill the family. As far 
    as the pfamA_reg_seed table is concerned, this just means deleting
    all records concerning that family from the table

=cut
sub update_pfamA_reg_seed {
   my ($self, $entries) = @_;
   my ($error, 
       $dbh,
       $stat,
       $rdb_acc, 
       $rdb_id, 
       $rdb_auto_num,
       $count,
       @regions);

   $dbh = $self->open_transaction( 'pfamA_reg_seed' , 'pfamseq');
 
   foreach my $en (@{$entries}) {
       eval {
	   $rdb_acc = $en->acc;
	   $rdb_id = $en->id();
	   
	   #### Get what the auto_pfamA is from pfamA table
	   my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_acc = '$rdb_acc'");
	   $st->execute();
	   my($temp_auto) = $st->fetchrow;
	   $st->finish();
	  
	   $rdb_auto_num = $temp_auto if(defined($temp_auto)); 

	   if (not $en->is_dead()) {
	       @regions = $en->annotated_regions('SEED');
	   }
       };

       if ($@) {
	   $error = "Could not fetch all the needed data from the pfamA entry [$@]";
	   last;
       }


       eval {
	   $self->report_mode and 
	       printf STDERR "Deleting from %s.pfamA_reg_seed for entry %s\n", $self->_database_name, $rdb_auto_num;


	   $dbh->do("delete from pfamA_reg_seed where auto_pfamA = '$rdb_auto_num'");
       };   
       if ($@) {
	   $error = "Could not delete from pfamA_reg_seed rec. $rdb_auto_num [$@]";
	   last;
       }

       if (not $en->is_dead) {
	   $self->report_mode and 
	       printf STDERR "Inserting into %s.pfamA_reg_seed from entry %s\n", $self->_database_name, $rdb_auto_num;

	   eval {
	       if (not defined $stat) {
		   $stat = $dbh->prepare( $self->__insert_sql( 'pfamA_reg_seed', 4));
	       }
	       foreach my $reg (@regions) {

		 

		 #### Get what the auto_pfamseq is from pfamseq table
		 my $pfamseq_id = $reg->seq_name;
		 
		 my $st = $dbh->prepare("select auto_pfamseq from pfamseq where pfamseq_id = '$pfamseq_id'");
		 $st->execute();
		 my($auto_pfamseq) = $st->fetchrow;
		 $st->finish();
		  print "$pfamseq_id \n" if (!$auto_pfamseq);
		   $stat->execute($rdb_auto_num,
				  $auto_pfamseq, 
				  $reg->from, 
				  $reg->to, 
				  );
		   $count += $stat->rows; 
	       }
	   };
	   if ($@) {
	       $error = "Could not insert data into pfamA_reg_seed table [$@]";
	       last;
	   }
       }
   }

   $self->close_transaction( $error );
   $error and $self->throw( $error );

   return $count;
}




=head2 update_pfamB_reg

 Title   : update_pfamB_reg
 Usage   :
 Function:
    This function takes a pfam EntryB and progresses the given
    tranaction by updating the corresponding pfamB table table in the RDB
 Returns :
    The number of rows inserted/updated;
    Throws an exception if something went wrong
 Args    :
    1. A ref. to a list of entries

 Notes   :


=cut

sub update_pfamB_reg {
   my ($self, $entries) = @_;
   my ($error,
       $dbh,
       $stat, 
       $rdb_acc, 
       $rdb_id, 
       $count,
       @regions);

   $dbh = $self->open_transaction( 'pfamB_reg' );

   foreach my $en (@{$entries}) {
       eval {
	   $rdb_acc = $en->acc;
	   $rdb_id = $en->id();
	   @regions = $en->annotated_regions();
       };
       if ($@) {
	   $error = "Could not fetch all the needed data from the pfamB entry [$@]";
	   last;
       }

       eval {
	   $self->report_mode and 
	       printf STDERR "Deleting from %s.pfamB_reg for entry %s\n", $self->_database_name, $rdb_acc;

          $dbh->do("delete from pfamB_reg where pfamB_acc = '$rdb_acc'");
       };
       if ($@) {
          $error = "Could not delete $rdb_acc records from pfamB_reg [$@]";
          last;
       }

       eval {
	   $self->report_mode and
	       printf STDERR "Inserting into %s.pfamB_reg from entry %s\n", $self->_database_name, $rdb_acc;

	   if (not defined $stat) {
	       $stat = $dbh->prepare( $self->__insert_sql('pfamB_reg', 4));
	   }
	   foreach my $reg (@regions) {
	       $stat->execute($reg->seq_name, 
			      $reg->from, 
			      $reg->to, 
			      $reg->accession );
	       $count += $stat->rows; 
	   }
       };
       if ($@) {
	   $error = "Could not insert data into pfamB_reg table [$@]";
	   last;
       }
   }
   
   $self->close_transaction( $error );
   $error and $self->throw( $error );
   
   return $count;
 }



########## NEW SUBS ############



sub update_literature_references {
  
  
  
  my ($self, $entries) = @_;
  my ($stat,
      $dbh,
      $rows, 
      $error, 
      $rdb_acc, 
      $rdb_id, 
      $rdb_auto_num
     );
  
  $dbh = $self->open_transaction( 'pfamA_literature_references', 'literature_references' );
  
  foreach my $en (@{$entries}) {
    
    
    
    eval {
      $rdb_acc = $en->acc;
      $rdb_id = $en->id();
      
      ####### AUTO NUM 
      my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_acc = '$rdb_acc'");
      $st->execute();
      my($temp_auto) = $st->fetchrow;
      $st->finish();
      
      $rdb_auto_num = $temp_auto if(defined($temp_auto)); 
      
      
    };
    
    
    eval {
      $self->report_mode and 
	printf STDERR "Deleting from %s.pfamA_literature_references for entry %s\n", $self->_database_name, $rdb_auto_num;
      
      $dbh->do("delete from pfamA_literature_references where auto_pfamA = '$rdb_auto_num'");
    };
    
    
    ## If there is a literature reference
    if ($en->ann->each_reference()) {
      
      my $count = 1;
      # For each reference
      foreach my $ref ( $en->ann->each_reference() ) {
	
	my($comment, $medline, $authors, $journal, $title);
	
	## Get out comment if there is one
	if (defined ($ref->comment() ) ) {
	  
	  foreach my $refcomm ( $ref->comment->each_flat() ) {
	    $comment .=  $refcomm . " ";
	    chop($refcomm);
	  }
	}
	
	

	$medline = $ref->medline();
	$title = $ref->title();
	$authors = $ref->authors;
	$journal = $ref->location();
	
	$title = $dbh->quote($title);
	
	$authors = $dbh->quote($authors);
	$journal = $dbh->quote($journal);
	
	
	### SEE IF MEDLINE ALREADY EXISTS in literature_reference table
	my $st = $dbh->prepare("select auto_lit from literature_references where medline = '$medline'");
	$st->execute();
	my($auto_lit) = $st->fetchrow;
	$st->finish();
	
	## Not already in medline so add it
	if (!$auto_lit) {
	  ## need to add the lit reference
	  eval {
	    $stat = undef;
	    if (not defined $stat) {
	      my $sql = "INSERT INTO literature_references VALUES ( NULL , $medline, $title, $authors, $journal)";
	      
	      $stat = $dbh->prepare($sql);
	    }
	    
	    $stat->execute();
	    $auto_lit = $stat->{insertid}; ## get the auto number
	    $rows += $stat->rows;
	  };
	  
	}
	
	
	### ok so added lit refs if we had to now add to pfamA_lit_ref whatsit table :-)
	
	eval {
	  $stat = undef;
	  if (not defined $stat) {
	    $stat = $dbh->prepare($self->__replace_sql('pfamA_literature_references', 4));
	  }
	  
	  $stat->execute( $rdb_auto_num, 
			  $auto_lit,
			  $comment,
			  $count);
	  
	};
	if ($@) {
	  $error = "Could not do the insertion/update on the pfamA_lit_refs table [$@]";
	  last;
	}
	
	$count++;	
      }				#/ for EACH REFERENCE     
      
    }				#/ if REFERENCES   
  }				#/ end for each entries
  
  
  $self->close_transaction( $error );
  $error and $self->throw( $error );
  
  return $rows;
  
  
}

sub update_pfamA_database_links {

  
  
  my ($self, $entries) = @_;
  my ($stat,
      $dbh,
      $rows, 
      $error, 
      $rdb_acc, 
      $rdb_id, 
      $rdb_auto_num
     );
  
  $dbh = $self->open_transaction( 'pfamA_database_links' );
  
  foreach my $en (@{$entries}) {
    
    eval {
      $rdb_acc = $en->acc;
      $rdb_id = $en->id();
      
      ####### AUTO NUM 
      my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_acc = '$rdb_acc'");
      $st->execute();
      my($temp_auto) = $st->fetchrow;
      $st->finish();
      
      $rdb_auto_num = $temp_auto if(defined($temp_auto)); 
           
    };
       
    eval {
      $self->report_mode and 
	printf STDERR "Deleting from %s.pfamA_database_references for entry %s\n", $self->_database_name, $rdb_auto_num;
      
      $dbh->do("delete from pfamA_database_links where auto_pfamA = '$rdb_auto_num'");
    };
    
    
    ## If there is a database reference

    if ($en->ann->each_link()) {
      foreach my $link ($en->ann->each_link) {

	my($database, $title, $db_link, @other_info, $all_other);
	$database = $link->database;
	$db_link = $link->primary_id();
	@other_info = $link->each_additional();
	foreach (@other_info) {

	  $all_other .= $_;
	}
	
	

	if (defined( $link->comment() )) {

	  foreach my $linkcomm ($link->comment->each_flat() ) {
	    $title = $linkcomm ;

	  }
	}

	eval {
	  $stat = undef;
	  if (not defined $stat) {
	    $stat = $dbh->prepare($self->__replace_sql('pfamA_database_links', 5));
	  }
	 
	  $stat->execute( $rdb_auto_num, 
			  $database,
			  $title,
			  $db_link,
			  $all_other);
	  
	};
	if ($@) {
	  $error = "Could not do the insertion/update on the pfamA_lit_refs table [$@]";
	  last;
	}
	
	
      }				#/ for EACH REFERENCE     
      
    }				#/ if REFERENCES   
  }				#/ end for each entries
  
  
  $self->close_transaction( $error );
  $error and $self->throw( $error );
  
  return $rows;
  
  

}





############ / end of new subs ##########




=head2 update_pfamseq

 Title   : update_pfamseq
 Usage   :
 Function:
    This function takes a list of hash references each with 4 fields:
    1. acc - accession number
    2. id - identifier
    3. desc - description for entry
    4. len - length of associated sequence
    5. org - hist organism os sequence
    
    and inserts the information into the pfamseq RDB table
 Returns :
    The number of rows inserted/_updated (should be the size of pfamseq);
    Throws an exception if something went wrong
 Args    :
    1. A ref. to a list of hash refs

 Notes   :
    1. If the transaction handle is undefined, a transaction is created and committed
    for just this _update. If a handle is given, then it is assumed that the update
    is part of a larger transaction.

=cut


sub update_pfamseq {
   my ($self, $entries) = @_;
   my ($stat, 
       $dbh,
       $rows, 
       $error, 
       $rdb_acc, 
       $rdb_id, 
       $rdb_desc, 
       $rdb_len, 
       $rdb_org,
      $rdb_auto,
      $rdb_crc64,
      $rdb_version,
      $rdb_is_fragment,
      $rdb_taxonomy);

   $dbh = $self->open_transaction( 'pfamseq' );

   foreach my $en (@{$entries}) {
     $rdb_acc = $en->{'acc'};
     $rdb_id = $en->{'id'};
     $rdb_desc = $en->{'desc'};
     $rdb_len = $en->{'len'};
     $rdb_org = $en->{'org'};
     $rdb_crc64= $en->{'crc64'};
     $rdb_is_fragment= $en->{'is_fragment'};
     $rdb_version = $en->{'version'};
     $rdb_taxonomy = $en->{'taxonomy'};

       eval {
	   if (not defined $stat) {
	       $stat = $dbh->prepare($self->__replace_sql('pfamseq', 10));
	   }
	   $stat->execute($rdb_auto,
			  $rdb_id, 
			  $rdb_acc, 
			  $rdb_crc64,
			  $rdb_desc, 
			  $rdb_len, 
			  $rdb_org,
			  $rdb_taxonomy,
			  $rdb_is_fragment,
			  $rdb_version
			 );
	   $rows += $stat->rows;
       };
       if ($@) {
	   $error = "Could not do the insertion/update on the pfamseq table [$@]";
	   last;
       }
   }

   $self->close_transaction( $error );
   $error and $self->throw( $error );

   $self->report_mode and 
       printf STDERR "Just added %d records to %s.pfamseq\n", $rows, $self->_database_name();

   return $rows;
}




=head2 update_other_reg

 Title   : update_other_reg
 Usage   :
 Function:
    This function takes a list of hash references each with 5 fields:
    1. seqid - id of pfamseq seq
    2. from - start residue
    3. to - end residue
    4. type - annotation type for this region
    5. source - source of the annotation for this region
    
    and inserts the information into the other_reg RDB table
 Returns :
    The number of rows inserted/_updated;
    Throws an exception if something went wrong
 Args    :
    1. A ref. to a list of hash refs

 Notes   :
    1. If the transaction handle is undefined, a transaction is created and committed
    for just this _update. If a handle is given, then it is assumed that the update
    is part of a larger transaction.

=cut


sub update_other_reg {
   my ($self, $entries, $read_in_pfamseq) = @_;
   my ($stat, 
       $dbh,
       $rows, 
       $error, 
       $rdb_seqid, 
       $rdb_from, 
       $rdb_to, 
       $rdb_type, 
       $rdb_source,
       $rdb_score,
      $rdb_orie,
      $rdb_other_auto);

   $dbh = $self->open_transaction( 'other_reg' , 'pfamseq');

   foreach my $en (@{$entries}) {
       $rdb_seqid = $en->{'seqid'};

       ## This is a flag use when the pfamseq_id has not been converted to the auto_pfamseq as used in new schema. 
       if ($read_in_pfamseq) {
	 my $st = $dbh->prepare("select auto_pfamseq from pfamseq where pfamseq_id = '$rdb_seqid'");
	 $st->execute();
	 $rdb_seqid = $st->fetchrow;		 
	 $st->finish();
       }
       if ($@) {
	 $error = "Problem with getting the auto_pfamseq from pfamseq table, maybe a problem with the format";
	 last;
       }
		  
       $rdb_from = $en->{'from'};
       $rdb_to = $en->{'to'};
       $rdb_type = $en->{'type'};
       $rdb_source = $en->{'source'};
       $rdb_score = $en->{'score'};
       $rdb_orie = $en->{'orientation'};

       eval {
	   if (not defined $stat) {
	       $stat = $dbh->prepare($self->__replace_sql('other_reg', 8));
	   }
	   $stat->execute( $rdb_other_auto,
			  $rdb_seqid, 
			   $rdb_from, 
			   $rdb_to, 
			   $rdb_type, 
			   $rdb_source,
			   $rdb_score,
			   $rdb_orie
			 );
	   $rows += $stat->rows;
       };
       if ($@) {
	   $error = "Could not do the insertion/update on the other_reg table [$@]";
	   last;
       }
   }

   $self->close_transaction( $error );
   $error and $self->throw( $error );

   $self->report_mode and 
       printf STDERR "Just added %d records to %s.other_reg\n", $rows, $self->_database_name;

   return $rows;
}


###################
######## THIS SUB READ ALL THE INTERPRO FILES FROM A DIR AND PUTS INFO + QiuckGO into RDB :-)
###################


sub upload_interpro_from_dir {
  
  my($self, $upload_dir) = @_;
  
  my($stat, $rows, $error);

  my  $dbh = $self->open_transaction( 'pfamA' , , 'pfam_to_interpro',  'interpro_and_go');


  eval {
    $self->report_mode and 
      printf STDERR "Deleting from pfamA_interpro\n";
    
    $dbh->do("delete from pfam_to_interpro");
    $dbh->do("delete from interpro_and_go");
  };
  if ($@) {
    $error = "Could not delete from pfam_to_interpro";
    last;
  }

  ## get all the acc & auto in one go
  my %store_acc;
  my $st = $dbh->prepare("select auto_pfamA, pfamA_acc from pfamA");
  $st->execute();
  my ($auto, $acc);
  while ( ($auto, $acc) = $st->fetchrow) {
    $store_acc{$acc} = $auto;
  }
  $st->finish();

  close(_READ);

  opendir(_EX_DIR,$upload_dir) || die("Could not open $upload_dir $!");
  
  foreach my $file ( readdir(_EX_DIR) ) {
    $file =~ /^\.+$/ && next;

;
    my ($acc_type, $junk) = split(/\./, $file);
    
    
    my $pfam_acc;
    if ($acc_type =~ /^PF/) {
      $pfam_acc = $acc_type;
    }


    ## temp taken out
    my $rdb_auto_pfamA = $store_acc{$pfam_acc};

      my $open_file = $upload_dir . "/" . $file;
      my($interpro_id, $interpro_abstract, $function, $process, $component);
      
      open(_FILE, $open_file) or die "CANNA open $file $! \n";
      
      while(<_FILE>) {
	
	
	if ($_ =~ /^ABSTRACT\s+(.*)/) {
	  $interpro_abstract .= $1;
	} elsif ($_ =~  /^INTERPRO\s+(\S+)/){
	  $interpro_id = $1;
	} elsif ($_ =~  /^PROCESS\s+(.*)/) {
	  $process = $1;
	} elsif ($_ =~  /^COMPONENT\s+(.*)/){
	  $component = $1;
	} elsif ($_ =~  /^FUNCTION\s+(.*)/) {
	  $function = $1;
	} else {
	  $interpro_abstract .= $_;
	}
	
	
      }
      
      
      eval {
	if (not defined $stat) {
	  $stat = $dbh->prepare($self->__replace_sql('interpro_and_go', 5));
	}
	$stat->execute( $interpro_id, 
			$interpro_abstract, 
			$function, 
			$component, 
			$process
		      );
	$rows += $stat->rows;
      };
      if ($@) {
	my $error = "Could not do the insertion/update on the pfamA_interpro table [$@]";
	last;
      }
      
      if ($rdb_auto_pfamA) {
	my $st;
	eval {
	  if (not defined $st) {
	    $st = $dbh->prepare($self->__replace_sql('pfam_to_interpro', 2));
	  }
	  $st->execute( $rdb_auto_pfamA, 
			$interpro_id 
		      );
	  $rows += $st->rows;
	};
	
	
	
	if ($@) {
	  my $error = "Could not do the insertion/update on the pfamA_interpro table [$@]";
	  last;
	}
      }
      
    close(_FILE);
      

    
  }
  
  return $rows;
  $self->close_transaction( $error );
}



sub delete_pfamB_database_links {



}


sub upload_pfamB_database_links {
  my ($self, $family_desc_file) = @_;

  my ($dbh, $auto_pfamA);

 
  $dbh = $self->open_transaction('pfamA', 'pfamA_database_links');

 
  my %store_acc;

  my $st = $dbh->prepare("select auto_pfamA , pfamA_acc from pfamA");
  $st->execute();
  my ($auto, $acc);

  while ( ($auto, $acc) = $st->fetchrow) {
    $store_acc{$acc} = $auto;
  }
  

 
  open(_DESC, "$family_desc_file") or print "CANNNA OPEN $family_desc_file as $! \n";;
  while(<_DESC>) {

    if ($_ =~ /^AC\s+(\w+)/) {
      
      my $pfam_acc = $1;
      
      $auto_pfamA = "";
      $auto_pfamA = $store_acc{$pfam_acc};
      
      
    }
    
    
    if ($_ =~ /^DR\s+PFAMB;\s+(\w+);/) {
      my $pfamB_acc = $1;
      my $empty = "";
      
      
      my $st = $dbh->prepare("insert into pfamA_database_links values('$auto_pfamA', 'PFAMB', '$empty', '$pfamB_acc' , '$empty' )");
      
      $st->execute();
      $st->finish();
      
    } 


  }
  close(_DESC) or print "CANNNA CLOSE $family_desc_file as $! \n";
}


sub upload_tree_information {

  my($self, $family) = @_;
  my $error;
  my ($dbh,
      @regions );
   
  $dbh = $self->open_transaction('pfamA', 'pfamA_reg_full', 'pfamseq');
  
  chop($family)   if ($family =~ /\/$/);
  open(_FAMILY, "$family/ALIGN.ann") or print "canna open the family ALIGN.ann file for : $family $! \n";
  
  my $got_gs = 0;
  my $count = 1;
  my $auto_pfamA;
  my $no_auto = 1;
  my $acc;
  my %store_pfamseq;
  
  
  ###### UPDATE TO DELETE TREE BEFORE ADDS NEXT LOT ! 
  while(<_FAMILY>) {
 
    $got_gs  = 1 if ($_ =~ /^\#\=GS/);
    $acc = $1 if ($_ =~ /^\#\=GF\s+AC\s+(\S+)/);
    
    if ( ($acc) && ($no_auto) ) {
      eval {
	
	####### AUTO NUM 
	my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_acc = '$acc'");
	$st->execute();
	($auto_pfamA) = $st->fetchrow;
	$st->finish();
	
	### RESET ALL TREE ORDER To NULL
	eval {
	  my $reset_tree;
	  my $st = $dbh->prepare("update pfamA_reg_full set tree_order = '$reset_tree' where auto_pfamA = '$auto_pfamA'");
	  $st->execute();
	  $st->finish();
	  
	  
	};
	
      };
      $no_auto = 0;
      
      ### RESET THE COUNT !
      
    }
    
    
    if ( ($got_gs) && ($_ =~ /^\#\=/) ) {
      my $temp_id = $1 if ($_ =~ /^\#\=GS\s+(\S+)\s+\S+/);
      my($id, $residues) = split(/\//, $temp_id);
      my($start, $end) = split(/-/, $residues);
     
      my $auto_pfamseq;
      if (!defined($store_pfamseq{$id})) {
	eval {
	  
	  ####### AUTO NUM 
	  my $st = $dbh->prepare("select auto_pfamseq from pfamseq where pfamseq_id = '$id'");
	  $st->execute();
	  ($auto_pfamseq) = $st->fetchrow;
	  
	  $st->finish();
	};
	
	$store_pfamseq{$id} = $auto_pfamseq;
	
      } else {
	$auto_pfamseq = $store_pfamseq{$id};
      }
      
      my $st = $dbh->prepare("update pfamA_reg_full set tree_order = '$count' where auto_pfamA = '$auto_pfamA' and auto_pfamseq = '$auto_pfamseq' and seq_start = '$start' and seq_end = '$end' and in_full = 1");

      $st->execute();
      $st->finish();

    

      $count++;
    }
    
  }
 
  close(_FAMILY);
  




#  #####  num full & num seed to pfamA table


  my $seed = $family . "/SEED.ann";


 open(_SEED, $seed) || $self->throw("For E entry object, got no valid directory for SEED.ann $family ($seed) $!");


  my ($num_seed, $num_full);
  while(<_SEED>) {
    $_ =~ /^\#\=GF\s+SQ\s+(.*)/ && do {  
      $num_seed = $1;
    };
    last if ( $num_seed);
    


  }

  close(_SEED);

  open(_FULL,"$family/ALIGN.ann") || $self->throw("For entry object, got no valid directory for ALIGN.ann $family $!");

   while(<_FULL>) {

     $_ =~ /^\#\=GF\s+SQ\s+(.*)/ && do {   
       $num_full = $1;
     };
     last if ( $num_full);
    


  }

   close(_SEED);


  my $st = $dbh->prepare("update pfamA set num_seed = '$num_seed', num_full = '$num_full'  where auto_pfamA = '$auto_pfamA' ");

  $st->execute();
  $st->finish();


  $self->close_transaction( $error );
}



sub delete_pfamA_seed_full {


  my($self, $acc) = @_;
  my $error;
  my ($dbh,
      @regions, $auto_pfamA );
   
  $dbh = $self->open_transaction('pfamA', 'pfamA_reg_full', 'pfamA_reg_seed');
  ####### AUTO NUM 

  eval {
    my $st = $dbh->prepare("select auto_pfamA from pfamA where pfamA_acc = '$acc' ;");
    $st->execute();
    ($auto_pfamA) = $st->fetchrow;
    $st->finish();
  };


  if ($auto_pfamA) {
    $dbh->do("delete from pfamA_reg_full where auto_pfamA = '$auto_pfamA'");
    $dbh->do("delete from pfamA_reg_seed where auto_pfamA = '$auto_pfamA'");
  
  } else {
    $self->throw("Couldnt delete pfamA_reg_full/pfamA_reg_seed as no auto_pfamA for $acc ! \n");
  }
  
  
  $self->close_transaction( $error );


}

1;
