
#
# BioPerl module for Bio::Pfam::AnnotatedRegion
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::AnnotatedRegion - Object for the annotation of a specified region of a sequence

=head1 SYNOPSIS

Derive from this class to create a new kind of AnnotatedRegion. The three
core fields are -FROM, -TO, and -ANNOTATION (containing the start and end residue numbers
and a reference to an Annotation object):

    use Bio::Pfam::AnnotatedRegion;

    $reg = Bio::Pfam::AnnotatedRegion->new('-FROM' => $from,
					   '-TO' => $to,
					   '-ANNOTATION' => $annotRef );


=head1 DESCRIPTION

This object is the superclass of several different types of AnnotatedRegion,
including PfamRegion. The generic AnnotatedSequence class (which
encapsulates a sequence together with its annotation) will contain a list of
instances of AnnotatedRegion derivatives.

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::AnnotatedRegion;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

# Object preamble - inherits from Bio::Root::Object

use Bio::Root::Object;


@ISA = qw(Bio::Root::Object Exporter);
#@EXPORT_OK = qw();
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params) = @_;
  my( $start, $end, $annot ) = (
     ( $params{'-FROM'}||$params{'-from'} ), ( $params{'-TO'}||$params{'to'} ),
     ( $params{'-ANNOTATION'}||$params{'-annotation'} ) );

  my $make = $self->SUPER::_initialize;

  $self->{ 'reg_from' } = $start;
  $self->{ 'reg_to' } = $end;
  $self->{ 'reg_annotation' } = $annot;

  return $make; # success - we hope!
}



=head2 annotation

 Title   : annotation
 Usage   : 
    $annReg->annotation(); # or ...
    $annReg->annotation( 123 );
 Function: For setting and getting the ANNOTATION field in the object

=cut

sub annotation{
   my ($self,$value) = @_;

   if (defined $value) {
       $self->{'reg_annotation'} = $value;
   }
   return $self->{'reg_annotation'};
}



=head2 from

 Title   : from
 Usage   : 
    $annReg->from(); # or ...
    $annReg->from( 123 );
 Function: For setting and getting the FROM field in the object

=cut

sub from{
   my ($self, $value) = @_;


   if (defined $value) {
       $self->{'reg_from'} = $value;
   }
   return $self->{'reg_from'};
}



=head2 to

 Title   : to
 Usage   : 
    $annReg->to(); # or ...
    $annReg->to( 123 );
 Function: For setting and getting the TO field in the object

=cut

sub to{
   my ($self, $value) = @_;


   if (defined $value) {
       $self->{'reg_to'} = $value;
   }
   return $self->{'reg_to'};
}



=head2 write

 Title   : write
 Usage   : $annSeq->write( \*STDOUT );
 Function: Spits out the contents of the object in standard fashion to the 
    given file handle
 Returns : 
 Args    : The output file handle
 Notes   :
    Developers may wish to over-ride this method in sub-classes. However, if this implementation
    is to be called via SUPER, then any extra fiels in the sub-class should be printed first,
    as this function adds a new line to the output (i.e. the SUPER call should be the last line
    in the over-ridden function).
=cut

sub write{
   my ($self, $file) = @_;

   print $file $self->from().":".$self->to().":";
   if (defined $self->annotation() ) {
       print $file $self->annotation()->description();
   }
   print $file "\n";

}
