
#
# BioPerl module for Bio::Pfam::Annotation
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::Annotation - A generic object for annotations

=head1 SYNOPSIS

    $annotRef = Bio::Pfam::Annotation->new();
    $annotRef->annotationText( "Hello" );

    # or ...
    $annotRef = Bio::Pfam::Annotation->new( '-DESCRIPTION' => "hello" );

=head1 DESCRIPTION

This object can be used to represent protein, family, and sub-family annotations.
At the moment, it just contains a string. 

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::Annotation;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

# Object preamble - inherits from Bio::Root::Object

use Bio::Root::Object;
use Bio::Pfam::Annotation::Comment;

@ISA = qw(Bio::Root::Object Exporter);
#@EXPORT_OK = qw();
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params) = @_;
  my( $text ) = ( $params{'-DESCRIPTION'}||$params{'-description'} );

  my $make = $self->SUPER::_initialize;

  $self->{ 'description' } = $text;
  $self->{ 'refs' } = [];
  $self->{ 'comment' } = [];
  $self->{ 'link' } = [];
  $self->{'flatcomment'} = Bio::Pfam::Annotation::Comment->new();
  return $make; # success - we hope!
}



=head2 annotationText

 Title   : annotationText
 Usage   :
    $string = $annot->annotationText();

    $annot->annotationText( "A new Annotation" );

 Function: Simply a way to set or get the annotation string of this annotation object

=cut

sub annotationText{
   my ($self, $value) = @_;

   $self->warn("Deprecated use of annotationText - use description instead");
   if ( defined $value ) {
       $self->{ 'desc' } = $value;
   }

   return $self->{ 'desc' };
}


=head2 description

 Title   : description
 Usage   : $obj->description($newval)
 Function: 
 Example : 
 Returns : value of description
 Args    : newvalue (optional)


=cut

sub description{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'description'} = $value;
    }
    return $self->{'description'};

}

=head2 add_reference

 Title   : add_reference
 Usage   : $self->add_reference($ref)
 Function: adds a reference object
 Example :
 Returns : 
 Args    :


=cut

sub add_reference{
   my ($self,$ref) = @_;
   push(@{$self->{'refs'}},$ref);
}

=head2 each_reference

 Title   : each_reference
 Usage   : foreach $ref ( $self->each_reference() )
 Function: gets an array of reference of selfects
 Example :
 Returns : 
 Args    :


=cut

sub each_reference{
   my ($self,@args) = @_;
   
   return @{$self->{'refs'}}; 
}




=head2 add_comment

 Title   : add_comment
 Usage   : $self->add_comment($ref)
 Function: adds a comment object
 Example :
 Returns : 
 Args    :


=cut

sub add_comment{
   my ($self,$com) = @_;
   push(@{$self->{'comment'}},$com);
}

=head2 each_comment

 Title   : each_comment
 Usage   : foreach $ref ( $self->each_comment() )
 Function: gets an array of comment of objects
 Example :
 Returns : 
 Args    :


=cut

sub each_comment{
   my ($self) = @_;
   
   return @{$self->{'comment'}}; 
}


=head2 add_link

 Title   : add_link
 Usage   : $self->add_link($ref)
 Function: adds a link object
 Example :
 Returns : 
 Args    :


=cut

sub add_link{
   my ($self,$com) = @_;
   push(@{$self->{'link'}},$com);
}

=head2 each_link

 Title   : each_link
 Usage   : foreach $ref ( $self->each_link() )
 Function: gets an array of link of objects
 Example :
 Returns : 
 Args    :


=cut

sub each_link{
   my ($self) = @_;
   
   return @{$self->{'link'}}; 
}

=head2 flatcomment

 Title   : flatcomment
 Usage   : $obj->flatcomment($newval)
 Function: 
 Example : 
 Returns : value of flatcomment
 Args    : newvalue (optional)


=cut

sub flatcomment{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'flatcomment'} = $value;
    }
    return $self->{'flatcomment'};

}






