
#
# BioPerl module for Bio::Pfam::DB_RCS
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::DB_RCS - The RCS version of Pfam::DB

=head1 SYNOPSIS

This is a concrete implementation of the abstract database object
found in Bio::Pfam::DB

=head1 DESCRIPTION

Describe the object here

=head1 CONTACT

Describe contact details here

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::DB_RCS;
use vars qw($AUTOLOAD @ISA);
use strict;
use Fcntl;
use DB_File;
use Bio::Seq;
use FileHandle;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Root::Object;
use Bio::Pfam::Entry_RCS;

my $DEAD_FAMILY_STRING = '__DEAD_FAMILY__';

@ISA = qw(Bio::Pfam::DB);
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self,@args) = @_;
  my(%testtie);
  my $make = $self->SUPER::_initialize(@args);

  my($current_dir,$index_file,$lock_file,$srs,$fastadb) = $self->_rearrange([qw(CURRENT
						       INDEX
						       LOCK_FILE
						       SRSDB
						       FASTADB
						       )],@args);
 
  if( ! -d $current_dir ) {
      $self->throw("$current_dir is not a directory and therefore cannot be made into a RCS based CURRENT db");
  }
  if( tie(%testtie,'DB_File',$index_file,O_RDONLY,0666) == 0){
      $self->throw("Could not open $index_file as a DB_file [$!] [$?]");
  }
  if(! $lock_file =~ /\w/ ) {
      $self->throw("No lock file provided!");
  }

  untie(%testtie);

  # ok - ready to rock

  $self->{'rcs_current'} = $current_dir;
  $self->{'rcs_index'} = $index_file;
  $self->{'rcs_lock_file'} = $lock_file;
  $self->{'rcs_srsdb'} = $srs;
  $self->{'underlying_fasta_db'} = $fastadb;
  
  # set stuff in self from @args
  return $make; # success - we hope!
}


=head2 id2acc

 Title   : id2acc
 Usage   : $acc = $db->id2acc("pkinase"); 
 Function: converts an id to an accession. Only works for
	   current ids - does not work for previous ids
 Returns : string of the accession, or undef if no accession
 Throws  : Exceptions thrown if this $id does not exist or due to RCS misconfiguration
 Args    :

=cut

sub id2acc{
   my ($self,$id) = @_;
   
   my ($accmap,%idmap,$acc,$name);

   my $index_file = $self->{'rcs_index'};

   if( $id =~ /$DEAD_FAMILY_STRING/ ) {
     $self->throw("Id $DEAD_FAMILY_STRING indicates that this family was killed.");
   }

   $accmap = $self->_get_tied_accmap();

   %idmap = reverse %{$accmap};

   $acc = $idmap{$id};
   if( !defined $acc) {
     $self->throw("Id $id does not exist in $index_file");
   }

   return $acc;
}

=head2 acc2id

 Title   : acc2id
 Usage   : $id = $db->acc2id("PF00001");
 Function: converts an accession to an id
 Returns : the id, throws an exception if no accession, or dead accession
 Args    :

=cut

sub acc2id{
   my ($self,$acc) = @_;
   my ($accmap,$id,$name);
   my $index_file = $self->{'rcs_index'};
   #open the index file for Reading, get out the hash and return it.


   $accmap = $self->_get_tied_accmap();

   $id = $accmap->{$acc};
   if( !defined $id ) {
       $self->throw("Accession $acc does not exist in $index_file");
   }

   if( $id =~ /$DEAD_FAMILY_STRING/ ) {
     $self->throw("accession $acc has been killed");
   }

   return $id;
}

=head2 is_dead_acc

 Title   : is_dead_acc
 Usage   : if( $db->is_dead_acc('PF00034') ) { print "Is a dead accession" }
 Example :
 Returns : 1 if dead, undef if not
 Args    : accesion number


=cut

sub is_dead_acc{
   my ($self,$acc) = @_;
   my ($accmap,$id,$name);
   my $index_file = $self->{'rcs_index'};
   #open the index file for Reading, get out the hash and return it.


   $accmap = $self->_get_tied_accmap();

   $id = $accmap->{$acc};
   if( !defined $id ) {
       $self->throw("Accession $acc does not exist in $index_file");
   }

   if( $id =~ m/$DEAD_FAMILY_STRING/ ) {
      return 1;
   } else {
    return undef;
   }
}

=head2 get_Entry_by_id

 Title   : get_Entry_by_id
 Usage   : $en = $db->get_Entry_by_id('pkinase');
 Function: get a Bio::Pfam::Entry object by id
 Returns : Bio::Pfam::Entry object
 Args    : string of the id of the family


=cut

sub get_Entry_by_id{
   my ($self,$id) = @_;
   my ($entry,$currentdir);

   # ok - load up entry object directory
   $currentdir = $self->{'rcs_current'};

   if( ! -d  "$currentdir/$id" ) {
       $self->throw("$id is not a valid id for this Pfam database");
   }
   
   open(_DESC,"$currentdir/$id/DESC") || $self->throw(" [$currentdir/$id/DESC] does not exist, despite family directory existing $!");
   $entry = Bio::Pfam::Entry_RCS->new();
   $entry->_read_std_desc(\*_DESC);
   
   close(_DESC);
   $entry->id($id);
   $entry->_directory("$currentdir/$id");
   
   return $entry;
}

=head2 get_Entry_by_acc

 Title   : get_Entry_by_acc
 Usage   : $en = $db->get_Entry_by_acc('PF00076');
 Function: get a Bio::Pfam::Entry object by acc
 Returns : Bio::Pfam::Entry object
 Args    : string of the accession number of the family


=cut

sub get_Entry_by_acc{
   my ($self,$acc) = @_;
   my ($entry,$id,$currentdir);

   $id = $self->acc2id($acc);
   # ok - load up entry object directory
   $currentdir = $self->{'rcs_current'};

   if( ! -d  "$currentdir/$id" ) {
       $self->throw("$id is not a valid id for this Pfam database");
   }
   
   open(_DESC,"$currentdir/$id/DESC") || $self->throw(" [$currentdir/$id/DESC] does not exist, despite family directory existing $!");
   $entry = Bio::Pfam::Entry_RCS->new();
   $entry->_read_std_desc(\*_DESC);
   close(_DESC);
   $entry->id($id);
   $entry->_directory("$currentdir/$id");
   
   return $entry;

}

=head2 get_allacc

 Title   : get_allacc
 Usage   : @list = $db->getallacc('ALPHA');
 Function: Gets a list of all accession numbers in the current db
 Returns : a list of accession numbers
 Args    : Either 'ALPHA' or 'DATE' for the sort
           by alphabetical order of id or date.

=cut

sub get_allacc{
   my ($self,$type) = @_;
   my ($accmap,@acc,@tempacc,$acc,$name);
   my $index_file = $self->{'rcs_index'};

   if( !defined $type ) {
       $type = 'ALPHA';
   }

   #open the index file for Reading, get out the hash and return it.

   $accmap = $self->_get_tied_accmap();

   @acc = keys %{$accmap};

   # remove everything with __DEAD_FAMILY__

   foreach $acc ( @acc ) {
     if( $accmap->{$acc} =~ /$DEAD_FAMILY_STRING/ ) {
       next;
     }
     push(@tempacc,$acc);
   }
   @acc = @tempacc;

   # now sort it etc.

   SWITCH : {
       $type =~ /ALPHA/ && do {
	   @acc = sort { lc($accmap->{$a}) cmp lc($accmap->{$b}) } @acc;
	   last SWITCH;
       };
       $type =~ /DATE/ && do {
	   @acc = sort {my ($aa,$bb); $a =~ /^PF+0+(\d+)$/; $aa = $1; $b =~ /^PF+0+(\d+)$/; $bb = $1; return $aa <=> $bb; } @acc;
	   last SWITCH;
       };
       $self->throw("No proper type for this $type");
   }

   return @acc;
}

=head2 get_Seq_pfamseq

 Title   : get_Seq_pfamseq($id)
 Usage   : $seq = $db->get_Seq_pfamseq($id)
 Function: gets a Bio::Seq object from the underlying database
 Returns : a newly made Bio::Seq object
 Args    :


=cut

sub get_Seq_pfamseq{
   my ($self,$input_id) = @_;
   my ($temp,$id,$seq,$fh,$srsdb,$out);

   # read into $id and $seq;

   $srsdb = $self->{'rcs_srsdb'};

   $seq = "";
   $fh = new FileHandle;
   if( $fh->open("getz -d -sf fasta '[$srsdb-id:$input_id]' |") == 0) {
       $self->throw("Cannot fork getz process off");
   }
   $temp = <$fh>;
   chomp $temp;
   if( $temp =~ /^>(\S+)\s/ ) {
       $self->throw("[$temp] does not look like a fasta line");
   }
   $id = $1;

   while(<$fh>) {
       /^>/ && $self->throw("Request $id gave multiple sequences in $srsdb. Problem!");
       s/\W//g;
       $seq .= $_;
   }
   if( $fh->close() == 0 ) {
       $self->throw("Getz pipe failed to close");
   }

   $out = Bio::Seq->new(-seq=>$seq,-id=>$id);

   return $out;
}

=head2 pfamseq_stats

 Title   : pfamseq_stats
 Usage   : ($residues,$sequences) = $db->pfamseq_stats();
 Function: Calculate statistics for underlying pfamseq database
           
 Returns : An array of number of residues and sequences
         : in pfamseq
 Args    :


=cut

sub pfamseq_stats {
  my ($self) = @_;
  my ($residues,$sequences);
  my($file)=$self->_get_fasta_pfamseq();
  # Use seqstat to make stats!
   my ($nseq,$fromlen,$tolen,$status,$avlen);
  open (TMP,"seqstat $file |");
  while (<TMP>) {
    if (/^Total \# residues:\s+(\d+)/) {$residues = $1;}
    if (/^Number of sequences:\s+(\d+)/) {$sequences = $1;}
  }
  close(TMP) || $self->throw("Couldn't close pipe");
  return ($residues,$sequences);
}


=head2 get_sfr_file

 Title   : get_sfr_file
 Usage   : $file = $db->get_sfr_file( $accession )
 Function: Parses the bait file for the given accesison number and returns a filled
    SFRObject
 Returns : A ref to a filehandle glob
 Args    : A Pfam Accession number

=cut

sub get_sfr_file{
   my ($self, $accession) = @_;
       
   my $fh = FileHandle->new();

   $fh->open( "<".$self->{'rcs_current'}."/".$self->acc2id( $accession )."/BAIT") or return undef;

   return $fh;    
}


=head2 _get_fasta_pfamseq

 Title   : get_fasta_pfamseq
 Usage   : $filename = $db->_get_fasta_pfamseq()
 Function: returns the filename (full path) of pfamseq, the 
           underlying sequence database of pfam
 Returns : a string of the filename
 Args    :


=cut

sub _get_fasta_pfamseq{
   my ($self) = @_;

   return $self->{'underlying_fasta_db'};
}



=head2 _add_accession

 Title   : _add_accession
 Usage   : $self->_add_accession('PF001002','newfamilyname')
 Function: adds an accession. Assummes you have the lock
 Example : 
 Returns : 
 Args    :


=cut

sub _add_accession{
   my ($self,$acc,$family) = @_;
   my ($obj,%accmap);
   if( $self->{'is_locked'} != 1 ) {
       $self->throw("Have not locked the database - cannot add an accession!");
   }

   # now get and tie the hash, saving the object
   
   if( ($obj = tie(%accmap,'DB_File',$self->{'rcs_index'},O_RDWR,0666)) == 0){
       $self->throw("Could not open $self->{'rcs_index'} as a DB_file $! Leaving the lock on");
       
   }
   if( exists $accmap{$acc} ) {
       $self->throw("$acc already exists in the database");
   }

   $accmap{$acc} = $family;
   $obj->sync();
   untie(%accmap);
}

=head2 _move_accession

 Title   : _move_accession
 Usage   : $self->_move_accession('PF001002','newname')
 Function: Moves an accession *only in the db file*, not in 
           the underlying database. Assummes you have the lock
 Example :
 Returns : 
 Args    :


=cut

sub _move_accession{
   my ($self,$accession,$to) = @_;
   my ($obj,%accmap);
   if( $self->{'is_locked'} != 1 ) {
       $self->throw("Have not locked the database - cannot add an accession!");
   }

   # now get and tie the hash, saving the object
   
   if( ($obj = tie(%accmap,'DB_File',$self->{'rcs_index'},O_RDWR,0666)) == 0){
       $self->throw("Could not open $self->{'rcs_index'} as a DB_file $! Leaving the lock on");
       
   }
   if( !exists $accmap{$accession} ) {
       $self->throw("$accession does not exist in db!");
   }

   $accmap{$accession} = $to;
   $obj->sync();
   untie(%accmap);
}

=head2 _kill_accession

 Title   : _kill_accession
 Usage   : $self->_kill_accession('PF001002')
 Function: Kills an accession *only in the db file*, not in 
           the underlying database. Assummes you have the lock
 Example :
 Returns : 
 Args    :


=cut

sub _kill_accession{
   my ($self,$accession) = @_;
   my ($obj,%accmap);
   if( $self->{'is_locked'} != 1 ) {
       $self->throw("Have not locked the database - cannot kill a accession!");
   }

   # now get and tie the hash, saving the object
   
   if( ($obj = tie(%accmap,'DB_File',$self->{'rcs_index'},O_RDWR,0666)) == 0){
       $self->throw("Could not open $self->{'rcs_index'} as a DB_file $! Leaving the lock on");
       
   }
   if( !exists $accmap{$accession} ) {
       $self->throw("$accession does not exist in db!");
   }

   $accmap{$accession} = $DEAD_FAMILY_STRING;
   $obj->sync();
   $obj = undef;
   untie(%accmap);
}


=head2 _get_lock

 Title   : _get_lock
 Usage   : if( ($lockee = $db->_get_lock()) ) {
               $db->throw("Lock already got by $lockee");
           }
 Function:
 Example :
 Returns : undef for success, or lockee for failure
 Args    :


=cut

sub _get_lock{
   my ($self) = @_;
   my ($line,$name,$obj,%accmap,$index_file);


   my ($me) = `whoami`;
   chomp($me);


   if( !-e $self->{'rcs_lock_file'} ) {
       # get the lock now!

       open(_LOCK_,">$self->{'rcs_lock_file'}") || die "Could not open lock file $!";
       print _LOCK_ "$me has locked the database\n";
       close(_LOCK_);
       $self->{'is_locked'} = 1;

       return undef;
   } else {
       open(_LOCK_,"$self->{'rcs_lock_file'}") || die "Could not open lock file $!";
       $line = <_LOCK_>;
       $line =~ /(\S+)/;
       $name = $1;
       close(_LOCK_);
       return $name;
   }
}

=head2 _unlock

 Title   : _unlock
 Usage   : $self->_unlock();
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _unlock{
   my ($self,@args) = @_;
   my ($key);

   if( $self->{'is_locked'} != 1 ) {
       $self->throw("Trying to release the lock when you haven't got it!");
   }

   if( ! -e $self->{'rcs_lock_file'} ) {
       $self->throw("Bad error - you have the lock but there is no lock file!");
   }

   unlink($self->{'rcs_lock_file'});
   $self->{'is_locked'} = 0;

   return 0;
}

=head2 _get_tied_accmap

 Title   : _get_tied_accmap
 Usage   : $refhash = $db->_get_tied_accmap()
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _get_tied_accmap{
   my ($self) = @_;
   my ($obj,%accmap,$out,$temp);

   if( defined $self->_accmap_cache() ) {
     return $self->_accmap_cache();
   }


   # now get and tie the hash, saving the object
   
   if( tie(%accmap,'DB_File',$self->{'rcs_index'},O_RDONLY,0666) == 0){
       $self->throw("Could not open $self->{'rcs_index'} as a DB_file $! Leaving the lock on");
   }
   $out = {};
   %{$out} = %accmap;
   untie(%accmap);
   $self->_accmap_cache($out);

   return $out;
}

=head2 _accmap_cache

 Title   : _accmap_cache
 Usage   : $obj->_accmap_cache($newval)
 Function: 
 Example : 
 Returns : value of _accmap_cache
 Args    : newvalue (optional)


=cut

sub _accmap_cache{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'_accmap_cache'} = $value;
    }
    return $self->{'_accmap_cache'};

}





