
#
# BioPerl module for Bio::Pfam::DB_RCS
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::DB_Web - The Web version of Pfam::DB

=head1 SYNOPSIS

This is a concrete implementation of the abstract database object
found in Bio::Pfam::DB

=head1 DESCRIPTION

Describe the object here

=head1 CONTACT

pfam@sanger.ac.uk

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::DB_Web;
use vars qw(@ISA);
use strict;
use Bio::Seq;
use FileHandle;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Pfam::DB;
use Bio::Pfam::Entry_Web;


@ISA = qw(Bio::Pfam::DB);
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params) = @_;

  my ($srspfamseq, $srspfam,$datadir) = 
      (($params{'-pfamseqdb'}||$params{'-PFAMSEQDB'}),
       ($params{'-pfamdb'}||$params{'-PFAMDB'}) ,
       ($params{'-datadir'} || $params{'-DATADIR'})
       );

  my $make = $self->SUPER::_initialize;

  $self->{'web_srspfamseq'} = $srspfamseq;
  $self->{'web_srspfam'} = $srspfam;
  $self->{'datadir'} = $datadir;

  # set stuff in self from @args
  return $make; # success - we hope!
}


=head2 id2acc

 Title   : id2acc
 Usage   : $acc = $db->id2acc("pkinase"); 
 Function: converts an id to an accession. Only works for
	   current ids - does not work for previous ids
 Returns : string of the accession, or undef if no accession
 Throws  : Exceptions thrown if this $id does not exist or due to RCS misconfiguration
 Args    :
 Notes   : USING GETZ IS A TEMPORARY SOLUTION TO THIS PROBLEM

=cut

sub id2acc{
   my ($self,$id) = @_;
   my ($srspfam, $fh, $acc, $temp);

   my $srsdb = $self->{'web_srspfam'};
   $fh = new FileHandle;

   if( $fh->open("getz -f 'acc' '[$srsdb-id:$id]' |") == 0) {
       $self->throw("Cannot fork getz process off");
   }
   $temp = <$fh>;

   if( $temp =~ /^AC\s+(\S+)/ ) {
       $acc = $1;
   }
   else {
       $self->throw("No Pfam family with that id");
   }
   if( $fh->close() == 0 ) {
       $self->throw("Getz pipe failed to close");
   }

   return $acc;
}

=head2 acc2id

 Title   : acc2id
 Usage   : $id = $db->acc2id("PF00001");
 Function: converts an accession to an id
 Returns : the id, or undef if no id for that accession
 Args    :
 Notes   : USING GETZ IS A TEMPORARY SOLUTION TO THIS PROBLEM

=cut

sub acc2id{
   my ($self,$acc) = @_;
   my ($srspfam, $fh, $id, $temp);

   $srspfam = $self->{'web_srspfam'};
   $fh = new FileHandle;

   if( $fh->open("getz -f 'id' '[$srspfam-acc:$acc]' |") == 0) {
       $self->throw("Cannot fork getz process off");
   }
   $temp = <$fh>;

   if( $temp =~ /^ID\s+(\S+)/ ) {
       $id = $1;
   }
   else {
       $self->throw("No Pfam family with that accession");
   }
   if( $fh->close() == 0 ) {
       $self->throw("Getz pipe failed to close");
   }

   return $id;


}

=head2 get_allacc

 Title   : get_allacc
 Usage   : @list = $db->getallacc('ALPHA');
 Function: Gets a list of all accession numbers in the current db
 Returns : a list of accession numbers
 Args    : Either 'ALPHA' or 'DATE' for the sort
           by alphabetical order of id or date.
=cut

sub get_allacc{
    my ($self, @args) = @_;
    my ($dir,@files,@out,$acc);

    $dir = $self->{'datadir'};
    opendir(_DIR,"$dir/desc") || $self->throw("Could not open data directory $dir/desc $!");
    @files = readdir(_DIR);
    closedir(_DIR);

    foreach $acc ( @files ) {
	$acc =~ /^PF/ || next;
	$acc =~ s/\..*$//g;
	push(@out,$acc);
    }

    return @out;

}


=head2 get_Seq_pfamseq

 Title   : get_Seq_pfamseq($id)
 Usage   : $seq = $db->get_Seq_pfamseq($id)
 Function: gets a Bio::Seq object from the underlying database
 Returns : a newly made Bio::Seq object
 Args    :


=cut

sub get_Seq_pfamseq{
   my ($self,$input_id) = @_;
   my ($temp,$id,$seq,$des,$fh,$srspfamseq);

   $srspfamseq = $self->{'web_srspfamseq'};

   $fh = new FileHandle;
   if( $fh->open("getz -f 'des' -f 'seq' -sf fasta '[$srspfamseq-id:$input_id]' |") == 0) {
       $self->throw("Cannot fork getz process off");
   }
   $seq = "";
   $des = "";
   $id = undef;

   while(<$fh>) {
       if ( $_ =~ /^DE\s+(\S+)/ ) {
	   $des .= $1;
       }
       elsif ( $_ =~ /^>(\S+)/ ) {
	   if (defined $id) {
	       $self->throw("Request $id gave multiple sequences in $srspfamseq. Problem!");
	   }
	   else {
	       $id = $1;
	   }
       }
       else {
	   s/\W//g;
	   $seq .= $_;
       }
   }

   if( $fh->close() == 0 ) {
       $self->throw("Getz pipe failed to close");
   }

   return  Bio::Seq->new(-seq=>$seq,-id=>$id,-desc=>$des);
}



=head2 get_sfr_file

 Title   : get_sfr_file
 Usage   : $file = $db->get_sfr_file( $accession )
 Function: Parses the bait file for the given accesison number and returns a filled
    SFRObject
 Returns : A ref to a filehandle glob
 Args    : A Pfam Accession number
 Notes   :
    This method has an extra argument not present in sibling classes or indeed the abstract
    superclass. This is because on the web, the system bait files are editable (the very 
    reason for a web interface to the bait system. 

=cut

sub get_sfr_file{
   my ($self, $accession, $write) = @_;
       
   return PfamWWWConfig::bait_FH_by_acc( $accession, $write );    
}


=head2 get_Entry_by_id

 Title   : get_Entry_by_id
 Usage   : $en = $db->get_Entry_by_id('pkinase');
 Function: get a Bio::Pfam::Entry object by id
 Returns : Bio::Pfam::Entry object
 Args    : string of the id of the family


=cut

sub get_Entry_by_id{
   my ($self,$id) = @_;
   my ($dir,$out,$acc);

   $acc = $self->id2acc($id);

   $dir = $self->{'datadir'};
   open(_DESC,"$dir/desc/$acc.desc") || $self->throw("Could not open [$dir/desc/$acc.desc] as a web site description");
   $out = Bio::Pfam::Entry_Web->new();
   $out->_read_std_desc(\*_DESC);
   $out->_datadir($dir);
   close(DESC);
   return $out;
}

=head2 get_Entry_by_acc

 Title   : get_Entry_by_acc
 Usage   : $en = $db->get_Entry_by_acc('PF00076');
 Function: get a Bio::Pfam::Entry object by acc
 Returns : Bio::Pfam::Entry object
 Args    : string of the accession number of the family


=cut

sub get_Entry_by_acc{
   my ($self,$acc) = @_;
   my ($dir,$out);


   $dir = $self->{'datadir'};
   open(_DESC,"$dir/desc/$acc.desc") || $self->throw("Could not open [$dir/desc/$acc.desc] as a web site description");
   $out = Bio::Pfam::Entry_Web->new();
   $out->_read_std_desc(\*_DESC);
   $out->_datadir($dir);
   close(DESC);
   return $out;

}







