

#
# BioPerl module for Bio::Pfam::Entry
#
# Cared for by Ewan Birney <pfam@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::Entry - Abstract Class for entry

=head1 SYNOPSIS

  $en = $db->get_entry_by_name('pkinase');
  $acc = $en->acc();
  $full = $en->full();

=head1 DESCRIPTION

Entry is the abstract class for Pfam entries. Each database will supply
its own concrete class. The functions here therefore the things you
can expect from any database

=head1 CONTACT

contact pfam@sanger.ac.uk for problems with this module

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::Entry;
use vars qw($AUTOLOAD @ISA);
use strict;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Root::Object;
use Bio::Pfam::Annotation;
use Bio::Pfam::Annotation::Comment;
use Bio::Pfam::Annotation::Link;
use Bio::Pfam::Annotation::Reference;
use Bio::SimpleAlign;

@ISA = qw(Bio::Root::Object Exporter);
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self,@args) = @_;

  my $make = $self->SUPER::_initialize;
  $self->{'build'} = [];

  # set stuff in self from @args
  return $make; # success - we hope!
}


=head2 acc

 Title   : acc
 Usage   : $self->acc($newval)
 Function: 
 Example : 
 Returns : value of acc
 Args    : newvalue (optional)


=cut

sub acc{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'acc'} = $value;
    }
    return $self->{'acc'};

}

=head2 id

 Title   : id
 Usage   : $self->id($newval)
 Function: 
 Example : 
 Returns : value of id
 Args    : newvalue (optional)


=cut

sub id{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'id'} = $value;
    }
    return $self->{'id'};

}

=head2 author

 Title   : author
 Usage   : $self->author($newval)
 Function: 
 Example : 
 Returns : value of author
 Args    : newvalue (optional)


=cut

sub author{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'author'} = $value;
    }
    return $self->{'author'};

}

=head2 source

 Title   : source
 Usage   : $self->source($newval)
 Function: 
 Example : 
 Returns : value of source
 Args    : newvalue (optional)


=cut

sub source{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'source'} = $value;
    }
    return $self->{'source'};

}
=head2 alignmethod

 Title   : alignmethod
 Usage   : $self->alignmethod($newval)
 Function: 
 Example : 
 Returns : value of alignmethod
 Args    : newvalue (optional)


=cut

sub alignmethod{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'alignmethod'} = $value;
    }
    return $self->{'alignmethod'};

}

=head2 add_build_line

 Title   : add_build_line
 Usage   : $self->add_build_line($line)
 Function: adds another build line to the object
 Example :
 Returns : 
 Args    :


=cut

sub add_build_line{
   my ($self,$line) = @_;
   push(@{$self->{'build'}},$line);

}

=head2 each_build_line

 Title   : each_build_line
 Usage   : gives you an array of build lines
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub each_build_line{
   my ($self,@args) = @_;
   return @{$self->{'build'}};
}


=head2 gathering

 Title   : gathering
 Usage   : $self->gathering($newval)
 Function: 
 Example : 
 Returns : value of gathering
 Args    : newvalue (optional)


=cut

sub gathering{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'gathering'} = $value;
    }
    return $self->{'gathering'};

}

=head2 trusted_cutoff

 Title   : trusted_cutoff
 Usage   : $self->trusted_cutoff($newval)
 Function: 
 Example : 
 Returns : value of trusted_cutoff
 Args    : newvalue (optional)


=cut

sub trusted_cutoff{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'trusted_cutoff'} = $value;
    }
    return $self->{'trusted_cutoff'};

}

=head2 noise_cutoff

 Title   : noise_cutoff
 Usage   : $self->noise_cutoff($newval)
 Function: 
 Example : 
 Returns : value of noise_cutoff
 Args    : newvalue (optional)


=cut

sub noise_cutoff{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'noise_cutoff'} = $value;
    }
    return $self->{'noise_cutoff'};

}

=head2 ann

 Title   : ann
 Usage   : $self->ann($newval)
 Function: 
 Example : 
 Returns : Annotation object
 Args    : 


=cut

sub ann{
   my ($self,$value) = @_;

   $self->throw("Abstract method, Bio::Pfam::Entry::ann - should be filled by subclass");
     
}


=head2 seed

 Title   : seed
 Usage   : $seed = $self->seed()
 Function:
 Example :
 Returns : Bio::SimpleAlign object of the seed alignment
 Args    :


=cut

sub seed{
   my ($self,@args) = @_;

   $self->throw("Abstract method, Bio::Pfam::Entry::seed - should be filled by subclass");

}


=head2 full

 Title   : full
 Usage   : $full = $self->full()
 Function:
 Example :
 Returns : Bio::SimpleAlign object of the full alignment
 Args    :


=cut

sub full{
   my ($self,@args) = @_;

   $self->throw("Abstract method, Bio::Pfam::Entry::full - should be filled by subclass");

}

=head2 name_start_end

 Title   : 
 Usage   : @names = $self->name_start_end('FULL')
 Function:
 Example :
 Returns : a list of "name/start-end" for the alignment of choice 
 Args    :


=cut

sub name_start_end {
   my ($self,@args) = @_;

   $self->throw("Abstract method, Bio::Pfam::Entry::full - should be filled by subclass");

}


=head2 _write_std_desc

 Title   : _write_std_desc
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _write_std_desc{
   my ($self,$file) = @_;
   my ($link,$ref,$count,$line);
   
   print $file "ID   ", $self->id(), "\n";
   print $file "AC   ", $self->acc(), "\n";
   print $file "AU   ", $self->author(), "\n";
   print $file "SE   ", $self->source(), "\n";
   print $file "AL   ", $self->alignmethod(), "\n";
   $count = 1;
   foreach $ref ( $self->ann()->each_reference() ) {
       print $file "RN   [$count]\n";
       print $file "RM   ", $ref->medline(), "\n";
       print $file "RT   ", $ref->title(), "\n";
       print $file "RL   ", $ref->location(), "\n";
       $count++;
   }
   foreach $link ( $self->ann()->each_link() ) {
       print $file "DR   ", $link->database() ,"; ", $link->primary_id() ,";\n";
   }
   foreach $line ( $self->ann()->flatcomment()->each_flat() ) {
       print $file "CC   ", $line, "\n";
   }

}


=head2 _read_std_desc

 Title   : _read_std_desc
 Usage   : $self->_read_std_desc(\*FILE,$ann)
 Function: This reads a 'standard' Pfam desc file
           with two letter codes in

           If the second argument is filled, it assummes it is a new
           annotation object to be read into
 Example : 
 Returns : 
 Args    :


=cut

sub _read_std_desc{
   my ($self,$file,$ann) = @_;
   my($ref,$link,$justseenref);

   while(<$file>) {
       $justseenref = 0;

       /^ID\s+(\S+)/ && do {
	   $self->id($1);
       };
       /^AC\s+(PF\d+)/ && do {
	   $self->acc($1);
       };
       /^AU\s+(.*?)\s+$/ && do {
	   $self->author($1);
       };
       /^AL\s+(.*?)\s+$/ && do {
	   $self->alignmethod($1);
       };
       /^SE\s+(.*?)\s+$/ && do {
	   $self->source($1);
       };

       /^GA\s+(.*?)\s+$/ && do {
	   $self->gathering($1);
       };
       /^TC\s+(.*?)\s+$/ && do {
	   $self->trusted_cutoff($1);
       };
       /^NC\s+(.*?)\s+$/ && do {
	   $self->noise_cutoff($1);
       };

       # end of non annotation parsing.

       if( !defined $ann ) {
	 next;
       }


       /^DE\s+(.*?)\s+$/ && do {
	 $ann->description($1);
       };

       /^RN\s/ && do { 
	   $ref = Bio::Pfam::Annotation::Reference->new();
	   $justseenref = 1;
       };
       /^RM\s+(\d+)/ && do {
	   $ref->medline($1);
	   $justseenref = 1;
       };
       /^RA\s+(.*)/ && do {
	   $ref->authors($1);
	   $justseenref = 1;
       };
       /^RL\s+(.*)/ && do {
	   $ref->location($1);
	   $justseenref = 1;
       };
       /^RT\s+(.*)/ && do {
	   $ref->title($1);
	   $justseenref = 1;
       };
       
       if( $justseenref == 0 ) {
	   # put away reference if there
	   if( defined $ref ) {
	       $ann->add_reference($ref);
	       $ref = undef;
	   } else {
	       $ref = undef;
	   }
       }
       /^DR\s+(\S+);\s+(\S+);\s+(\S+);/ && do {
	   $link = Bio::Pfam::Annotation::Link->new();
	   $link->database($1);
	   $link->primary_id($2);
	   $link->optional_id($3);
	   $ann->add_link($link);
	   next;
       };
       /^DR\s+(\S+);\s+(\S+);/ && do {
	   $link = Bio::Pfam::Annotation::Link->new();
	   $link->database($1);
	   $link->primary_id($2);
	   $ann->add_link($link);
       };
       /^CC   (.*)/ && do {
	   $ann->flatcomment()->add_flat($1);
       };
	   
   }


}






