
#
# BioPerl module for Bio::Pfam::PfamAnnSeqFactory.pm
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::PfamAnnSeqFactory - This static class responds to requests from
clients for AnnotatedSequence objects filled with PfamRegions.

=head1 SYNOPSIS


    use Bio::Pfam::PfamAnnSeqFactory;

    $fac = Bio::Pfam::PfamAnnSeqFactory::instance( $eng );

    # Now call methods on the single instance


=head1 DESCRIPTION

This class is derived from Bio::Pfam::AnnotatedSequenceFactory. Its sole purpose is
to allow the creation and intialisation of an AnnotatedSequence initially containing
PfamRegions. Once the AnnotatedSequence has been created and returned, the client
is free to add other kinds of AnnotatdedRegion to the AnnotatedSequence. The single
argument, a ref. to an SFREngine, provides an interface to the underlying database

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::PfamAnnSeqFactory;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

use Bio::Pfam::AnnotatedSequenceFactory;
use Bio::Pfam::PfamRegion;
use Bio::Pfam::SFRRegion;
use Bio::Pfam::Annotation;
$Bio::Pfam::PfamAnnSeqFactory::theInstance = undef;

@ISA = qw(Bio::Pfam::AnnotatedSequenceFactory);

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self,$eng) = @_;

  $self->engine($eng); 

  my $make = $self->SUPER::_initialize;

# set stuff in self from @args
 return $make; # success - we hope!
}



=head2 addSwissPfamToAnnSeq

 Title   : addSwissPfamToAnnSeq
 Usage   : Bio::Pfam::AnnotatedSequenceFactory::addSwissPfamToAnnSeq( $annSeq, \*MYHANDLE, 1 );
 Function: Adds AnnotatedRegion records Fills by reading from MYHANDLE, assuming input in swisspfam format. 
    The sequence part of the object needs to be filled in separately. Here is an example of SwissPfam: 

>SRC_HUMAN        |=================================================| P12931 535 a.a.
Pfam-B_248       1                      ---                          (42) PD00248  237-268
Pfam-B_765       1 -------                                           (25) PD00765  1-85
SH2              1              --------                             (381) PF00017  Src homology domain 2  150-232
SH3              1        -----                                      (495) PF00018  Src homology domain 3  86-142
pkinase          1                         -----------------------   (2942) PF00069  Eukaryotic protein kinase domain  269-522

 Returns : An AnnotatedSequence object
 Args    : 
    Ref. to the input-source file handle, 
    Bool. which should be true when Pfam-B domains are required

 QUESTION: SwissPfam format is surely only relevant for PfamRegions. This raises an issue:
  Should it be declared in the superclass? If it is not meant to be over-ridden by all 
    sub-classes, why declare it in the superclass?

=cut

sub addSwissPfamToAnnSeq {
   my ($self, $annSeq, $file, $pfamBneeded) = @_;
   my ($domainName, $acc, $annotation, $interval, $remainder );

   while (<$file>) {
       if (/^>(\S+)/) {
	   # we have found the id line.
	   $annSeq->id( $1 );
       }
       else {
	   # we must have a domain entry line
	   if (/^(\S+)\s+\d+.*\(\d+\)\s+(\w+\d+)\s+(.*?)(?=\s+\d+-\d+)/) {
	       $domainName = $1;
	       $acc = $2;
	       $annotation = $3;
	       $remainder = $';
	       if ($acc !~ /^PD/ || $pfamBneeded ) {
		   foreach $interval ( split /\s+/, $remainder ) { 
		       if ($interval =~ /(\d+)-(\d+)/) {
			   $annSeq->addAnnotatedRegion( 
			       Bio::Pfam::PfamRegion->new('-PFAM_ACCESSION' => $acc,
							  '-PFAM_ID' => $domainName,
							  '-FROM' => $1, 
							  '-TO' => $2,
							  '-ANNOTATION' => Bio::Pfam::Annotation->
							      new('-DESCRIPTION' => $annotation)
							  )
						       );
						      
		       }		       
	           }
	       }
	   }   
       }
   }
}




=head2 addHMMResultsToAnnSeq

 Title   : addHMMResultsToAnnSeq
 Usage   : Bio::Pfam::AnnotatedSequenceFactory::addSwissPfamToAnnSeq( $annSeq, $hmmRes );
 Function: Adds AnnotatedRegion records to the given AnnotatedSequence by reading the
    contents of the HMMResults object
 Returns : 
 Args    : AnnotatedSequence ref, HMMResults ref

=cut

sub addHMMResultsToAnnSeq{
   my ($self, $annSeq, $hmmResRef) = @_;
   my %annotations;

   foreach my $unit ($hmmResRef->eachHMMUnit()) {
       my $accession = $unit->hmmacc();
       my $annot;
       
       # need to create an SFREngine to access the underlying database; no other reason
       if ($annotations{ $accession } ) {
	   $annot = $annotations{ $accession };
       }
       else {
	   $annot = $self->engine()->underlyingDatabase()->get_Entry_by_acc($accession)->ann();
	   $annotations{ $accession } = $annot;
       }
       $annSeq->addAnnotatedRegion( Bio::Pfam::PfamRegion->new('-PFAM_ACCESSION' => $unit->hmmacc(),
							       '-PFAM_ID' => $unit->hmmname(),
							       '-FROM' => $unit->start_seq(),
							       '-TO' => $unit->end_seq(),
							       '-ANNOTATION' => Bio::Pfam::Annotation->
							           new('-DESCRIPTION' => $annot->description())
							       )
				    );
			  
   }

}




=head2 addScanPfamDBToAnnSeq

 Title   : addScanPfamDBToAnnSeq
 Usage   : Bio::Pfam::AnnotatedSequenceFactory::addScnaPfamDbToAnnSeq( $annSeq, \*MYHANDLE );
 Function: Adds AnnotatedRegion records to the given AnnotatedSequence by assuming that
    the contents of the given file is in scanPfamDB format
 Returns : 
 Args    : AnnotatedSequence ref, file handle ref

=cut

sub addScanPfamDBToAnnSeq{
   my ($self, $annSeq, $file) = @_;
   my ($annot, %annotations);

   while (<$file>) {
       if (/^\S+\s+(\d+)\s+(\d+)\s+(\S+)\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)$/) {
	   my ($start, $end, $accession, $id) = ($1, $2, $3, $4);

	   # acquire a default annotation for the region
	   # need to create an SFREngine to access the underlying database; no other reason
	   if ($annotations{ $accession } ) {
	       $annot = $annotations{ $accession };
	   }
	   else {
	       $annot = $self->engine()->underlyingDatabase()->get_Entry_by_acc($accession)->ann();
	       $annotations{ $accession } = $annot;
	   }
	   $annSeq->addAnnotatedRegion( Bio::Pfam::PfamRegion->new('-PFAM_ACCESSION' => $accession,
								   '-PFAM_ID' => $id,
								   '-FROM' => $start, 
								   '-TO' => $end,
								   '-ANNOTATION' => Bio::Pfam::Annotation->
							               new('-DESCRIPTION' => $annot->description())
								   )
				       );
       }
   }
}



=head2 engine

 Title   : engine
 Usage   : $eng = $fac->engine()
 Function:
    Returns the contained SFREngine object. This is simply for linking to he the database
 Returns : SFREngine
 Args    : SFREngine (optional)

=cut

sub engine{
   my ($self,$value) = @_;

   if (defined $value) {
       $self->{'factory_engine'} = $value;
   }
   return $self->{'factory_engine'};
}




=head2 instance

 Title   : instance
 Usage   : 
    $fac = Bio::Pfam::PfamAnnSeqFactory::instance( $engine );
 Function:
    Manages a single instance of the class, and returns the instance on request
    creating it first if necessary
 Returns : A ref to an AnnotatedSequenceFactory
 Args: An SFREngine reference, for linking to the underlying database

=cut

sub instance{
   my ($package, $eng) = @_;

   if (defined $Bio::Pfam::PfamAnnSeqFactory::theInstance) {
       return $Bio::Pfam::PfamAnnSeqFactory::theInstance;
   }
   else {
       $Bio::Pfam::PfamAnnSeqFactory::theInstance = $package->new( $eng );
       return $Bio::Pfam::PfamAnnSeqFactory::theInstance;
   }
}



=head2 readAnnotatedSequencesFromCache

 Title   : readAnnotatedSequencesFromCache
 Usage   :
    @annseqs = $fac->readAnnotatedSequnencesFromCache( $filehandle );
 Function:
    Returns a list of AnnotatedSequence objects (each without the actual sequence)
    form the given cache file
 Returns : A list of AnnotatedSequence
 Args    : File

=cut

sub readAnnotatedSequencesFromCache{
   my ($self, $file) = @_;

   my ($id, $start, $end, $desc, $currentAnnSeq, $sequence, @currentRegions, @seqList);

   while ( <$file> ) {
       if (/^NAME:\s*(\S+)/) {
	   if (defined $currentAnnSeq) {
	       push @seqList, $currentAnnSeq;
	   }
	   $id = $1;
       }
       elsif (/START:\s*(\S+)/ ) {
	   $start = $1;
       }
       elsif (/END:\s*(\S+)/ ) {
	   $end = $1;
       }
       elsif (/DESC:\s*(.*)/ ) {
	   $desc = $1;
	   $sequence = Bio::Seq->new('-id' => $id,
				     '-start' => $start,
				     '-end' => $end,
				     '-desc' => $desc );
	   $currentAnnSeq = $self->createAnnotatedSequence();
	   $currentAnnSeq->sequence( $sequence );
       }
       elsif (/PFAM_REGION:(\S+):(\S+):(\S+):(\S+):(.*)/) {
	   $currentAnnSeq->addAnnotatedRegion( 
	       Bio::Pfam::PfamRegion->new('-ACCESSION' => $1,
					  '-ID' => $2,
					  '-FROM' => $3,
					  '-TO' => $4,
					  '-ANNOTATION' => Bio::Pfam::Annotation->new('-DESCRIPTION' => $5 )
					  ) );
       }
       elsif (/SFR_REGION:(\d):(\S*):(\S*):(\S*):(\S*):(.*)/) {
	   $currentAnnSeq->addAnnotatedRegion( 
	       Bio::Pfam::SFRRegion->new('-TYPE' => $1,
					 '-HIGHEST_BAIT' => $2,
					 '-SCORE_OF_HIGHEST_BAIT' => $3,
					 '-FROM' => $4,
					 '-TO' => $5,
					 '-ANNOTATION' => Bio::Pfam::Annotation->new('-DESCRIPTION' => $6 )
					 ) );
       }
   }
   if (defined $currentAnnSeq) {
       push @seqList, $currentAnnSeq;
   }

   return \@seqList;

}



=head2 writeAnnotatedSequencesToCache

 Title   : writeAnnotatedSequencesToCache
 Usage   :
    $fac->writeAnnotatedSequencesToCache( $file, \%domainHash, @annSeqs );
 Function:
    Dumps the list of Annotated Sequecnes to file in a format that can easliy be read back
 Returns : 
 Args    : file, hash ref., list of AnnSeq

=cut

sub writeAnnotatedSequencesToCache{
   my ($self, $file, $domHashRef, $domNotUsedHashRef, $seqsRef) = @_;

   local $, = " ";

   print $file "BAITS_USED: ";
   print $file keys %{$domHashRef};
   print $file "\n";
   print $file "BAITS_NOT_FOUND: ";
   print $file keys %{$domNotUsedHashRef};
   print $file "\n";

   foreach my $seq (@$seqsRef) {
       print $file "NAME: ".$seq->sequence()->id()."\n";
       print $file "START: ".$seq->sequence()->start()."\n";
       print $file "END: ".$seq->sequence()->end()."\n";
       print $file "DESC: ".$seq->sequence()->desc()."\n";
       foreach my $region ( $seq->eachAnnotatedRegion() ) {
	   $region->write( $file );
       }
   }
}






