
#
# BioPerl module for Bio::Pfam::PfamRegion
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::PfamRegion - Representation of a protein sequence domain in Pfam

=head1 SYNOPSIS

    use Bio::Pfam::PfamRegion;

    $dm = new Bio::Pfam::PfamRegion( '-PFAM_ACCESSION' => $acc,
				     '-PFAM_ID' => $id,
				     '-FROM' => $start,
				     '-TO' => $end,
				     '-ANNOTATION' => $annotationRef);
    # parameter names may also be specified in lower case    

=head1 DESCRIPTION

This object stores the details for a Pfam domain. It is derived from the
AnnotatedRegion class. Information that must be given is the name and Pfam 
accession number of the domain, the extent of the domain (in terms of start and end
indices), and an annotation for the domain.

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::PfamRegion;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Pfam::AnnotatedRegion;

@ISA = qw(Bio::Pfam::AnnotatedRegion);
#@EXPORT_OK = qw();
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params ) = @_;
  my( $acc, $id, $start, $end, $annotation) = (
        ($params{'-PFAM_ACCESSION'}||$params{'-pfam_accession'}), ($params{'-PFAM_ID'}||$params{'-pfam_id'}),
	($params{'-FROM'}||$params{'-from'}), ($params{'-TO'}||$params{'-to'}), 
	($params{'-ANNOTATION'}||$params{'-annotation'}) );

  my $make = $self->SUPER::_initialize( %params );;

  $self->{'PfamReg_acc'} = $acc;
  $self->{'PfamReg_id'} = $id;

  return $make; # success - we hope!
}



=head2 accession

 Title   : accession
 Usage   : 
    $dom->accession(); # or ...
    $dom->accession( 123 );
 Function: For setting and getting the ACCESSION field in the object

=cut

sub accession{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'PfamReg_acc'} = $value;
   }
   return $self->{'PfamReg_acc'};
}



=head2 id

 Title   : id
 Usage   : 
    $dom->id(); # or ...
    $dom->id( "helloSir" );
 Function: For setting and getting the ID field in the object

=cut

sub id{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'PfamReg_id'} = $value;
   }
   return $self->{'PfamReg_id'};
}




=head2 write

 Title   : write
 Usage   : $dm->write( \*STDOUT );
 Function: Spits out the contents of the Domain object to the given filehandle
 Args    : The output file handle

=cut

sub write{
   my ($self, $file) = @_;

   print $file "PFAM_REGION:".$self->accession().":".$self->id().":";

   $self->SUPER::write( $file );
}






