#!/usr/local/bin/perl
#
# Perl script for SFRAnnotateFamily
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this script under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

SFRAnnotateFamily.pl - DESCRIPTION Attempts to perform an SFR ("bait") annotation
on the pfamseq sequences of a given family, printing the annotation along with the
swissprot description for the sequences.

=head1 SYNOPSIS

SFRAnnotateFamily.pl [options] family_name

=head1 DESCRIPTION

This script takes the sequences in the seed alignment for the given family and 
performs the SFR annotation ("bait") on them, printing the annotations along with 
the swissprot annotations for the sequences.

Options:

--full : The full alignment is annotated
--showscores : The highest bait score achieved for each annotation is shown with each
    annotation
--localbait : The current directory is searched for the apt. bait files rather than the
    installed version of pfam

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=cut


use strict;
use Getopt::Long;

use ProgramMinder;
use FileHandle;

use Bio::Pfam;
use Bio::Pfam::PfamAnnSeqFactory;
use Bio::Seq;
use Bio::Pfam::SFRObject;
use Bio::Pfam::SFREngine;
use Bio::Pfam::SFROutput;

local $| = 1;
my $pfamPath = "/nfs/disk100/pubseq/Pfam/CURRENT/";
my ($engine, $fh, $fullSequencesRequired, $scoresRequired, $localBaitRequired, $family, 
    $file, $fac, @allseqs, %seqs );
NeedProgram("getz5");

&GetOptions( 'full' => \$fullSequencesRequired,
	     'showscores'=> \$scoresRequired,
	     'localbait' => \$localBaitRequired );
($family) = @ARGV;

# since we are the first caller of this function, subsequent callers will get the same instance.
# Hence it is just the side effect we are looking for rather than the db instance itself
# If the option is disabled, the instance will be created with a null argument by a later call

if ($localBaitRequired) {
    $fh = FileHandle->new();
    $fh->open( "$family.BAIT" ) or die "Could not find local bait file $family.BAIT $!";
    $engine = Bio::Pfam::SFREngine->new( '-DATABASE' => Bio::Pfam->default_db(),
					 '-LOCAL_BAIT_FAMILY' => $family, 
					 '-LOCAL_BAIT_FILE' => $fh ); 
}
else {
    $engine = Bio::Pfam::SFREngine->new('-DATABASE' => Bio::Pfam->default_db() );
}

$fac = Bio::Pfam::PfamAnnSeqFactory->instance( $engine );

$file = $fullSequencesRequired?"ALIGN":"SEED";
open SEQUENCES, $pfamPath."$family/$file" or die "Could not get the necessary information from Pfam for the given family";
while (<SEQUENCES>) {
    my ($seqName, $start, $end) = /^(\S+)\/(\d+)-(\d+)/;

    if (! $seqs{ $seqName } ) {
	$seqs{ $seqName }++;

	# first create the empty annotated sequence
	my $annSeq = $fac->createAnnotatedSequence();
	
	# now determine the domain structure
	open SWISS, "getz5 -e \'[swisspfam-id:$seqName]\' |" or die "Failed to initiate getz call";
	$fac->addSwissPfamToAnnSeq( $annSeq, \*SWISS, 0); 
	close SWISS or die "Could not close getz command";

	# now get the actual sequence
	my ($accumulatedSequence, $swissprotDescription, $amReadingSequence);
	open SEQ, "getz5 -e \'[pfamseq-id:$seqName]\' |" or die "Failed to initiate getz call";
	while (<SEQ>) {
	    if (/^SQ/) {
	        $amReadingSequence = 1;
		next;
	    }
	    elsif (/^\/\//) {
		last;
	    }
	    elsif (/^DE\s+(.*)/) {
		$swissprotDescription .= $1." ";
	    }
	    else {
		if ($amReadingSequence) { 
		    $accumulatedSequence .= $_;
		}
	    }
	}
	close SEQ or die "Could not close getz command";

	$accumulatedSequence =~ s/\s+//g;
	$annSeq->sequence( Bio::Seq->new('-id' => $seqName,
					 '-seq' => $accumulatedSequence,
					 '-desc' => $swissprotDescription
					 )
			  );
	
	# finally, annotate

        $engine->resolveAnnotatedSequence( $annSeq );

	push @allseqs, $annSeq;
    }
}
close SEQUENCES;

Bio::Pfam::SFROutput->writeFamily(  \*STDOUT, $scoresRequired, \@allseqs );





