#!/usr/local/bin/perl
#
# Perl script for SFRAnnotation
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this script under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

SFRAnnotation - DESCRIPTION Driver script for the Sub-family Representative
Annotation process.

=head1 SYNOPSIS

prompt% SFRAnnotation.pl --format <FORMAT> file1 file2

=head1 DESCRIPTION

This script performs fine-grained annotation of a given sequence using the
sub-family representative, or SFR method ("bait").
It takes a single query sequence and the domain structure of that sequence
and performs the SFR annotation of that sequence

file1 should contain the pfam-domain structure of the given sequence in some  
format. The format should be specified by the string following the --format option.
Supported formats are:

    --format HMM 
         (HMM results format)
    --format SWISS
         (SwissPfam format) 
    --format SPFSB
         (scanPfamDb format)

If no --format option is supplied, ScanPfamDB format is assumed

file2 should contain the sequence to be annotated in fasta format. Other formats
may be supported later.

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=cut



use strict;
use Getopt::Long;

use Bio::Pfam;
use Bio::Pfam::PfamAnnSeqFactory;
use Bio::Seq;
use Bio::Pfam::SFRObject;
use Bio::Pfam::SFREngine;

my ($format, $structure, $querySequence);

&GetOptions( 'format=s' => \$format );
($structure, $querySequence) = @ARGV;

# First, standardise the option string
if (not $format) {
    $format = "SPFDB";
}
$format =~ tr/a-z/A-Z/;

my $eng = Bio::Pfam::SFREngine->new('-DATABASE' => Bio::Pfam->default_db());
my $fac = Bio::Pfam::PfamAnnSeqFactory->instance( $eng );
my $annSeq = $fac->createAnnotatedSequence();

open INPUT, $structure or die "Could not open file $structure";
open QUERY, $querySequence or die "Could not open file $querySequence";

# First create the AnnotatedSequence object

if    ( $format eq "SWISS" ) { $fac->addSwissPfamToAnnSeq( $annSeq, \*INPUT, 0); }
elsif ( $format eq "SPFDB" ) { $fac->addScanPfamDBToAnnSeq( $annSeq, \*INPUT); }
elsif ( $format eq "HMM"   ) { $fac->addHMMResultsToAnnSeq( $annSeq, \*INPUT); }
else                         { die "Invalid format specified"; }

# now initialise the Annotated sequence with a sequence

my ($id, $seqString );
while (<QUERY>) {
    if (/^>(\S+)/ ) {
	$id = $1;
    }
    else {
	$seqString .= $_;
    }
}
$seqString =~ s/\s+//g;

$annSeq->sequence( Bio::Seq->new('-id' => $id,
				 '-seq' => $seqString
				 )
		  );

# now we have a sequence together with its Pfam annotation. Annotate it

$eng->resolveAnnotatedSequence( $annSeq  );

print STDOUT "Here is the complete annotation:\n";
$annSeq->write( \*STDOUT );

close INPUT;
close QUERY;

