
#
# BioPerl module for Bio::Pfam::SFREngine
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::SFREngine - Sub-family Representative Engine - This is the main
    agent of the bait annotation process. It handles requests for bait files, 
    does the top level annotation (via the method 'resolveAnnotatedSequence').
    It can be configured with an author, so that it can attempt to get bait 
    entries written by this author on request. The two fields 'localbaitfile'
    and 'local baitfamily' are used when annotation is done by referencing 
    a temporary local bait file (i.e. 'in development')rather than the one stored
    in 'the database'
    

=head1 SYNOPSIS
    $sfreng = Bio::Pfam::SFREngine->new('-DATABASE' => $db,
					'-AUTHOR' =>  $aut,
					'-LOCAL_BAIT_FILE' => $file,
					'-LOCAL_BAIT_FAMILY' => $name);

    $sfrObj = $sfreng->getSFRObject( $accession );

    $sfreng->resolveAnnotatedSequence( $anAnnotatedSequenceRef );

=head1 DESCRIPTION
    This object is the brains behind the bait system. It handles the actual annotation as well as
    providing methods for reading and writing bait files. Review the methos documentation for
    education of its power.

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::SFREngine;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

# Object preamble - inherits from Bio::Root::Object

use Bio::Root::Object;
use Bio::Seq;

use Bio::Pfam::SFRObject;

use FileHandle;

@ISA = qw(Bio::Root::Object Exporter);
#@EXPORT_OK = qw();
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self,%params) = @_;

  my ($database, $author, $localBaitFile, $localBaitFamily) = 
      ( ($params{'-DATABASE'}||$params{'-database'}),
        ($params{'-AUTHOR'}||$params{'-author'}),
        ($params{'-LOCAL_BAIT_FILE'}||$params{'local_bait_file'}),
	($params{'-LOCAL_BAIT_FAMILY'}||$params{'-local_bait_family'}) );

  my $make = $self->SUPER::_initialize;

  $self->{'sfreng_cache'} = {}; 
  $self->{'sfreng_localBaitFile'} = $localBaitFile;
  $self->{'sfreng_theDatabase'} = $database; 

  if (defined $author ) {
      $self->{'sfreng_author'} = $author;
  }
  else {
      $self->{'sfreng_author'} = "Pfam";
  }
  if (defined $localBaitFamily) {
      if ($localBaitFamily =~ /^PF/) {
	  $self->{'sfreng_localBaitFamily'} = $localBaitFamily;
      }
      else {
	  $self->{'sfreng_localBaitFamily'} = $self->{'sfreng_theDatabase'}->id2acc( $localBaitFamily );
      } 
  }
  return $make; # success - we hope!
}



sub DESTROY {
    my $self = shift;

    # just need to delete all of the indexed sequence files (and their indices) from the objects
    # in the cache

    foreach my $el (keys %{$self->{'sfreng_cache'}}) {
	my $obj = $self->{'sfreng_cache'}{$el};
	my $name = $obj->indexedSequences();
	unlink $name;
	# the following will delete the index files, hopefully

	my @names = glob("$name.*");
	unlink @names;
    }
}



=head2 author

 Title   : author
 Usage   :
    $eng->author( "Bob" );
    $author = $eng->author();
    
 Function:
    An SFREngine can be associated with a bait author. 
 Returns : (optional ref.) author string
 Args    : (optional ref) author string

=cut

sub author{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'sfreng_author'} = $value;
   }
   return  $self->{'sfreng_author'};
}



=head2 localBaitFamily

 Title   : localBaitFamily
 Usage   :
    $eng->localBaitFamily( $name ); or ...
    $name = $eng->localBaitFamily();
    
 Function:
    Some databases are intialised with a local bait file. This is the name of the family
    for that local bait file (undef if none exists);
 Returns : (optional) family name
 Args    : (optional) family name

=cut

sub localBaitFamily{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'sfreng_localBaitFamily'} = $value;
   }
   return  $self->{'sfreng_localBaitFamily'};
}




=head2 localBaitFile

 Title   : localBaitFile
 Usage   :
    $eng->localBaitFile( $fh ); or ...
    $fh = $eng->localBaitFile();
    
 Function:
    Some databases are intialised with a local bait file. This is the handle to
    that local bait file (undef if none exists);
 Returns : (optional) file handle
 Args    : (optional) file handle

=cut

sub localBaitFile{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'sfreng_localBaitFile'} = $value;
   }
   return  $self->{'sfreng_localBaitFile'};
}





=head2 getSFRObject

 Title   : getSFRObject
 Usage   : $myObj = $eng->getSFRObject( $pfam_accession );
 Function: Creates, fills and returns an SFRObject based on the accession number given.
 Returns : A refrence to an SFRObject
 Args    : A Pfam accession number

=cut

sub getSFRObject{
    my ($self, $accession ) = @_;
    my ($sfrObjRef, $sfrFile);
    
    my $author = $self->author();

    if ( $self->{'sfreng_cache'}{ $accession } ) {
	$sfrObjRef = $self->{'sfreng_cache'}{ $accession };
    }
    else {
	# If localBaitFamily is undefined (which will be the case if the object was created without one)
	# then the following check will always be false, which is what is desired

	$sfrObjRef = Bio::Pfam::SFRObject->new();

	if ( $accession eq $self->localBaitFamily() ) {
	    $sfrFile = $self->localBaitFile();
	}
	else {
	    if (not defined($sfrFile = $self->underlyingDatabase()->get_sfr_file( $accession ) ) ) {
		return undef;
	    } 
	}

	$sfrObjRef->read( $sfrFile, $author );

	if (! $sfrObjRef->eachSFRUnit() ) {
	    # There must have been no bait file entry for the given author.
	    # If the author was not Pfam, we could always give them the Pfam
	    # entry ...
       
	    if ($author !~ /^Pfam$/i) {
		$sfrObjRef->read( $sfrFile, "Pfam");
	    }
	}

	# after the addition of bait authors and 'temporary' bait files, it may not be safe to keep
	# a cache. Hence comment out the following for now

	$self->{'sfreng_cache'}{ $accession } = $sfrObjRef;
    }

    return $sfrObjRef;

}




=head2 putSFRObject

 Title   : putSFRObject
 Usage   :
    $eng->putSFRObject( $obj );
 Function:
    This function writes the given SFRObject to the correct place in the
    database bait file. Note that it is only called from the web 
    implementation of the bait system where the bait files are editable
 Returns : 
 Args    :  An SFRObject ref

=cut

sub putSFRObject{
   my ($self, $sfrObj ) = @_;
   my ($fh, $author, $tfiles, $newBaitFile, $needToCopy, $readAccession, $readId);

   $fh = FileHandle->new();
   $author = $sfrObj->author();

   $newBaitFile = "/tmp/".$sfrObj->accession().".$$";

   $fh->open( ">".$newBaitFile ) or $self->throw("Could not open temporary bait file for writing");
   
   my $source = $self->underlyingDatabase()->get_sfr_file( $sfrObj->accession() );

   if (defined $source ) {
       $needToCopy = 1;
       
       while (<$source>) {
	   if (/^AUTHOR:\s*(\S+)/) {
	       $needToCopy = ($1 eq $author)?0:1;   
	   }
	   
	   if ($needToCopy) {
	       print $fh $_;
	   }
       }
       $sfrObj->write( $fh, 1 );
   }
   else {
       $sfrObj->write( $fh );
   }

   $fh->close() or $self->throw("Could not close the temp bait file after writing");
   # fh now contains the new bait entries. We need to copy them to the system bait file
   
   $fh->open("<".$newBaitFile) or  $self->throw("Could not re-open temporary bait file for reading");
   my $dest = $self->underlyingDatabase()->get_sfr_file( $sfrObj->accession(), 1);
   while (<$fh>) {
       print $dest $_;
   }

   $fh->close() or $self->throw("Could not close the temp bait file after reading");
   $dest->close() or $self->throw("Could not close the system bait file after writing");

   unlink $newBaitFile;
}




=head2 resolveAnnotatedSequence

 Title   : resolveAnnotatedSequence
 Usage   : $eng->resolveAnnotatedSequence( $annSeq );
 Function: Resolves the fine level annotation of the sequence in $annSeq using the
    bait mechanism. The function inserts annotation records into $annSeq
    as it resolves the annotation.
     
 Returns : 
 Args    : A reference to an initialised AnnotatedSequence object 

=cut

sub resolveAnnotatedSequence{
   my ($self, $annSeq, $baitsUsedRef, $baitsNotFoundRef ) = @_;
   my ($sfrObj, @allDomains, %occurrences );

   @allDomains = $annSeq->eachAnnotatedRegion();

   if (! @allDomains ) {
       $annSeq->addAnnotatedRegion( Bio::Pfam::SFRRegion->new('-TYPE' => $Bio::Pfam::SFRRegion::_SFR,
							      '-FROM' => '1',
							      '-TO' => $annSeq->sequence()->seq_len(),
							      '-ANNOTATION' => Bio::Pfam::Annotation->new(
								    '-DESCRIPTION' => "No PfamA domains" ) 
							      )
				   );
   }
   else {
       foreach my $domain (@allDomains) {
	   $sfrObj = $self->getSFRObject( $domain->accession() );
	   if (not defined $sfrObj) {
	       if (defined $baitsNotFoundRef) {
		   push @$baitsNotFoundRef, $domain->accession();
	       }
	       $annSeq->addAnnotatedRegion( Bio::Pfam::SFRRegion->
					        new('-TYPE' => $Bio::Pfam::SFRRegion::_DEFAULT,
						    '-FROM' => $domain->from(),
						    '-TO' => $domain->to(),
						    '-ANNOTATION' => Bio::Pfam::Annotation->new()
						    )
					    );
	   }
	   else {
	       if ($sfrObj->isDomainSearchNeeded() or (not $occurrences{ $domain->accession() }) ) {
		   $occurrences{ $domain->accession() }++;
		   $annSeq->addAnnotatedRegion( $sfrObj->annotate( $annSeq->sequence(), $domain, @allDomains ) );
	       }
	       else {
		   $annSeq->addAnnotatedRegion(Bio::Pfam::SFRRegion->new('-TYPE' => $Bio::Pfam::SFRRegion::_REPEAT));
	       }
	       # The following gives us a way to track which bait files were used during the annotation process
	       if (defined $baitsUsedRef) {
		   push @$baitsUsedRef, $domain->accession();
	       }
	   }
       }
   }
}



=head2 underlyingDatabase

 Title   : underlyingDatabase
 Usage   :
    $eng->underlyingDatabase( $dbRef ); or ...
    $dbRef = $eng->underlyingDatabase();
    
 Function:
    This object sometimes delegates to another database object encapsulating the underlying 
    pfam database. This is a way of getting and setting that.
 Returns : (optional ref.) DB
 Args    : (optional ref) DB

=cut

sub underlyingDatabase{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'sfreng_theDatabase'} = $value;
   }
   return  $self->{'sfreng_theDatabase'};
}


