
#
# BioPerl module for Bio::Pfam::SFRRegion
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::SFRRegion - Sub-familt representative Region. Representation of a
region of a sequence annotated by the Bait mechanism

=head1 SYNOPSIS

    use Bio::Pfam::SFRRegion;

    $reg = Bio::Pfam::SFRRegion->new('-TYPE' => $Bio::Pfam::SFRRegion::_SFR,
				     '-HIGHEST_BAIT' => $id,
				     '-SCORE_OF_HIGHEST_BAIT' => $score,
				     '-FROM' => $start,
				     '-TO' => $end,
				     '-ANNOTATION' => $annotationRef);

    # or ...

    $reg = Bio::Pfam::SFRRegion->new('-TYPE' => $Bio::Pfam::_DEFAULT,
				     '-FROM' => $start,
				     '-TO' => $end);  

The possible values for TYPE are:

Bio::Pfam::SFRRegion::_SFR
Bio::Pfam::SFRRegion::_DEFAULT
Bio::Pfam::SFRRegion::_REPEAT
Bio::Pfam::SFRRegion::_DELEGATE

=head1 DESCRIPTION

This object stores the details for an AnnotatedRegion that has been constucted
by the Bait mechanism. It is derived from the AnnotatedRegion class. Information 
that must be given is the extent of the region (in terms of start and end
indices), and an annotation for the region.

Note that the two fields relating to bait entries (id and score) have been included
to provide a traceability to the output

=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::SFRRegion;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

use Bio::Pfam::AnnotatedRegion;

$Bio::Pfam::SFRRegion::_SFR = 1;
$Bio::Pfam::SFRRegion::_DEFAULT = 2;
$Bio::Pfam::SFRRegion::_REPEAT = 3;
$Bio::Pfam::SFRRegion::_DELEGATE = 4;

@ISA = qw(Bio::Pfam::AnnotatedRegion);
#@EXPORT_OK = qw();
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params ) = @_;
  my( $type, $match, $score ) = ( ($params{'-TYPE'}||$params{'-type'}), 
				 ($params{'-HIGHEST_BAIT'}||$params{'-highest_bait'}),
				 ($params{'-SCORE_OF_HIGHEST_BAIT'}||$params{'-score_of_highest_bait'}) );
 

  my $make = $self->SUPER::_initialize( %params );

  $self->{'SFRReg_type'} = $type;
  $self->{'SFRReg_highestBait'} = $match;
  $self->{'SFRReg_scoreOfHighestBait'} = $score;

  return $make; # success - we hope!
}



=head2 highestBait

 Title   : highestBait
 Usage   : 
    $id = $reg->highestBait(); # or ...
    $reg->highestBait( $id );
 Function:
    Sets/gets the score of the highest bait match for this region
 Returns : A string id for the sequence in the bait file that produced the best match
 Args    : A string id for the sequence in the bait file that produced the best match (optional)

=cut

sub highestBait{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'SFRReg_highestBait'} = $value;
   }
   return $self->{'SFRReg_highestBait'};
}




=head2 scoreOfHighestBait

 Title   : scoreOfHighestBait
 Usage   : 
    $score = $reg->scoreOfHighestBait(); # or ...
    $reg->scoreOfHighestBait( $score );
 Function:
    Sets/gets the score of the highest bait match for this region
 Returns : A score
 Args    : A score (optional)

=cut

sub scoreOfHighestBait{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'SFRReg_scoreOfHighestBait'} = $value;
   }
   return $self->{'SFRReg_scoreOfHighestBait'};
}





=head2 type

 Title   : type
 Usage   : 
    $myType = $sfrReg->type();
    $sfrReg->type( Bio::Pfam::SFRRegion::_DEFAULT ); # for example 
 Function: Fo getting and setting the TYPE field in this SFRRegion
 Notes: For safety, this function should inly be used with package
    constants defined at the top of the file.

=cut

sub type{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'SFRReg_type'} = $value;
   }
   return $self->{'SFRReg_type'};
}




=head2 write

 Title   : write
 Usage   : $dm->write( \*STDOUT );
 Function: Spits out the contents of the Domain object to the given filehandle
 Args    : The output file handle

=cut

sub write{
   my ($self, $file) = @_;

   print $file "SFR_REGION:".$self->type().":".$self->highestBait().":".$self->scoreOfHighestBait().":";
   $self->SUPER::write( $file );
}
