
#
# BioPerl module for Bio::Pfam::SFRUnit
#
# Cared for by Kevin Howe <klh@sanger.ac.uk>
#
# Copyright Kevin Howe
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::Pfam::SFRUnit - Sub-Family Relationship unit;
Internal representation of a single entry in a bait file.

=head1 SYNOPSIS

    $sfrUn = new Bio::Pfam::SFRUnit('-ID' => $id,
				    '-DB_NAME' => $nameOfDB,
				    '-FROM' => $start,
				    '-TO' => $end,				    
				    '-THRESHOLD' => $threshold,
				    '-SEQUENCE' => $seqRef,
				    '-ANNOTATION' => $annot );

=head1 DESCRIPTION

SFRUnit represents a single entry in the bait file associated with a particular family.
The named parameters define the information maintained by the object, and most are self-
explanatory.

The threshold and method (from the parent SFRObject) are interdependent in that the 
threshold should be used as the basis for a comparison to scores obtained from that
method only.


=head1 CONTACT

Mail pfam@sanger.ac.uk with any queries.

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::Pfam::SFRUnit;
use vars qw($AUTOLOAD @ISA @EXPORT_OK);
use strict;

# Object preamble - inherits from Bio::Root::Object

use Bio::Root::Object;
use Bio::Pfam::ThresholdAnnotationPair;

@ISA = qw(Bio::Root::Object Exporter);
#@EXPORT_OK = qw();
# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self, %params ) = @_;
  my( $id, $dbName, $start, $finish, $threshold, $annotation, $seqRef) =
      (($params{'-ID'}||$params{'-id'}), ($params{'-DB_NAME'}||$params{'db_name'}),
       ($params{'-FROM'}||$params{'-from'}), ($params{'-TO'}||$params{'-to'}), 
       ($params{'-THRESHOLD'}||$params{'-threshold'}),  ($params{'-ANNOTATION'}||$params{'-annotation'}),
       ($params{'-SEQUENCE'}||$params{'-sequence'}) );

  my $make = $self->SUPER::_initialize;

  $self->{'unit_id'} = $id;
  $self->{'unit_dbName'} = $dbName;
  $self->{'unit_sequence'} = $seqRef;
  $self->{'unit_from'} = $start;
  $self->{'unit_to'} = $finish;

  # create the (threshold, annoation) pair list, and add the first element to it
  $self->{'unit_ThreshAnnotPairList'} = [];
  $self->addThresholdAnnotationPair( Bio::Pfam::ThresholdAnnotationPair->
				         new('-THRESHOLD' => $threshold,
					     '-ANNOTATION' => $annotation 
					     )
				    ); 

  return $make; # success - we hope!
}




=head2 addThresholdAnnotationPair

 Title   : addThresholdAnnotationPair
 Usage   :
    $unit->addThresholdAnnotationPair( Bio::Pfam::ThresholdAnnotationPair->
				           new('-THRESHOLD' => $thres,
					       '-ANNOTATION' => $annotRef
					       )
				      );
 Function:
    Adds to the list of (Threshold, Annotation) pairs mainteained by 
    this SFRUnit;
 Returns : 
 Args    : A ref to a ThresholdAnnotationPair object

=cut

sub addThresholdAnnotationPair{
   my ($self, $pair) = @_;
   my ($needToAdd) = (1);

   # we need to check that there are no entries with this threshold already. If there are,
   # we just over-write the annotation with the new one
   foreach my $pr (@{$self->{'unit_ThreshAnnotPairList'}}) {
       if ($pr->threshold() eq $pair->threshold() ) {
	   $pr->annotation( $pair->annotation() );
	   $needToAdd = 0;
	   last;
       }
   }
   if ( $needToAdd ) {
       push @{$self->{'unit_ThreshAnnotPairList'}}, $pair;
   }
}




=head2 dbName

 Title   : dbName
 Usage   : 
    $unit->dbName(); # or ...
    $unt->dbName( "swissprot" );
 Function: For setting and getting the NAME field in the object

=cut

sub dbName{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'unit_dbName'} = $value;
   }
   return $self->{'unit_dbName'};
}



=head2 eachThresholdAnnotationPair

 Title   : eachThresholdAnnotationPair
 Usage   : @list = $unit->eachThresholdAnnotationPair();
 Function:
    Returns the list of (Threshold, Annotation) pairs in this SFRUnit 
 Returns : 
    A list of (Threshold, Annotation) pairs 
 Args    : None

=cut

sub eachThresholdAnnotationPair{
   my ($self,@args) = @_;

   return @{$self->{'unit_ThreshAnnotPairList'}};
}




=head2 from

 Title   : from
 Usage   : 
    $dom->from(); # or ...
    $dom->from( 123 );
 Function: For setting and getting the FROM field in the object

=cut

sub from{
   my ($self, $value) = @_;


   if (defined $value) {
       $self->{'unit_from'} = $value;
   }
   return $self->{'unit_from'};
}




=head2 id

 Title   : id
 Usage   : 
    $unit->id(); # or ...
    $unt->id( 123 );
 Function: For setting and getting the ID field in the object

=cut

sub id{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'unit_id'} = $value;
   }
   return $self->{'unit_id'};
}




=head2 sequence

 Title   : sequence
 Usage   : 
    $sfrUn->sequence(); $sfrUn->sequence( $theSequence );
 Function: sets or returns this SFRUnit objects sequence field
 Returns : ref to a Bio::Seq object
 Args    : A Bio::Seq reference (optional)

=cut

sub sequence{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'unit_sequence'} = $value;
   }
   return $self->{'unit_sequence'};
}


=head2 to

 Title   : to
 Usage   : 
    $dom->to(); # or ...
    $dom->to( 123 );
 Function: For setting and getting the TO field in the object

=cut

sub to{
   my ($self, $value) = @_;

   if (defined $value) {
       $self->{'unit_to'} = $value;
   }
   return $self->{'unit_to'};
}



=head2 write

 Title   : write
 Usage   :
    $unit->write( \*HANDLE );
 Function:
    Spits out the contents of the SFRUnit to the file handle
 Returns : 
 Args    : Handle to the destination

=cut

sub write{
   my ($self, $handle) = @_;

   foreach my $pair ( $self->eachThresholdAnnotationPair() ) {
       print $handle ">".$self->id().";".$self->from()."-".$self->to().";"
	   .$pair->threshold().";".$pair->annotation->description()."\n";
       print $handle $self->sequence()->out_raw()."\n";
   }

}
