
#
# BioPerl module for Bio::ReSeed
#
# Cared for by Ewan Birney <birney@sanger.ac.uk>
#
# Copyright Ewan Birney
#
# You may distribute this module under the same terms as perl itself

# POD documentation - main docs before the code

=head1 NAME

Bio::ReSeed - Converts one multiple alignment to another database

=head1 SYNOPSIS

  $re = Bio::ReSeed->new( -olddb => 'pfamseq', -newdb => 'pfamseq4' );

  # $old is a Bio::SimpleAlign object

  $new = $re->convert($old);


=head1 DESCRIPTION

This object factory converts Bio::SimpleAlign objects into new SimpleAlign objects
which have a new underlying database

It is quite specific to Pfam. In particular, it expects the old and new databases
to be pfamseq style databases, where you want to follow the sequences by id, but
have to use the accession number to follow the database. This requires the databases
to be SRS indexed (it makes calls to getz).

However, if you want to use it on a different database mapping system, you can always
subclass it and replace the functions _get_old_seq and _convert_seq which provide
this logic.

Finally a note about the conversion process. If the old and new sequences are identical,
then it maps them without changing the sequence. If the old subsequence can be found 
identically inside the new sequence, that subsequence is used. Otherwise an HMM alignment
step is used to make the new sequence.

=head1 CONTACT

pfam@sanger.ac.uk

=head1 APPENDIX

The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _

=cut


# Let the code begin...


package Bio::ReSeed;
use vars qw($AUTOLOAD @ISA);
use strict;

# Object preamble - inheriets from Bio::Root::Object

use Bio::Root::Object;
use Bio::Seq;
use Bio::Tools::pSW;



@ISA = qw(Bio::Root::Object Exporter);

# new() is inherited from Bio::Root::Object

# _initialize is where the heavy stuff will happen when new is called

sub _initialize {
  my($self,@args) = @_;
  my($olddb,$newdb) = $self->_rearrange([qw(OLDDB NEWDB)],@args);
  my $make = $self->SUPER::_initialize(@args);

  $self->_olddb($olddb);
  $self->_newdb($newdb);
  $self->{'newseqcache'} = {};
  $self->{'oldseqcache'} = {};
  $self->{'idhash'} = {};

  # set stuff in self from @args
  return $make; # success - we hope!
}

=head2 convert

 Title   : convert
 Usage   : $new = $reseed->convert($old,\*ERROR_LOG,\*LOG);
 Function: Actually does the conversion of the old to the new
           database
 Returns : New Bio::SimpleAlign object
 Args    : old Bio::SimpleAlign object, and then two optional filehandles
           The first is only written to for subs/indel errors: the second
           records everything which is done

=cut

sub convert{
   my ($self,$old,$error,$log) = @_;
   my ($new,$seq,$newseq,$oldseq,$strippedseq,@toalign,$start,$end,$newsubseq,$psw,$name,$oldid,$newid,$oldstart,$oldend);

   #
   # flush sequence caches
   #

   $self->_flush_newseqcache();
   $self->_flush_oldseqcache();

   $new = Bio::SimpleAlign->new();

   # in case we need some alignments, we need the pSW alignment
   # factory

   $psw = Bio::Tools::pSW->new(); # use defaults. Should be fine!

   foreach $seq ( $old->eachSeq ) {

     # so we can report easily

     $name = $seq->get_nse();

     # get old and new underlying sequences

     $oldseq = $self->_get_old_seq($seq);
     $newseq = $self->_convert_seq($seq);

     # if we can't get them - cry to ERROR

     if( !defined $oldseq || length($oldseq->str()) <= 0 || !defined $newseq || length($newseq->str()) <= 0 ) {
       if ( defined $error ) {
	 print $error "$name has not been converted into old/new sequence pairs\n";
	 print $log   "$name has not been converted into old/new sequence pairs\n";
	 next;
       } else {
	 $self->throw("Could not convert $name");
       }
     }
       

     # get fields out for convience
     # so we can report stuff to log files etc

     $oldstart = $seq->start();
     $oldend   = $seq->end();

     $oldid = $oldseq->id();
     $newid = $newseq->id();


     #print STDERR $oldseq->out_fasta();
     #print STDERR $newseq->out_fasta();


     # if they are the same - yippee!

     if( $oldseq->str() eq $newseq->str() ) {
       # id's might be different

       $seq->id($newseq->id());

       # add the old sequence in
       $new->addSeq($seq);

       $log && print $log "$name maps without any changes $oldid:$newid\n";
       next;
     }

     # check to see if the subsequence is unique somewhere
     # we need to take the old subsequence, remove the gaps and
     # see if it matches to anything in the new subsequence

     $strippedseq = $seq->str();

     $strippedseq =~ s/[^A-Za-z]//g;
     $strippedseq =~ tr/[a-z]/[A-Z]/;


     if( $newseq->str() =~ /^([A-Z]*)$strippedseq([A-Z]*)$/ ) {
       
       # we have an identical subsequence

       my $pre = $1;
       my $post = $2;
       $start = length($pre)+1;
       $end = length($strippedseq)-1+$start;


       $newsubseq = Bio::Seq->new( -seq=> $seq->str(), -id => $newseq->id(), -start => $start, -end => $end , -type => 'aligned');

       # add it to the alignment

       $new->addSeq($newsubseq);
       $log && print $log "$name sequence has changed but subsequence ok $oldid:$newid, old[$oldstart:$oldend] vs new[$start:$end]\n";

       next; # back to main loop
     }

     # yuk! We have a subs/indel problem.
     # we need to chop out the sequence as best we can using bp_sw
     # to get start/end points. Then we place it on the @toalign
     # array. At the end of processing all the sequences, we realign
     # these by making a temporary hmm...

     my $tempseq = Bio::Seq->new ( -seq => $strippedseq, -id => 'tempseq');

     my $tempaln = $psw->pairwise_alignment($tempseq,$newseq);

     # start - end points from the alignment
     # reusing tempseq here to get the second 'sequence' in the alignment of 
     # the old subsequence to the new complete sequence

     my @tarray = $tempaln->eachSeq();

     $tempseq = $tarray[1];

     $newsubseq = Bio::Seq->new ( -seq => $newseq->str($tempseq->start,$tempseq->end) ,
				  -id => $newseq->id(), 
				  -start => $tempseq->start, 
				  -end => $tempseq->end,
				  );

     # push it on to the array

     push(@toalign,$newsubseq);
     $log && print $log "$name has changes\n";
     $error && print $error "$name has changes\n";

   } # end of loop over all sequences.


   # now we need to align the remaining sequences

   if( @toalign >= 1 ) { 
     # we have some sequences to align

     # build a temporary alignment file

     open(_TEMP_,">reseed.temp.$$") || $self->throw("Could not open temp file $!");
     $new->write_Pfam(\*_TEMP_);
     close(_TEMP_);
     # build an HMM

     open(_T_,"hmmbuild reseed.HMM.$$ reseed.temp.$$ |");
     while(<_T_>) {
       ;
     }

     # write out all the other sequences to be aligned

     open(_TEMP_,">reseed.unaln.$$") || $self->throw("Could not open temporary unaligned file $!");
     foreach $seq ( @toalign ) {
       my $name = $seq->get_nse();
       print _TEMP_ ">$name\n";
       print _TEMP_ $seq->str();
     }

     close(_TEMP_);

     # make the alignment

     open(_TEMP2_,"hmmalign --mapali reseed.temp.$$ reseed.HMM.$$ reseed.unaln.$$ |");

     # read it in

     my $newaln = Bio::SimpleAlign->new();
     while(<_TEMP2_>) {
       /^#/ && last;
     }

     
     $newaln->read_selex(\*_TEMP2_);

     close(_TEMP2_) || $self->throw("Could not close hmmalign ");

     # remove temporary files

     unlink("reseed.temp.$$");
     unlink("reseed.HMM.$$");
     unlink("ressed.unaln.$$");

     # reset new to newaln

     $new = $newaln;
     $new->map_chars('\-','.');
   }

     
   return $new;
  
}

=head2 _get_old_seq

 Title   : _get_old_seq
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _get_old_seq{
  my ($self,$subseq) = @_;
  my ($new,$olddb,$id);

  $olddb = $self->_olddb();
  $id = $subseq->id();

  $new = $self->_is_oldseqcache($id);
  if( !defined $new ) {
    $new = Bio::Seq->new(-file => "getz -sf fasta -d '[$olddb-id:$id]' |");
  } else {
    return $new;
  }

  $self->_add_to_oldseqcache($id,$new);
  return $new;
  
}


=head2 _convert_seq

 Title   : _convert_seq
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _convert_seq{
  my ($self,$subseq) = @_;
  my ($new,$olddb,$newdb,$id,$acc);

  $olddb = $self->_olddb();
  $newdb = $self->_newdb();

  $id = $subseq->id();
  $acc = $self->{'idhash'}->{$id};

  if( !defined $acc) {
    open(_TEMP_,"getz -f acc '[$olddb-id:$id]' |");
    while(<_TEMP_>) {
      /^AC\s+(\S+);/ && do { $acc = $1; last; };
    }
    

    if( !defined $acc ) {
      $self->throw("Unable to find $acc for $id");
    }
    $self->{'idhash'}->{$id} = $acc;

    close(_TEMP_) || $self->throw("Unable to close filehandle for reading _TEMP_");
  }


  $new = $self->_is_newseqcache($acc);

  if( !defined $new ) {
    $new = Bio::Seq->new(-file => "getz -sf fasta -d '[$newdb-acc:$acc]' |");
  } else {
   # print STDERR "Using cache...\n";
    return $new;
  }


  $self->_add_to_newseqcache($acc,$new);

  return $new;
}

=head2 _olddb

 Title   : _olddb
 Usage   : $obj->_olddb($newval)
 Function: 
 Example : 
 Returns : value of _olddb
 Args    : newvalue (optional)


=cut

sub _olddb{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'_olddb'} = $value;
    }
    return $self->{'_olddb'};

}

=head2 _newdb

 Title   : _newdb
 Usage   : $self->_newdb($newval)
 Function: 
 Example : 
 Returns : value of _newdb
 Args    : newvalue (optional)


=cut

sub _newdb{
   my ($self,$value) = @_;
   if( defined $value) {
      $self->{'_newdb'} = $value;
    }
    return $self->{'_newdb'};

}

=head2 _add_to_newseqcache

 Title   : _add_to_newseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _add_to_newseqcache{
   my ($self,$hashval,$seq) = @_;
   
   $self->{'newseqcache'}->{$hashval} = $seq;

}

=head2 _flush_newseqcache

 Title   : _flush_newseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _flush_newseqcache{
   my ($self) = @_;

   $self->{'newseqcache'} = {};
}

=head2 _is_newseqcache

 Title   : _is_newseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _is_newseqcache{
   my ($self,$hashval) = @_;

   return $self->{'newseqcache'}->{$hashval};

}


=head2 _add_to_oldseqcache

 Title   : _add_to_oldseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _add_to_oldseqcache{
   my ($self,$hashval,$seq) = @_;
   
   $self->{'oldseqcache'}->{$hashval} = $seq;

}

=head2 _flush_oldseqcache

 Title   : _flush_oldseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _flush_oldseqcache{
   my ($self) = @_;

   $self->{'oldseqcache'} = {};
}

=head2 _is_oldseqcache

 Title   : _is_oldseqcache
 Usage   :
 Function:
 Example :
 Returns : 
 Args    :


=cut

sub _is_oldseqcache{
   my ($self,$hashval) = @_;

   return $self->{'oldseqcache'}->{$hashval};

}
