#-----------------------------------------------------------------------------
# PACKAGE : Bio::Root::Object.pm
# AUTHOR  : Steve A. Chervitz (sac@genome.stanford.edu)
# CREATED : 23 July 1996
# STATUS  : Alpha
#            
# For documentation, run this module through pod2html 
# (preferably from Perl v5.004 or better).
#
# AUTOLOADing: This module has a number of seldom-used, highly experimental,
# and deprecated methods that are AUTOLOADed in order to reduce
# the overhead of loading the module. To disable AUTOLOADING 
# (and always load all methods), comment out the __END__ line.
# To reactivate AUTOLOADing (and speed up loading of your Perl 
# scripts), uncomment the __END__ line and run the autosplit.pl
# program on this module. If you installed this module using the
# Makefile.PL of the Blast distribution, AUTOLOADing should be enabled.
#
# MODIFIED: 
#    0.031, 2 Sep 1998, sac:
#      * Documentation changes only. Wrapped the data member docs
#        at the bottom in POD comments which fixes compilation bug
#        caused by commenting out __END__.
#
#    0.03, 16 Aug 1998, sac:
#      * Calls to warn() or throw() now no longer result in Err.pm objects
#        being attached to the current object. For discussion about this 
#        descision, see comments under _set_err().
#      * Added the -RECORD_ERR constructor option and Global::record_err() 
#        method to enable the attachment of Err.pm object to the current 
#        object.
#      * Minor bug fixes with parameter handling (%param -> @param).
#      * Added note about AUTOLOADing.
#
#    0.023, 20 Jul 1998, sac:
#      * Changes in Bio::Root::IOManager::read().
#      * Improved memory management (destroy(), DESTROY(), and changes
#        in Bio::Root::Vector.pm).
#
#    0.022, 16 Jun 1998, sac:
#      * Changes in Bio::Root::IOManager::read().
#
#    0.021, May 1998, sac:
#      * Touched up _set_clone().
#      * Refined documentation in this and other Bio::Root modules
#        (converted to use pod2html in Perl 5.004)
#
# Copyright (c) 1996-98 Steve A. Chervitz. All Rights Reserved.
#           This module is free software; you can redistribute it and/or 
#           modify it under the same terms as Perl itself.
#           Retain this notice and note any modifications made.
#-----------------------------------------------------------------------------

package Bio::Root::Object;

require 5.002;
use Bio::Root::Global qw(:devel $AUTHORITY $CGI);
use Exporter ();

@EXPORT_OK = qw($VERSION &find_object &stack_trace &containment &_rearrange);  
%EXPORT_TAGS = ( std => [qw(&stack_trace &containment)] );

use strict;
use vars qw($ID $VERSION %Objects_created $revision);  
# %Objects_created can be used for tracking all objects created.
# See _initialize() for details.

$ID      = 'Bio::Root::Object';
$VERSION = 0.031;
$revision = '$Id: Object.pm,v 1.2 1999/01/26 17:09:28 birney Exp $';  #'

### POD Documentation:

=head1 NAME

Bio::Root::Object - A core Perl 5 object.

=head1 SYNOPSIS

=head2 Object Creation

    require Bio::Root::Object;
 
    $dad = new Bio::Root::Object();
    $son = new Bio::Root::Object(-name    => 'Junior', 
			         -parent => $dad,
			         -make    => 'full');


See the L<new>() method for a complete description of parameters.
See also L<USAGE>.


=head1 INSTALLATION

At present, there is no special distribution package for this module.
It is included and installed with my Blast module distribution:

   http://genome-www.stanford.edu/perlOOP/bioperl/blast

Otherwise, you can obtain the source from the following URL:

   http://genome-www.stanford.edu/perlOOP/bioperl/lib/Bio/Root

Download all C<*.pm> files and place them in your module library in 
the proper location (e.g., C<my_lib/Bio/Root/>). Bio::Root::Object.pm
needs to be B<autosplit> since some experimental methods are AUTOLOADed.
I have created some scripts that help with autospliting available at:

   http://genome-www.stanford.edu/perlOOP/bioperl/lib/Bio/admin/autosplit.pl
   http://genome-www.stanford.edu/perlOOP/bioperl/lib/Bio/admin/split_modules.pl


=head1 DESCRIPTION

This module attempts to encapsulate the "core" Perl5 object: What are the
key data and methods ALL (or at least most) Perl5 objects should have? 

Not all objects need to derive from Bio::Root::Object.pm. However, when generating
lots of different types of potentially complex objects which should all 
conform to a set of basic expectations, this module may be handy. At
the very least, this module saves you from re-writing the L<new>() method
for each module you develop. It also permits a robust way to handle 
C<-tag =E<gt> value> method arguments via the L<_rearrange>() method
and provides a object-oriented way handle exceptions via the L<throw>()
method.


=head2 Fault-Tolerant Objects

A major motivation for this module was to promote the creation of robust,
fault-tolerant Perl5 objects. The L<throw>() method relies on Perl's built-in 
C<eval{}/die> exception mechanism to generate fatal exceptions.
The data comprising an exception is managed by the B<Bio::Root::Err.pm> 
module, which essentially allows the data thrown by a C<die()> event to be
wrapped into an object that can be easily examined and possibly re-thrown.

The intent here is three-fold:

=over 4

=item 1 Detailed error reporting. 

Allow objects to report detailed information about the error condition 
(who, what, where, why, how).

=item 2 Handle complex errors in objects.

The goal is to make it relatively painless to detect and handle the wide
variety of errors possible with a complex Perl object.
Perl's error handling mechanism is a might clunky when it comes to
handling complex errors within complex objects.

=item 3 Efficient & easy error handling.

To enable robust error handling without incurring a significant
performance penalty in the resulting code. Ideally, error handling
code should be transparent to the cpu until and unless an error
arises. 

=back

These goals may at times be at odds and I am not claiming 
to have achieved the perfect balance. Ultimately, we want self-
sufficient objects able to deal with their own errors. 
This area should improve as the module, and Perl, evolve. 
One possible modification might be to utilize Graham Barr's B<Error.pm>
module or Torsten Ekedahl's B<Experimental::Exception.pm> module 
(see L<Other Exception Modules>).
Technologies such as these may eventually be
incorporated into future releases of Perl. The exception handling 
used by Bio::Root::Object.pm can be expected to change as Perl's 
exception handling mechanism evolves.

NOTE: In this discussion and elsewhere in this module, the terms 
"Exception" and "Error" are used interchangeably to mean "something went
wrong" either as a result of user input or internal processing.

=head1 USAGE

For a usage demo of Bio::Root::Object.pm, see:

    http://genome-www.stanford.edu/perlOOP/bioperl/lib/Bio/drivers/object/


=head1 DEPENDENCIES

As the B<Bio::Root::Object.pm> does not inherit from any modules 
but wraps (i.e., provides an interface and delegates 
functionality to) other modules in the Bio::Root:: hierarchy:

   Module                    Purpose
   --------------------      ------------------------------------
   Bio::Root::Err.pm         Exception handling
   Bio::Root::IOManager.pm   Input/output of object data or error data
   Bio::Root::Xref.pm        Arbitrary links between objects

All of these modules are used only when necessary.
B<Bio::Root::Err.pm> is an object representing an exception.
B<Bio::Root::IOManager.pm> and B<Bio::Root::Xref.pm> are more experimental. They are
utilized via delegation, which permits them to be developed and utilized
independently of Bio::Root::Object.pm.

Since this module is at the root of potentially many different objects
in a particular application, efficiency is important. Bio::Root::Object.pm is intended
to be a lightweight, lean and mean module. For this reason, a fair number of
methods, including all with "Experimental" status, are AUTOLOADED.


=head1 SPECIAL NOTES

The API for this module is under development and therefore apt to change in future versions.
The notion of what should be public and what should be private is still open for
debate, hence the inclusion of apparently private methods in the API.

This module was formerly called Obj::Object.pm. The new name reflects its
use with in the Bio::Object framework and also makes it easier to keep all
Bioperl modules together within a defined namespace.

=head1 FEEDBACK

The API for this module is under development. User feedback is an integral part 
of this evolutionary process so send your comments to Steve Chervitz
(sac@genome.stanford.edu). Thanks!

=head1 AUTHOR

Steve A. Chervitz [sac@genome.stanford.edu].

=head1 VERSION

Bio::Root::Object.pm, 0.031


=head1 TODO

=over 0

=item * Experiment with other Exception classes.

Consider incorporating a more widely-used Error/Exception module 
(see L<Other Exception Modules>).

=item * Think about integration with Data::Dumper.pm for persisting objects.

=back

=head1 SEE ALSO

  Bio::Root::Err.pm       - Error/Exception object
  Bio::Root::IOManager.pm - Input/Output manager object
  Bio::Root::Vector.pm    - Manages dynamic lists of objects
  Bio::Root::Xref.pm      - Cross-reference object
  Bio::Root::Global.pm    - Manages global variables/constants

  The above modules are available from
    http://genome-www.stanford.edu/perlOOP/bioperl/lib/Bio/Root

  General Links:
  http://bio.perl.org/                                   - Bioperl Project Homepage 
  http://genome-www.stanford.edu/perlOOP/bioperl/schema/ - Bio::Object schema 


=head2 Other Exception Modules

  Experimental::Exception.pm   - ftp://ftp.matematik.su.se/pub/teke/
  Error.pm                     - http://www.cpan.org/authors/id/GBARR/
  Throwable.pm                 - mailto:kstevens@globeandmail.ca

  http://genome-www.stanford.edu/perlOOP/exceptions.html 

=head1 COPYRIGHT

Copyright (c) 1996-98 Steve A. Chervitz. All Rights Reserved.
This module is free software; you can redistribute it and/or 
modify it under the same terms as Perl itself.

=cut


# 
## 
### 
#### END of main POD documentation. '
###
##
# 


=head1 APPENDIX

Methods beginning with a leading underscore are considered private
and are intended for internal use by this module. They are
B<not> considered part of the public interface and are described here
for documentation purposes only.

=cut


#######################################################
#               CONSTRUCTOR/DESTRUCTOR                #
#######################################################


=head2 new

 Usage     : new CLASS_NAME [ %named_parameters];
 Purpose   : Creates a blessed object reference (hash) for the indicated class
           : and calls _initialize() for the class passing it all parameters.
 Example   : $obj = new Bio::Root::Object 'george';
           : $obj = Bio::Root::Object->new(-name    => 'L56163',
	   : 			           -parent => $obj2 );
           : $obj = Bio::Root::Object->new();
 Returns   : Blessed hash reference.
 Argument  : Named parameters:  (PARAMETER TAGS CAN BE UPPER OR LOWERCASE).
           : (all are optional)
           :  -NAME       => arbitrary string to identify an object;
           :                 should be unique within its class.
           :  -PARENT    => blessed reference for an object that
           :                 is responsible for the present object
           :                 (e.g., a container).
           :  -MAKE       => string to specify special constructor option.
           :  -STRICT     => integer (level of strictness: -1, 0, 1, 2).
           :  -TERSE      => boolean (activate terse mode for minimal reporting).
           :  -VERBOSE    => boolean (activate verbose mode for maximal reporting).
           :  -VERBOSE    => boolean (activate verbose mode for maximal reporting).
           :                 (in verbose mode, calls to $self->warn() will
           :                  output stack traces for warnings instead of
           :                  just the message/notes).
           :  -OBJ        => object reference for an object to be cloned.
           :  -RECORD_ERR => boolean (if true, attach all Err.pm objects generated by
	   :                 warn() or throw() calls to the present object; 
	   :		     default = false).
           :
 Comments  : This method creates blessed HASH references.
           : An object is free to define its own strict, terse, and verbose
           : behavior as well as its own make (constructor) options.

See Also   : L<_initialize>(), L<name>(), L<parent>(), L<make>(), L<strict>()

=cut

#----------
sub new {
#----------
    my($class, @param) = @_;
    my $self = {};
    bless $self, ref($class) || $class;
    $DEBUG==2 && print STDERR "CREATING $self";
    $self->_initialize(@param);
    $self;
}


=head2 _initialize

 Usage     : n/a; automatically called by Bio::Root::Object::new()
 Purpose   : Initializes key Bio::Root::Object.pm data (name, parent, make, strict).
           : Called by new().
 Returns   : String containing the -MAKE constructor option or 'default' 
           : if none defined (if a -MAKE parameter is defined, the value
           : returned will be that obtained from the make() method.)
           : This return value saves any subclass from having to call
           : $self->make() during construction. For example, within a
           : subclass _initialize() method, invoke the Bio::Root::Object::
           : initialize() method as follows:
           :    my $make = $self->SUPER::_initialize(@param);
 Argument  : Named parameters passed from new()
           :  (PARAMETER TAGS CAN BE ALL UPPER OR ALL LOWER CASE).
 Comments  : This method calls name(), make(), parent(), strict(), index()
           : and thus enables polymorphism on these methods. To save on method
           : call overhead, these methods are called only if the data need 
           : to be set.
           :
           : The _set_clone() method is called if the -MAKE option includes
           : the string 'clone' (e.g., -MAKE => 'clone').
           :
           : The index() method is called if the -MAKE option includes
           : the string 'index'. (This is an experimental feature)
           : (Example: -MAKE => 'full_index').
           :
           : NOTE ON USING _rearrange():
           :
           : _rearrange() is a handy method for working with tagged (named)
           : parameters and it permits case-insensitive in tag names
           : as well as handling tagged or un-tagged parameters.
           : _initialize() does not currently call _rearrange() since
           : there is a concern about performance when setting many objects.
           : One issue is that _rearrange() could be called with many elements 
           : yet the caller is interested in only a few. Also, derived objects 
           : typically invoke _rearrange() in their constructors as well. 
           : This could particularly degrade performance when creating lots 
           : of objects with extended inheritance hierarchies and lots of tagged
           : parameters which are passes along the inheritance hierarchy.
           :
           : One thing that may help is if _rearrange() deleted all parameters
           : it extracted. This would require passing a reference to the param list
           : and may add excessive dereferencing overhead.
           : It also would cause problems if the same parameters are used by
           : different methods or objects.

See Also   : L<new>(), L<make>(), L<name>(), L<parent>(), L<strict>(), L<index>(), L<_rearrange>(), L<_set_clone>(), L<terse>(), L<verbose>()

=cut

#----------------
sub _initialize {
#----------------
    local($^W) = 0;
    my($self, %param) = @_;
    
    my($name, $parent, $make, $strict, $terse, $verbose, $obj, $record_err) = (
	($param{-NAME}||$param{'-name'}), ($param{-PARENT}||$param{'-parent'}), 
	($param{-MAKE}||$param{'-make'}), ($param{-STRICT}||$param{'-strict'}),
	($param{-TERSE}||$param{'-terse'}), ($param{-VERBOSE}||$param{'-verbose'}),
        ($param{-OBJ}||$param{'-obj'}, $param{-RECORD_ERR}||$param{'-record_err'})
					  );

    ## See "Comments" above regarding use of _rearrange().
#	$self->_rearrange([qw(NAME PARENT MAKE STRICT TERSE VERBOSE OBJ)], %param);

    $DEBUG and do{ print STDERR ">>>> Initializing $ID (${\ref($self)}) ",$name||'anon';<STDIN>};

    if(defined($make) and $make =~ /clone/i) { 
	$self->_set_clone($obj);

    } else {
	$name ||= ($#_ == 1 ? $_[1] : '');  # If a single arg is given, use as name.

	## Another performance issue: calling name(), parent(), strict(), make()
	## Any speed diff with conditionals to avoid method calls?
	
	$self->name($name) if $name; 
	$self->parent($parent) if $parent;
	$self->{'_strict'}  = $strict  || '';
	$self->{'_terse'}   = $terse   || '';
	$self->{'_verbose'} = $verbose || '';
	$self->{'_record_err'} = $record_err || '';

	if($make) {
	    $make = $self->make($make);
	
	    # Index the Object in the global object hash only if requested.
	    # This feature is not used much. If desired, an object can always 
	    # call Bio::Root::Object::index()  any time after construction.
	    $self->index() if $make =~ /index/; 
	}
    }

    $DEBUG and print STDERR "---> Initialized $ID (${\ref($self)}) ",$name,"\n";

    ## Return data of potential use to subclass constructors.
#    return (($make || 'default'), $strict);   # maybe (?)
    return $make || 'default';
}



=head2 DESTROY

 Usage     : n/a; automatically called by Bio::Root::Object::destroy()
 Purpose   : Destroys objects created by this module (stored in fields 
           : _err and _io), and breaks links to associated objects
           : (stored in fields _parent and _xref).
 Argument  : n/a
 Comments  : Setting the global $DEBUG to 2 will print messages upon
           : object destruction.

See Also   : L<destroy>() 

=cut

#-----------
sub DESTROY { 
#-----------
## Note: Cannot delete scrObj and xref object refs since they are not 
##       owned by this object, merely associated with it.
    my $self=shift; 
    $self->{'_parent'} = undef; 
    $self->{'_xref'}   = undef;

    $DEBUG==2 && print STDERR "DESTROYING $self ${\$self->name} by $ID";

    if(ref($self->{'_io'})) {
	$self->{'_io'}->destroy;
	$self->{'_io'} = undef;
    } 

    if(ref($self->{'_err'})) {
	$self->{'_err'}->remove_all;
	%{$self->{'_err'}} = ();
	$self->{'_err'} = undef;
    }
}  


=head2 destroy

 Usage     : destroy()
 Purpose   : To destroy an object and signal the GC to reclaim its memory.
 Example   : $self->destroy();
 Returns   : undef
 Argument  : n/a
 Comments  : Invokes the object's destructor method (DESTROY)
           : and undefines the referenced hash for the object.

See Also   : L<DESTROY>() 

=cut

#-------------'
sub destroy {
#-------------
    my $self = shift;
    local ($^W) = 0;
    $self->DESTROY;
    %$self = ();
}



#################################################################
#                    ACCESSORS & INSTANCE METHODS
#################################################################


=head2 _rearrange

 Usage     : _rearrange( array_ref, list_of_arguments)
 Purpose   : Rearranges named parameters to requested order.
 Example   : $self->_rearrange([qw(SEQUENCE ID DESC)],@param);
           : Where @param = (-sequence => $s, 
	   :                 -id       => $i, 
	   :	             -desc     => $d);
 Returns   : @params - an array of parameters in the requested order.
           : The above example would return ($s, $i, $d)
 Argument  : $order : a reference to an array which describes the desired
           :          order of the named parameters.
           : @param : an array of parameters, either as a list (in
           :          which case the function simply returns the list),
           :          or as an associative array with hyphenated tags
           :          (in which case the function sorts the values 
           :          according to @{$order} and returns that new array.)
	   :	      The tags can be upper, lower, or mixed case
           :          but they must start with a hyphen (at least the
           :          first one should be hyphenated.)
 Source    : This function was taken from CGI.pm, written by Dr. Lincoln
           : Stein, and adapted for use in Bio::Seq by Richard Resnick and
           : then adapted for use in Bio::Root::Object.pm by Steve A. Chervitz.
 Comments  : (SAC)
           : This method may not be appropriate for method calls that are
           : within in an inner loop if efficiency is a concern.
           :
           : Parameters can be specified using any of these formats:
           :  @param = (-name=>'me', -color=>'blue');
           :  @param = (-NAME=>'me', -COLOR=>'blue');
           :  @param = (-Name=>'me', -Color=>'blue');
           :  @param = ('me', 'blue');  
           : A leading hyphenated argument is used by this function to 
           : indicate that named parameters are being used.
           : Therefore, the ('me', 'blue') list will be returned as-is.
           :
	   : Note that Perl will confuse unquoted, hyphenated tags as 
           : function calls if there is a function of the same name 
           : in the current namespace:
           :    -name => 'foo' is interpreted as -&name => 'foo'
	   :
           : For ultimate safety, put single quotes around the tag:
	   :    ('-name'=>'me', '-color' =>'blue');
           : This can be a bit cumbersome and I find not as readable
           : as using all uppercase, which is also fairly safe:
	   :    (-NAME=>'me', -COLOR =>'blue');
	   :
           : Personal note (SAC): I have found all uppercase tags to
           : be more managable: it involves less single-quoting,
           : the code is more readable, and there are no method naming conlicts.
           : Regardless of the style, it greatly helps to line
	   : the parameters up vertically for long/complex lists.

See Also   : L<_initialize>() 

=cut

#----------------'
sub _rearrange {
#----------------
    my($self,$order,@param) = @_;
    
    # If there are no parameters, we simply wish to return
    # an empty array which is the size of the @{$order} array.
    return ('') x $#{$order} unless @param;
    
    # If we've got parameters, we need to check to see whether
    # they are named or simply listed. If they are listed, we
    # can just return them. 

    return @param unless (defined($param[0]) && $param[0]=~/^-/); 
    
    # Tester
#    print "\n_rearrange() named parameters:\n";
#    my $i; for ($i=0;$i<@param;$i+=2) { printf "%20s => %s\n", $param[$i],$param[$i+1]; }; <STDIN>;

    # Now we've got to do some work on the named parameters.
    # The next few lines strip out the '-' characters which
    # preceed the keys, and capitalizes them.
    my $i;
    for ($i=0;$i<@param;$i+=2) {
	$param[$i]=~s/^\-//;
	$param[$i]=~tr/a-z/A-Z/;
    }
    
    # Now we'll convert the @params variable into an associative array.
    local($^W) = 0;  # prevent "odd number of elements" warning with -w.
    my(%param) = @param;
    
    my(@return_array);
    
    # What we intend to do is loop through the @{$order} variable,
    # and for each value, we use that as a key into our associative
    # array, pushing the value at that key onto our return array.
    my($key);
    
    foreach $key (@{$order}) {
	my($value) = $param{$key};
	delete $param{$key};
	push(@return_array,$value);
    }
    
#    print "\n_rearrange() after processing:\n";
#    my $i; for ($i=0;$i<@return_array;$i++) { printf "%20s => %s\n", ${$order}[$i], $return_array[$i]; } <STDIN>;

    return (@return_array);
}



=head2 name

 Usage     : $object->name([string]);
 Purpose   : Set/Get an object's common name. 
 Example   : $myName = $myObj->name;
           : $myObj->name('fred');
 Returns   : String consisting of the object's name or 
           : "anonymous <CLASSNAME>" if name is not set.
           : Thus, this method ALWAYS returns some string.
 Argument  : String to be used as the common name of the object. 
           : Should be unique within its class.

See also   : L<has_name>()

=cut

#---------
sub name {
#---------
    my $self = shift;

#    $DEBUG and do{ print STDERR "\n$ID: name(@_) called.";<STDIN>; };

    if (@_) { $self->{'_name'} = shift }
    return defined $self->{'_name'} ? $self->{'_name'} : 'anonymous '.ref($self);
}



=head2 parent

 Usage     : $object->parent([object | 'null']);
 Purpose   : Set/Get the current object's source object. 
           : An object's source object (parent) is defined as the object 
           : that is responsible for creating the current object (child).
           : The parent object may also have a special mechanism for
           : destroying the child object. This should be included
           : in the parent object's DESTROY method which should end with a
           : call to $self->SUPER::DESTROY.
 Example   : $myObj->parent($otherObject);
 Returns   : Object reference for the parent object or undef if none is set.
 Argument  : Blessed object reference (optional) or the string 'null'.
           :  'null' = sets the object's _parent field to undef,
           :           breaking the child object's link to its parent.
 Throws    : Exception if argument is not an object reference or 'null'. 
 Comments  : This method may be renamed 'parent' in the near future.
           : When and if this happens, parent() will still be supported but
           : will be deprecated.

See also   : L<DESTROY>(), L<destroy>()

=cut

#------------'
sub parent {
#------------
    my ($self) = shift;
    if (@_) { my $arg = shift; 
	     if(ref $arg) {
		 $self->{'_parent'} = $arg;
	     } elsif($arg =~ /null/i) {
		 $self->{'_parent'} = undef;
	     } else {
		 $self->throw("Can't set parent using $arg: Not an object");
	     }
	 }
    $self->{'_parent'};
}


=head2 src_obj

 Usage     : $object->src_obj([object | 'null']);
           : THIS METHOD IS NOW DEPRECATED. USE parent() INSTEAD.
 Purpose   : Set/Get the current object's source object (parent).
 
=cut

#------------'
sub src_obj {
#------------
    my ($self) = shift;
    $self->warn("DEPRECATED METHOD src_obj() CALLED. USE parent() INSTEAD.\n");
    $self->parent(@_);
}


=head2 has_name

 Usage     : $object->has_name();
 Purpose   : To determine if an object has a name.
 Returns   : True (1) if the object's {'Name'} data member is defined.
           : False otherwise.
 Comments  : One may argue, why not just use the name() method as a
           : combination setter/getter? has_name() is necessary for 
           : the following reasons:
           :   (1) If an object's name is not defined, name() returns 
           :        "anonymous <CLASSNAME>".
           :   (2) If an object's name is 0 (zero) or '' (empty string), 
           :       conditionals that simply check name() would fail incorrectly.

See also   : L<name>()

=cut

#--------------'
sub has_name { my $self = shift; return defined $self->{'_name'}; }
#--------------



=head2 make

 Usage     : $object->make([string]);
 Purpose   : Set/Get an object's constructor option.
           : make() is intended for use during object construction
           : to essentially permit alternate constructors since 
           : Perl doesn't have a built-in mechanism for this.
 Example   : $make = $object->make();
           : $object->make('optionA');
 Returns   : String consisting of the object's make option
           : or 'default' if make is not set.
           : Thus, this method ALWAYS returns some string.
 Argument  : String to be used as an option during object construction.
 Comments  : A typical use of a make option is when cloning an object
           : from an existing object. In this case, the new() method
           : is called with -MAKE => 'clone'.

See also   : L<_initialize>(), L<clone>()

=cut

#----------'
sub make { 
#----------
    my $self = shift;
    if(@_) { $self->{'_make'} = shift; }
    $self->{'_make'} || 'default'; 
}



=head2 throw

 Purpose   : Generate, report, and set a fatal error on the object.
           : Uses Perl's die() function to report error data.
           : This does not invalidate the object but will crash the script
           : unless it is trapped with eval{}. 
           : (fatal = un-recoverable)'
 Usage     : $object->throw([arguments for _set_err()])
 Returns   : die()s with the contents of the error object in a string.
           : If the strict() indicator is less than zero, or if the 
           : global $WARN_ON_FATAL value is non-zero,
           :   die is not called and the error is printed to STDERR.
           : If the global $DONT_WARN is non-zero, the error will not
           : be printed.
 Argument  : See _set_err()
 Comments  : Calling $self->throw() method will create an Bio::Root::Err.pm object 
           : and attach it to the $self object. This permits the object to be 
           : subsequently queried about the error for any custom handling 
           : to be performed. 
           :
           : There a number of error-generating options available:
           :
           :   1) $object->throw(<ERROR DATA>);
           :   2) $object->fatal(<ERROR DATA>);  # deprecated
           :   3) &Bio::Root::Err::throw($object, ERROR_DATA);
           :
           :   (ERROR_DATA = arguments for _set_err())
           :
           : The second option is the same as the first except that the 
           : Err object will be have its type set to 'FATAL' instead 
           : of 'EXCEPTION'. This is a remnant from an older polling mechanism
           : in which an object or its client can call $object->valid() to 
           : decide whether or not to continue processing. This mechanism is 
           : generally discouraged due to the complexities that polling involves. 
           : For example, a problem arises when a trivial exception (caused by 
           : improper usage by an ignorant client) invalidates an object
           : with perfectly valid data. This situation turns out to be 
           : rather common. Also, polling does not scale well.
           :
           : The third option (&Bio::Root::Err::throw()) is the same as the
           : first except that the current object will NOT have an Bio::Root::Err.pm
           : attached to it. This is the more traditional way to generate 
           : exceptions. Any module or script can "use Bio::Root::Err qw(:std)" 
           : to throw exceptions this way.
           :
           : Some auxilliary issues:
           :   * It would be great if Perl could throw an object reference with die().
           :     This would permit more intelligent exception handlers. For now the
           :     Err object is reconstructed from the output of Err::string().
           :
           : The use of the $WARN_ON_FATAL and $DONT_WARN flags is considered
           : experimental. These provide a convenient way to turn off warnings
           : globally for a script. Using $obj->strict(-1) provides an object-specific 
           : 'warn_on_fatal' flag.
           : 
           : All errors are reported to STDERR. 
           : Redirection to an alternate location for storing errors
           : should be performed by calling print_err with a WHERE flag:
           : (-WHERE=>$self->fh('err')).

See also   : L<_set_err>(), L<warn>(), L<fatal>(), L<strict>(), L<print_err>(), B<Bio::Root::Global.pm>

=cut

#----------
sub throw {
#----------
    my($self,@param) = @_;

#    printf "Throwing exception for object %s \"%s\"\n",ref($self),$self->name;
#    print "param: @param";<STDIN>;

    if($self->strict() < 0 or $WARN_ON_FATAL) {
	if($DONT_WARN) {
	    $self->_set_err(@param);
	} else {
	    print STDERR $self->_set_err(@param)->string;
	}
    } else {
	die $self->_set_err(@param)->string;
    }
    0;
}


=head2 warn

 Usage     : $object->warn([arguments for _set_err()])
 Purpose   : Generate, report, and set a recoverable error on the object.
 Returns   : Prints the contents of the error to STDERR and returns false (0).
           : If the strict() indicator is set to 2, or if the global $FATAL_ON_WARN
           : value is non-zero, warn() calls are converted into throw() calls.
           : If terse() is true, nothing is printed but a warning is still set.
           : If verbose() is true, the full error listing is show
           :  (message, note, tech note, containment hierarchy, stack).
           : Otherwise, only the message, note, and tech note are shown.
           : If the global $DONT_WARN is non-zero, the error will not
           : be printed.
 Argument  : See _set_err()
 Comments  : The return value is experimental. Typically, warnings are not
           : programatically trappable: a method will issue a warning and 
           : then go about its business. By allowing
           : warn() calls to evaluate to zero, a method can halt execution 
           : by returning a warning to signal the warning without setting a 
           : fatal error on itself. Still, returning a 0 does not guarantee
           : the exception will be noticed. This sort of polling-based
           : exception handling is generally frowned upon. Use throw()
           : unless the condition is truly inconsequential.
           :
           : The use of the $FATAL_ON_WARN and $DONT_WARN flags is considered
           : experimental. These provide a convenient way to turn off warnings
           : globally for a script.
           :
           : All errors are reported to STDERR. 
           : Redirection to an alternate location for storing errors
           : should be performed by calling print_err with a WHERE flag:
           : (-WHERE=>$self->fh('err')).

See also   : L<_set_warning>(), L<throw>(), L<strict>(), L<terse>(), L<verbose>(), B<Bio::Root::Global.pm>

=cut

#---------
sub warn {
#---------
    my($self) = shift;

    #print "\n\nWarning with:\n @_";<STDIN>;

    if($self->strict == 2 or $FATAL_ON_WARN) {
	$self->throw(@_);

    } else {
	if($DONT_WARN or $self->terse) {
	    $self->_set_warning(@_);
	} else {
	    if($self->verbose) {
		print STDERR $self->_set_warning(@_)->string(-BEEP=>1, -CURRENT=>1);
	    } else {
		print STDERR $self->_set_warning(@_)->string(-SHOW=>'msgnotech', -CURRENT=>1);
	    }
	}
    }
    0;
}


=head2 err

 Usage     : $self->err([$data], [$delimit])
 Purpose   : Check for exceptions/warnings and get data about them.
           : (object validation and error data retrieval)
 Example   : $self->err && print "has err";
           : $errCount = $self->err('count');
           : $errMsgs  = $self->err('msg',"\t");
           : @errNotes = $self->err('note');
 Returns   : One of the following:
           :   1. If no arguments are given
           :          a. If the object has an error, the err data member is 
           :             returned (this is an Bio::Root::Err.pm object),
           :          b. otherwise, undef is returned.
           :   2. The number of Errs in the object's err data member (if $data eq 'count').
           :   3. A string containing data from a specific field from an object's err member.
           :      -- If the object contains multiple errors, data for all errors will be
           :         strung together in reverse chronological order with each error's data 
           :         preceeded by "Error #n\n" and followed by two delimiters.
           :   4. A list containing data from a specific field from an object's err member.
           :      -- If the object contains multiple errors, data for all errors will be
           :         added in reverse chronological order as separate elements in the list 
           :         with NO "Error #n\n" identifier. Individual err list data 
           :         (note,tech,stack) will be tab-delimited.
 Arguments : $data    = The name of a specific Err data member (see %Bio::Root::Err::ERR_FIELDS)
           :            OR 'count'.
           : $delimit = The delimiter separating a single Err's list data member's elements.
           :            Default is "\n". For multi-error objects, two of these
           :            delimiters separate data from different errors.
           :            If wantarray is true or delimiter is 'list', data from multiple
           :            errors will be returned as a list
           :
 Comments  : Since Err objects are now fatal and are not attached to the object by default, 
           : this method is largely moot. It is a relic from the former 
           : error "polling" days.
           : It is handy for accessing non-fatal warnings thrown by the object,
           : or in situations where fatal errors are converted to warnings
           : as when $self->strict is -1 or $WARN_ON_FATAL is true.
           : (Note: an object now only attaches Err objects to itself when 
	   : constructed with -RECORD_ERR =>1 or if the global $RECORD_ERR is true).
           :
           : This method is intended mainly to test whether or not an object
           : has any Err objects associated with it and if so, obtaining the 
           : Err object or specific data about it.
           : For obtaining ALL data about an error, use err_string().
           : For more detailed manipulations with the Err data, retrieve the
           : Err object and process its data as necessary.

See also   : L<err_string>(), L<print_err>(), B<Bio::Root::Err.pm>::get_all

=cut

#----------
sub err {
#----------
    my( $self, $data, $delimit) = @_;

    return undef unless defined $self->{'_err'};
    
    $data    ||= 'member';
#    $delimit ||= (wantarray ? 'list' : "\n");
    $delimit ||= "\n";

    $data eq 'member' and return $self->{'_err'}; 
    $data eq 'count'  and return $self->{'_err'}->size();

    return $self->{'_err'}->get_all($data, $delimit );
}	


=head2 _set_err

 Purpose   : To attach an Bio::Root::Err.pm object to the current object.
 Usage     : This is an internal method and should not be called directly
           : Bio::Root::Object::throw() and Bio::Root::Object::warn().
           : $object->_set_err( msg)  
           : $object->_set_err( msg, note) 
           : $object->_set_err( msg, note, tech)
           : $object->_set_err( -MSG  =>"main message", 
	   :	                -TECH =>"technical note only")
           : $object->_set_err($object->err())  # Transfers pre-existing err
           : $object->_set_err()                # Re-sets an object's error state 
           :                                    # (Public method: clear_err())
 Example   : $self->throw("Data not found.");
           : To throw an error:
           : $myData eq 'foo' || return $self->throw("Data is not 'foo'.")
 Returns   : Object reference to the newly created Bio::Root::Err.pm object
           : via call to _set_err_state().
           :
 Argument  : @param may be empty, or contain a single error object, 
           : named parameters, or a list of unnamed parameters for 
           : building an Bio::Root::Err object.
           :   msg  = string, basic description of the exception.
           :   note = string, additional note to indicate cause or exception
           :          or provide information about how to fix/report it.
           :   tech = string, addition note with technical information
           :          of interest to developer.
           :
           : When using unnamed parameters, the number of items in @param 
           : is used as a "syntactic sugar" to indicate which fields in the 
           : err object to set (1 = msg, 2 = msg + note, 3 = msg + note + tech)
           : Calling _set_err() with no arguments clears the {'_err'} and 
           : {'_errState'} data members and destroys the Err object.
           : 
 Comments  : NEW VERSION: NOT ATTACHING EXCEPTIONS TO THE OBJECT.
           : Since exceptions are fatal, it is more expedient for the calling code
           : to handle them as they arise. Attaching exceptions to the objects
           : that generated them implies that the object assumes responsibility for 
           : any error it might throw, which is not usually appropriate and is 
           : difficult to manage.
           :
           : The new code now by default will not attach Err objects to the 
           : object that. Attaching Err objects can be enabled using the -RECORD_ERR
           : constructor option or Bio::Root::Global::record_err() to turn on 
           : Err attaching for all objects in a script.
           :
           : Attaching exceptions to the objects that produced them is considered
           : non-standard and must be explicitly requested. This behavior might be 
           : useful in situations where one runs some code in an unsupervised 
           : setting and needs a means for reporting all warnings/errors later.
           :
           : One problem with attaching Err objects is that if an object is contained 
           : within another object, the containing object will not know about the 
           : warning unless it polls all of it contained objects, (bad design).
           : One could propagate the warning through the containment hierarchy
           : but the hierarchy may not be accessible to the objects themselves:
           : a given object may not know where it is contained (i.e, it may not 
           : have a parent()).
           :    
 Bugs      : * For objects with multiple errors, there is currently
           :   no way to delete a specific error. 
           :
           : * To transfer an error between objects, you can use 
           :   $self->warn($object->err) or $self->throw($object->err) or
           :   $self->_set_err($object->err) to not generate a warning.

See also   : L<_set_warning>(), L<warn>(), L<throw>(), L<fatal>(), L<_set_err_state>(), B<Bio::Root::Err.pm>

=cut

#--------------
sub _set_err {   
#--------------
    my($self, @param) = @_;
    
    require Bio::Root::Err; import Bio::Root::Err qw(:data);

    ## Re-set the object's 'Err' member and clear the 'ErrState'
    ## if no data was provided. 
    if(!@param) {
	 defined $self->{'_err'} and $self->{'_err'}->remove_all;
	 $self->{'_err'} = undef;
	 $self->{'_errState'} = undef;
	 return;
     } 
    
    # If client specifies any error field (ie. -MSG=>'Bad data') the 'custom' constructor
    # is used. If no error field is specified by name, the constructor is selected based on
    # the number of arguments in @param.

    local($^W) = 0;  
    my $constructor = 'custom';
    if( !grep exists $Bio::Root::Err::ERR_FIELDS{uc($_)}, @param) {
	$constructor = scalar @param;
    }
    
    $DEBUG and do{ print STDERR "\n$ID: setting error ${\ref($self)} with constructor = $constructor"; <STDIN>; };
    
    ## Adjust the constructor number if STACK_NUM was given in @param (for warnings).
    ## This is a bit of a hack, but will do for now.
    ## The constructor needs to be adjusted since it included the "-STACK_NUM=>#" data
    ## which increases the number of arguments by 2 and is not to be included in 
    ## the exception's data.

    my $stackNum = 3;  
    if($constructor =~ /\d/ and grep (/STACK_NUM/i, @param)) { 
	$constructor -= 2;  ## Since STACK_NUM data was appended to @_.
	$stackNum=$param[$#param];
    }

    my ($err);
    eval {
	local $_ = $constructor;
	## Switching on the number of items in @param (now in $_).
	SWITCH: {
	    ## Single argument: $param[0] is either an Err object or a message.
	    /1/ && do{ 	if((ref($param[0]) =~ /Err/)) {
#		print "$ID: cloning err for object ${\ref $self}, \"${\$self->name}\"\n";<STDIN>;
		$err = $param[0]->clone(); ## Cloning error object.
	    } else {
		$err = new Bio::Root::Err(-MSG     =>$param[0], 
					  -STACK   =>scalar($self->stack_trace($stackNum)),
					  -CONTEXT =>$self->containment,
					  -PARENT =>$self);
	    }
			last SWITCH; };
	    
	    ## Two arguments: Message and note data.
	    /2/ && do{ $err = new Bio::Root::Err(-MSG     =>$param[0], 
						 -NOTE    =>$param[1], 
						 -STACK   =>scalar($self->stack_trace($stackNum)),
						 -CONTEXT =>$self->containment,
						 -PARENT =>$self );
		       last SWITCH; };
	    
	    ## Three arguments: Message, note, and tech data.
	    /3/ && do{ $err = new Bio::Root::Err(-MSG     =>$param[0], 
						 -NOTE    =>$param[1],  
						 -TECH    =>$param[2], 
						 -STACK   =>scalar($self->stack_trace($stackNum)),
						 -CONTEXT =>$self->containment,
						 -PARENT =>$self );
		       last SWITCH; };
	    
	    ## Default. Pass arguments to Err object for custom construction.
	    ## Note: Stack data is not added. Should be provided in @_.
	    $err = new Bio::Root::Err( @param, 
				       -STACK   =>scalar($self->stack_trace($stackNum)),
				       -PARENT =>$self );
	}
    };
    if($@) {
	printf STDERR "%s \"%s\": Failed to create Err object: \n $@", ref($self),$self->name;<STDIN>;
	print STDERR "\nReturning $self->{'_err'}:";<STDIN>;
	return $self->{'_err'}; }  ## Err construction will fail if the err is a duplicate.
                                  ## In any event, the Err object creation error is
                                  ## simply reported to STDERR and 
                                  ## the current 'Err' member is returned since 
                                  ## there is no call to _set_err_state().

    ## ONLY ATTACH THE Err OBJECT IF DIRECTED TO.
    if($self->{'_record_err'} or $RECORD_ERR) {
	if( ref($self->{'_err'})) {
	    ## Expand the linked list of Err objects if necessary.
#	    print "Expanding linked list of errors for ${\$self->name}.\nError = ${\$err->msg}\n";
	    $self->{'_err'}->last->add($err);
	} else {
#	    print "Adding new error for ${\$self->name}.\nError = ${\$err->msg}\n";
	    $self->{'_err'} = $err;
	}

	## Propagate the error up the containment hierarchy to ensure it gets noticed.
	## No longer needed since exception is fatal.
#	$self->_propagate_err($err);
    }
#    else { print "NOT SETTING ERR OBJECT: \n${\$err->string}\n\n"; }

    $self->_set_err_state($err);
}



=head2 _set_err_state

 Usage     : n/a; called automatically by _set_err()
 Purpose   : Sets the {'_errState'} data member to one of @Bio::Root::Err::ERR_TYPES.
           : This method is called after setting a new error with _set_err().
 Returns   : An Err.pm object (the current {'_err'} data member)
 Argument  : An Err.pm object (the one jsut created by _set_err()).
 Comments  : Modifications to state are permitted only if the object:
           :   1. has only one error, OR
           :   2. has multiple errors and none of those errors are fatal.
           : This prevents an object from setting its state to warning
           : if it already has a fatal error.
           :
           : The unfatal() method circumvents this method since the conditions
           : under which unfatal() is called are different. _set_err_state() is
           : only called when setting new errors.

See also   : L<_set_err>(), L<_set_warning>(), L<unfatal>()

=cut

#--------------------
sub _set_err_state {  
#--------------------
    my( $self, $err ) = @_;
    my @state = ();
    
    require Bio::Root::Err; import Bio::Root::Err qw(:data);
    
    my $data = $err->type || 'EXCEPTION';

    if($self->{'_errState'} and $self->{'_errState'} !~ /EXCEPTION|FATAL/) {
	
	if( @state = grep /$data/i, @Bio::Root::Err::ERR_TYPES ) {
	    $self->{'_errState'} = $state[0];
	} else {
	    $self->{'_errState'} = 'UNKNOWN STATE';
	}
    }
    $DEBUG and do{ print STDERR "$ID: Setting state to $self->{'_errState'} (arg=$data)\n"; <STDIN>; };

#    $self->{'_err'}->last;
    return $err;
}



=head2 err_state

 Usage     : $object->err_state();
 Purpose   : To assess the status of the object's Err object (if any).
 Returns   : A string: 'EXCEPTION' | 'WARNING' | 'FATAL' | 'OKAY'
           : (OKAY is returned if there are no Errors)
 Status    : Experimental

See also   : L<_set_err_state>()

=cut

#-------------'
sub err_state { 
#-------------
    my $self = shift; 
    return 'OKAY' if not defined $self->{'_err'};
    $self->{'_errState'} || 'OKAY'; 
}



=head2 _set_warning

 Purpose   : To record data regarding recoverable error conditions.
 Usage     : n/a; called automatically by Bio::Root::Object::warn() 
 Arguments : Arguments passed as-is to _set_err().
 Comments  : An object with a warning should be considered 
           : completely operational, so use this type of error sparingly. 
           : These errors are intended for problem conditions which:
           :  1. Don't destroy the basic functionality of the object.
           :  2. Might be of incidental interest to the user.
           :  3. Are of interest to the programmer but not the end user.

See also   : L<warn>(), L<_set_err>()

=cut

#-----------------'
sub _set_warning {  
#-----------------
    my( $self, @data ) = @_;  
    $DEBUG and print STDERR "\n$ID: setting warning.\n";
    
    my $err = $self->_set_err(@data, -STACK_NUM=>4);
    $err->last->set('type','WARNING');
    $self->_set_err_state($err);
}



=head2 stack_trace

 Usage     : $stack_aref = $myObj->stack_trace([start_index, [end_index]]);
           : @stack_list = $myObj->stack_trace([start_index, [end_index]]);
           : @stack_list = stack_trace($object);  # As an exported method.
 Purpose   : Returns the contents of the current call stack
           : in a slightly modified, more intuitive form. 
           : Permits extraction of a portion of the stack. 
           : Call stack is obtained from the perl caller() function.
           : MODIFIED FORMAT: Line numbers are shifted down one
           : level in the stack entries so that they correspond 
           : to the location of the indicated method.
 Example   : @stackData = $self->stack_trace(2); 
 Argument  : start_index : number of the beginning entry in the
           :               desired stack trace. The call to stack_trace()
           :               is at index 0.
           : end_index   : number of the last entry in the
           :               desired stack trace.
 Returns   : A list or list reference (depending on wantarray)
           : consisting of the desired portion of the call stack.

See also   : B<Bio::Root::Err::format_stack_entry()>

=cut

#-----------------
sub stack_trace {
#-----------------
    my($self,$beg,$end) = @_;
    my(@call,@data);

    ## Set the complete stack trace.
    my $i = 0;
    while( @call = caller($i++)) { 
	my @callData = @call; 
#	print "CALL DATA $i: \n";
#	my $j = 0; for($j=0; $j<@callData; $j++) {print "$j: $callData[$j]\n"; }; <STDIN>;
	next if $callData[3] eq '(eval)';  ## Screening out the (eval) calls.
	push @data, \@callData;
    }

    ## Shift the line numbers down so that they correspond to 
    ## the location of the shown method. This is more intuitive.
    ## Processing stack in reverse.
    my( @base_call, $temp);
    for($i=$#data; $i > 0; $i--) {
	$temp = $data[$i]->[2];
	$data[$i]->[2] = $data[$i-1]->[2];
	if($i == $#data) { @base_call = @{$data[$i]}; 
			   $base_call[2] = $temp;
			   $base_call[3] = "$data[$i]->[0]::$data[$i]->[1]";
		       }
    }
    @data = (@data, \@base_call);
#    print "FULL STACK:\n";foreach(@data){print "@$_\n";};<STDIN>;

    ## Get everything but the call to stack_trace
    $beg ||= 1;
    $end ||= $#data;
    @data = @data[$beg..$end];

    wantarray ? @data : \@data;
}



=head2 containment

 Usage     : $aref = $object->containment();
           : $aref = containment($object);   # As an exported method
 Purpose   : To determine the containment hierarchy of a object.
 Returns   : An array reference in which each element is a string
           : containing the class and name of
           : the object in which this object is contained.
           : Indentation increases progressively as the
           : hierarchy is traversed.
           : E.g.,  Object MyClass "Foo"
           :         Contained in object YourClass "Bar"
           :          Contained in object HisClass "Moo"
 Comments  : This method will report only one object at each level
           : since an object can currently have only one source object.
 Status    : Exported

See also   : L<_set_err>()

=cut

#------------------
sub containment {
#------------------
    my( $self) = @_;
    my(@hierarchy);

#    print "$ID: getting err hierarchy.\n";
    push @hierarchy, sprintf "Object %s \"%s\"", ref($self), $self->name;
    my $obj = $self;
    my $count = 0;

    while( ref $obj->parent) {
	$obj = $obj->parent;
	push @hierarchy, sprintf "%sContained in object %s \"%s\"", ' ' x ++$count, ref($obj), $obj->name;
    }
    return \@hierarchy;
}


=head2 set_stats

 Usage     : $object->set_stats(KEY => DATA [,KEY2 => DATA2])
 Purpose   : To declare and initialize a set of statistics germain
           : to an object. Each statistic name becomes a data member
           : prefixed with an underscore (if not already) and first
           : character after the underscore is lowercased.
 Example   : $object->set_stats('num_A' =>1,
           :                    'Num_B' =>10 ):
           : This sets :
           :     $object->{'_num_A'} = 1
           :     $object->{'_num_B'} = 10;
 Returns   : n/a
 Comments  : This method implements a convention for naming Perl
           : object data members with a leading underscore,
           : consistent with the naming convention of private methods.
           : Data members should not be part of an object's public
           : interface. The leading underscore helps flag the members
           : as private and also prevents inadvertant clobbering.

=cut    

#--------------'
sub set_stats {  
#--------------
    my( $self, %param ) = @_;
    
    my ($val);
    foreach (keys %param) { 
	$val = $param{$_};;
	s/^(\w)/_\l$1/; 
	$self->{$_} = $val;  
    }
}



=head2 _display_stats

 Usage     : n/a; called automatically by Bio::Root::Object::display(-SHOW=>'stats');
 Purpose   : Display stereotypical data for an object.
           : Automatically called via display().
 Argument  : Filehandle reference or string 'STDOUT' 'STDIN' 'STDERR'
 Status    : Experimental

See also   : L<display>()

=cut

#-------------------
sub _display_stats {
#-------------------
    my($self, $OUT) = @_;
    

    printf ( $OUT "%-12s: %s\n","NAME", $self->name());
    printf ( $OUT "%-12s: %s\n","MAKE", $self->make());
    if($self->parent) {
	printf ( $OUT "%-12s %s: %s\n","SRC OBJ", ref $self->parent, $self->parent->name);
    }
    printf ( $OUT "%-12s: %d\n",'ERRORS',    $self->err('count'));
    printf ( $OUT "%-12s: %s\n","ERR STATE", $self->err_state());
    if($self->err()) {
	print $OUT "ERROR:\n";
	$self->print_err();
    }
}



=head2 strict

 Usage     : $object->strict( [-1|0|1|2] );
           : $object->strict ? die "message" : warn "message";
 Purpose   : To make the object hyper- or hyposensitive to errors/exceptions.
           : Strict = 2  : extremely hyper-sensitive, all calls to warn() are fatal.
           : Strict = 1  : hyper-sensitive, calls to warn are not fatal.
           : Strict = -1 : hypo-sensitive.
           :               converts fatal errors into warnings 
           :               (i.e., throw()  --> warn()).
           : Strict = 0  : no change (throw() = fatal, warn() = non-fatal).
           :
           : Two positive values for strict permit the following functionality:
           :   1. By setting strict to 2, all calls to warn() become calls
           :      to throw() and are therefore fatal.
           :
           :   2. By setting strict to 1, calls to warn() generate non-fatal
           :      warnings as usual. This allows an object to produce
           :      additional warnings without producing fatal errors
           :      This is kind of like an object-specific -w flag.
           : An experimental use of strict() mode is described in handle_init_err().
           :
 Returns   : Integer between -1 to 2.
 Comments  : This method lets the object(s) to decide how strict to be
           : regarding warnings and exceptions. An alternative approach  
           : is the use of globals ($WARN_ON_FATAL, $FATAL_ON_WARN, and
           : $DONT_WARN).
 Status    : Experimental

See also   : L<throw_strict>(), L<handle_init_err>(), L<warn>(), L<throw>()

=cut

#------------
sub strict {
#------------
    my $self = shift;
    if (@_) { $self->{'_strict'} = shift; }
    $self->{'_strict'} || 0; 
}




=head2 clone

 Purpose   : To deeply copy an object.
           : Creates a new object reference containing an exact 
           : copy of an existing object and all its data members.
 Usage     : $myClone = $myObj->clone;
 Comments  : This method only clones the Bio::Root::Object data members. 
           : To fully clone an object that has data members beyond 
           : those inherited from Bio::Root::Object, you must provide a 
           : constructor in your class to copy all data of an object
           : data into the clone. For an example, see how _set_clone()
           : is called by _initialize() in this class.
           :
           : clone() will pass the named parameters {-MAKE=>'clone'} 
           : and {-OBJ=>$self} to the object's constructor. The
           : constructor should then either check the -MAKE parameter
           : directly or should check the return value from
           : a call to the superclass constructor (see _initialize()
           : for an example) and then copy the required data members from OBJ
           : into the new object, bypassing the normal construction process.
           : Cloning of objects has not been extensively tested.
           : USE WITH CAUTION.
 Status    : Experimental

See Also   : L<_set_clone>(), L<_initialize>()

=cut

#-------------'
sub clone {
#-------------
    my($self) = shift; 

#    warn sprintf "\nCloning %s \"%s\"\n\n", ref($self),$self->name;

    my $clone = $self->new(-MAKE    =>'clone', 
			   -OBJ     =>$self);  
    if($self->err()) { $clone->_set_err($self->err); } 
    $clone; 
} 



=head2 _set_clone

 Usage     : n/a; internal method used by _initialize()
           : $self->_set_clone($object_to_be_cloned)
 Purpose   : Deep copy all Bio::Root::Object.pm data members
           : into a new object reference. 
           : (This is basically a copy constructor).
 Argument  : object ref for object to be cloned.
 Throws    : Exception if argument is not an object reference. 
 Comments  : Data members which are objects are cloned (parent, io, err).
           : Cloning of objects has not been extensively tested.
           : USE WITH CAUTION.

See Also   : L<_initialize>()

=cut

#----------------
sub _set_clone {
#----------------
    my($self, $obj) = @_;

    ref($obj) || throw($self, "Can't clone $ID object: Not an object ref ($obj)");

    local($^W) = 0;  # suppress 'uninitialized' warnings.

    $self->{'_name'}     = $obj->{'_name'};
    $self->{'_strict'}   = $obj->{'_strict'};
    $self->{'_make'}     = $obj->{'_make'};
    $self->{'_terse'}    = $obj->{'_terse'};
    $self->{'_verbose'}  = $obj->{'_verbose'};
    $self->{'_errState'} = $obj->{'_errState'};
    ## Better to use can() with Perl 5.004.
    $self->{'_parent'}   = ref($obj->{'_parent'}) and $obj->{'_parent'}->clone;
    $self->{'_io'}       = ref($obj->{'_io'}) and $obj->{'_io'}->clone;
    $self->{'_err'}      = ref($obj->{'_err'}) and $obj->{'_err'}->clone;
}


=head2 terse

 Usage     : $object->terse([0|1]);
 Purpose   : Set/Get an indicator to report less than the normal amount.
 Argument  : Boolean (0|1)
 Returns   : Boolean (0|1)
 Comments  : This method is for reducing the amount of reporting
           : an object will do. 
           : terse can be set during object construction with the
           : -TERSE => 1 flag.
           : Not putting this method in IOManager.pm since that class
           : is concerned with "where" to report, not "what" or "how much".
 Status    : Experimental

See also   : L<verbose>()

=cut

#----------
sub terse { 
#----------
    my $self = shift; 
    if(@_) { $self->{'_terse'} = shift; }
    $self->{'_terse'} || 0;
}



=head2 verbose

 Usage     : $object->verbose([0|1]);
 Purpose   : Set/Get an indicator to report more than the normal amount.
 Argument  : Boolean (0|1)
 Returns   : Boolean (0|1)
 Comments  : This method is for increasing the amount of reporting
           : an object will do. 
           : verbose can be set during object construction with the
           : -VERBOSE => 1 flag.
           : Not putting this method in IOManager.pm since that class
           : is concerned with "where" to report, not "what" or "how much".
 Status    : Experimental

See also   : L<terse>()

=cut

#------------
sub verbose { 
#------------
    my $self = shift; 
    if(@_) { $self->{'_verbose'} = shift; }
    $self->{'_verbose'} || 0;
}



=head1 I/O-RELATED METHODS (Delegated to B<Bio::Root::IOManager.pm>)

These methods should probably be AUTOLOADed as well.

=head2 _io

 Usage     : $object->_io()
 Purpose   : Get the Bio::Root::IOManager.pm object for the current object.

See also   : L<display>(), L<read>(), L<file>()

=cut

#-------
sub _io  {  my $self = shift;   return $self->{'_io'}; }
#-------



=head2 _set_io

 Usage     : n/a; internal use only.
 Purpose   : Sets a new Bio::Root::IOManager.pm object for the current object.

See also   : L<display>(), L<read>(), L<file>()

=cut

#------------
sub _set_io {
#------------
    my $self = shift;
    
    require Bio::Root::IOManager;

    $self->{'_io'} = new Bio::Root::IOManager(-PARENT=>$self, @_);
}



=head2 set_display

 Usage     : $object->set_display( %named_parameters).
           : See Bio::Root::IOManager::set_display() for a description of parameters.
 Purpose   : Sets the output stream for displaying data associated with an object.
           : Delegates to Bio::Root::IOManager::set_display().
 Argument  : Named parameters (optional).
           : See Bio::Root::IOManager::set_display() for a
           : description of arguments.
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.
           : I'm not satisfied with the current display()/set_display() strategy.

See also   : B<Bio::Root::IOManager.pm>::set_display

=cut

#----------------'
sub set_display {
#----------------
    my($self, @param) = @_;
    
    $self->_set_io(@param) if !ref($self->{'_io'});

    eval { $self->{'_io'}->set_display(@param);  };

    if($@) {
	my $er = $@;
	$self->throw(-MSG=>$er, -NOTE=>"Can't set_display for ${\$self->name}");
    }

   return $self->{'_io'}->fh;
}


=head2 display

 Usage     : $object->display( named parameters)
           : See Bio::Root::IOManager::display() for a description of parameters.
 Purpose   : Output information about an object's data.
           : Delegates this task to Bio::Root::IOManager::display()
 Argument  : Named parameters for IOManager::set_display()
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.
           : IOManager::set_display()is then called on the new IOManager object.
           :
           : The motivation behind the display() method and IOManager.pm
           : is to allow for flexible control over output of an
           : object's data to/from filehandles, pipes, or STDIN/STDOUT,
           : and for passing file handles between objects. Currently, 
           : it is used mainly for output to STDOUT.
           :  
           : There is some concern whether this much functionality is
           : actually necessary, hence the "Experimental" status of this
           : method.
           : 
           : -------
           : It might be worthwhile to also have a string() method
           : that will put an object's data into a string that can be
           : further processed as desired. Stringification for persistence 
           : issues might be best handled by Data::Dumper.pm.
           :
           : When overriding this method, use the following syntax:
           :
           : sub display {
           :    my ($self, %param) = @_;
           :    $self->SUPER::display(%param);
           :    my $OUT = $self->fh();
           :    print $OUT "\nSome data...\n";
           :  ...
           : }
           : Now $OUT holds a filhandle reference (or the string 'STDOUT')
           : which can be passed to other methods to display different
           : data for the object. 
           : _set_display() is automatically called with $OUT as the sole
           : argument (after $self) by IOManager.pm::display()
           : if the -SHOW parameter is set to 'stats' or 'default'.
           :
 Bugs      : Because the $OUT variable can be a FileHandle or a string,
           : it is necessary to include the line before using $OUT in 
           : print statements:
           : I am considering a cleaner way of dealing with this.
           : Setting $OUT to a glob (*main::STDOUT) was unsuccessful.
           :
           : I'm not satisfied with the current display()/set_display() strategy.

See also   : B<Bio::Root::IOManager::display()>

=cut

#-------------
sub display { 
#-------------
    my( $self, @param ) = @_; 
    $self->{'_io'} || $self->set_display(@param);
    $self->{'_io'}->display(@param); 
}



=head2 read

 Usage     : $object->read( named parameters)
           : See Bio::Root::IOManager::read() for a description of parameters.
 Purpose   : Inputs data from an arbitrary source (file or STDIN).
           : Delegates this task to Bio::Root::IOManager::read().
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.
           : See the comments for the display() method for some comments
           : about IO issues for objects.
           : Note that the read() method uses a different strategy than
           : the display() method. 
           : IO issues are considered experimental.
 
See also   : display(), B<Bio::Root::IOManager::read()>

=cut

#--------
sub read { 
#--------
    my $self = shift; 

    $self->_set_io(@_) if not defined $self->{'_io'};
	
    $self->{'_io'}->read(@_); 
}



=head2 fh

 Usage     : $object->fh(['name'])
           : See Bio::Root::IOManager::fh() for a complete usage description.
 Purpose   : Get an object's current FileHandle object or IO stream indicator.
           : Delegates to Bio::Root::IOManager.pm.
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.

See also   : B<Bio::Root::IOManager.pm>::fh

=cut

#--------'
sub fh      { 
#--------
    my $self = shift; 
    $self->_set_io(@_) if !defined $self->{'_io'};
    $self->{'_io'}->fh(@_); 
}


=head2 show

 Usage     : $object->show()
           : See Bio::Root::IOManager::show() for details.
 Purpose   : Get the string used to specify what to display
           : using the display() method.
           : Delegates to Bio::Root::IOManager.pm.
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.

See also   : B<Bio::Root::IOManager.pm>::show, B<Bio::Root::IOManager.pm>::set_display

=cut

#-----------
sub show    { 
#-----------
    my $self = shift; 
    $self->_set_io(@_) if !defined $self->{'_io'};
    $self->{'_io'}->show; 
}



=head2 file

 Usage     : $object->file()
           : See Bio::Root::IOManager::file() for details.
 Purpose   : Set/Get name of a file associated with an object.
           : Delegates to Bio::Root::IOManager.pm.
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.

See also   : B<Bio::Root::IOManager::file()>

=cut

#---------
sub file    {   
#---------
    my $self = shift;  
    $self->_set_io(@_) if !defined $self->{'_io'};
    $self->{'_io'}->file(@_); 
}


=head2 compress_file

 Usage     : $object->compress_file([filename])
           : See Bio::Root::IOManager::compress_file() for details.
 Purpose   : Compress a file associated with the current object.
           : Delegates to Bio::Root::IOManager.pm.
 Throws    : Propagates exceptions thrown by Bio::Root::IOManager.pm
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.

See also   : L<file>(), B<Bio::Root::IOManager.pm>::compress_file

=cut

#-------------------
sub compress_file { 
#-------------------
    my $self = shift;
    $self->_set_io(@_) if !defined $self->{'_io'};
    $self->{'_io'}->compress_file(@_); 
}



=head2 uncompress_file

 Usage     : $object->uncompress_file([filename])
           : Delegates to Bio::Root::IOManager.pm.
 Purpose   : Uncompress a file associated with the current object.
 Throws    : Propagates exceptions thrown by Bio::Root::IOManager.pm
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.

See also   : L<file>(), B<Bio::Root::IOManager.pm>::uncompress_file

=cut

#--------------------
sub uncompress_file { 
#--------------------
    my $self = shift;  
    $self->_set_io(@_) if !defined $self->{'_io'};
    $self->{'_io'}->uncompress_file(@_); 
}


=head2 delete_file

 Usage     : $object->delete_file([filename])
           : See Bio::Root::IOManager::delete_file() for details.
 Purpose   : Delete a file associated with the current object.
           : Delegates to Bio::Root::IOManager.pm.
 Throws    : Propagates exceptions thrown by Bio::Root::IOManager.pm
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.

See also   : L<file>(), B<Bio::Root::IOManager.pm>::delete_file

=cut

#-----------------
sub delete_file { 
#-----------------
    my $self = shift;
    $self->_set_io(@_) if !defined $self->{'_io'};
    $self->{'_io'}->delete_file(@_); 
}



=head2 file_date

 Usage     : $object->file_date( %named_parameters )
           : See Bio::Root::IOManager::file_date() for details.
 Purpose   : Obtain the last modified data of a file.
           : Delegates to Bio::Root::IOManager.pm.
 Example   : $object->file_date('/usr/home/me/data.txt');
 Throws    : Propagates exceptions thrown by Bio::Root::IOManager.pm
 Status    : Experimental
 Comments  : Sets the IOManager.pm object if it is not set.

See also   : L<file>(), B<Bio::Root::IOManager.pm>::file_date

=cut

#---------------
sub file_date { 
#---------------
    my $self = shift;  
    $self->_set_io(@_) if !defined $self->{'_io'};
    $self->{'_io'}->file_date(@_); 
}




# Stubs for AUTOLOADED methods
# These are more experimental or deprecated methods.

sub xref;
sub index;
sub find_object;
sub has_warning; 
sub print_err;
sub err_string;
sub clear_err;
sub throw_strict; 
sub valid;
sub invalidate;
sub _propagate_err;
sub fatal;
sub unfatal;
sub handle_init_err;
sub set_err_data;
sub set_read; 
sub set_log_err;


1;
#__END__



#######################################################################
#                        AUTOLOADED METHODS  
#######################################################################

=head1 AUTOLOADED METHODS (EXPERIMENTAL OR SELDOM-USED)


=head2 xref

 Usage     : $object->xref([object | 'null']);
 Purpose   : Sets/Gets an object(s) cross-referenced 
           : to the current object. 
 Example   : $myObj->xref('null');       #remove all xrefs
           : $myObj->xref($otherObject); #add a cross referenced object
 Argument  : Object reference or 'null' ('undef' also accepted).
 Returns   : Object reference or undef if the object has no xref set.
 Throws    : fatal error if argument is not an object reference or 'null'. 
 Comments  : An Xref.pm object is a vectorized wrapper for an object.
           : Thus, the number of objects cross-referenced can grow
           : and shrink at will.
 Status    : Experimental
 WARNING   : NOT FULLY TESTED.

See Also   : B<Bio::Root::Xref.pm>

=cut

#---------
sub xref  { 
#---------
    my $self = shift; 
    if(@_) { 
	my $arg = shift;
	if(ref $arg) {
	    require Bio::Root::Xref;
	    
	    if( !defined $self->{'_xref'}) {
		$self->{'_xref'} = new Bio::Root::Xref(-PARENT =>$self,
						       -OBJ     =>$arg);
	    } else {
		$self->{'_xref'}->add($arg);
	    }
	} elsif($arg =~ /null|undef/i) {
	    $self->{'_xref'} = undef;
	} else {
	    $self->throw("Can't set Xref using $arg: Not an object");
	}
    }
    
    $self->{'_xref'}; 
}



=head2 index

 Purpose   : To add an object to a package global hash of objects
           : for tracking or rapid retrieval.
 Usage     : $self->index();
 Status    : Experimental
 Comments  : The object's name is used to index it into a hash. Objects in
           : different classes (packages) will be indexed in different hashes.
           : An object's name should thus be unique within its class.
           : To find an object, use find_object().
           : Uses the package global %Objects_created.

See also   : L<find_object>()

=cut

#----------
sub index {
#----------
    my $self    = shift;
    my $class   = ref $self;
    my $objName = $self->{'_name'};
    
    if( not defined $objName ) {
	$self->throw("Can't index $class object \"$objName\".");
    }
    
    $DEBUG and do{ print STDERR "$ID: Indexing $class object \"$objName\"."; <STDIN>; };
    
    $Objects_created{ $class }->{ $objName } = $self;
}



=head2 find_object

 Purpose   : To obtain any object reference based on its unique name
           : within its class.
 Usage     : $myObj = &find_object('fred');
           : No need to specify the class (package) name of the object.
 Comments  : To use this method, the object must be previously 
           : indexed by Bio::Root::Object.pm. This can be accomplished
           : by including 'index' in the -MAKE parameter during object
           : construction OR by calling the index() method on the
           : the object at any point after construction.
           : This is not an instance method.
 Status    : Experimental

See  also  : L<index>()

=cut

#---------------
sub find_object {
#---------------
    my $name   = shift;   # Assumes name has been validated.
    my $class  = undef;
    my $object = undef;
    
    foreach $class ( keys %Objects_created ) {
	if( exists $Objects_created{ $class }->{ $name } ) {
	    $object = $Objects_created{ $class }->{ $name };
	    last;
	}
    }
    $object;
}



=head2 has_warning

 Purpose   : Test whether or not an object has a non-fatal error (warning).
 Usage     : $self->has_warning;
 Comments  : Equivalent to ($self->err() and !$self->valid()).
           : This method is not usually needed. Checking err() is 
           : sufficient since throw()ing an exception is a fatal event 
           : and must be handled.
 Status    : Experimental

See also   : L<err>(), L<valid>()

=cut

#----------------
sub has_warning { 
#----------------
    my $self = shift; 
    my $errData = $self->err('type');
    return 1 if $errData =~ /WARNING/;
    0;
}



=head2 print_err

 Usage     : print_err([-WHERE=>FileHandle_object [,-SHOW=>msg|note|tech|stack] or any combo])
 Purpose   : Reports error data for any errors an object may have
           : as a string.
 Example   : $myObj->print_err;
           : $myObj->print_err(-WHERE=>$myObj->fh('err'), -SHOW=>'msgtechstack');
 Argument  : SHOW parameter  : specify a sub-set of the err data.
           : WHERE parameter : specify a filehandle for printing.
 Returns   : n/a

See also   : L<err_string>()

=cut

#-------------
sub print_err {
#-------------
    my( $self, %param ) = @_;
    
#    print "$ID: print_err()\n";

    my $OUT = $self->set_display(%param);

#    print "$ID: OUT = $OUT\n";

    print $OUT $self->err_string( %param );
    
#    print "$ID: done print_err()\n";
}



=head2 err_string

 Usage     : err_string([-SHOW =>msg|note|tech|stack])
           : err_string([-SHOW =>'msgnote'] or other combos)
 Purpose   : Reports all warnings generated by the object as a string.
 Example   : $errData = $myObj->err_string;
           : print MYHANDLE $myObj->err_string();
 Argument  : SHOW parameter : return a specific sub-set of the err data.
 Returns   : A string containing the error data of the object.
 Comments  : This method is provided as a safer and slightly easier to type
           : alternative to $self->err->string.

See also   : L<print_err>(), B<Bio::Root::Err.pm>::string

=cut

#----------------
sub err_string {
#----------------
    my( $self, %param ) = @_;
    my($out);
    my $errCount = $self->err('count');

#    print "$ID: err_string(): count = $errCount\n";

    if( $errCount) { 
	$out = sprintf("\n%d error%s in %s \"%s\"\n",
		       $errCount, $errCount>1?'s':'', ref($self), $self->name);
	$out .= $self->err->string( %param );
    } else {
	$out = sprintf("\nNo errors in %s \"%s\"\n", ref($self), $self->name);
    }
    $out;
}


=head2 clear_err

 Purpose   : To remove any error associated with the given object.
 Usage     : $myObj->clear_err;

See  also  : L<_set_err>()

=cut

#-------------
sub clear_err {
#-------------
    my $self = shift;
    $self->_set_err();
}




#################################################################
#            DEPRECATED or HIGHLY EXPERIMENTAL METHODS
#################################################################

=head1 HIGHLY EXPERIMENTAL/DEPRECATED METHODS

=head2 valid

 Usage     : $self->valid;
 Purpose   : Test whether or not an object has a fatal error.
 Comments  : This method is only useful with a polling type of error
           : Handling. As such, it is no longer nessary since fatal 
           : errors now result in a fatal exception being raised. 
           : Does NOT examine the Err objects themselves.
           : Only checks for a defined 'err' data member and
           : the value of err_state(). 
           : Thus, an object could conceivably have a fatal error
           : yet still be valid. See unfatal() for more info.
 Status    : Experimental

See also   : L<err>(), L<has_warning>(), L<unfatal>()

=cut

#-----------
sub valid {
#-----------
    my $self = shift;
    not(defined $self->{'_err'} and $self->err_state =~ /FATAL|EXCEPTION/i); 
}



=head2 invalidate

 Usage     : invalidate(["string"])
 Purpose   : To invalidate an object.
 Arguments : $_[0] = a string describing the reason for invalidation.
 Comments  : This method is only useful with a polling type of error
           : handling. It will be no effect unless valid() is checked.
           : Current error handling relies upon invalid objects to 
           : throw fatal exceptions.
 Status    : Experimental

See also   : L<valid>()

=cut

#--------------
sub invalidate {
#--------------
    my ($self, $reason) = @_;
    $self->_set_err(sprint("Invalidating object %s \"%s\".",ref($self),$self->name),
		  "Reason: $reason");
}


=head2 _propagate_err

 Usage     : _propagate_err(<Bio::Root::Err object>)
 Purpose   : To propagate an err up an object's containment hierarchy.
 Arguments : $_[0] = a reference for an Bio::Root::Err.pm object
 WARNING   : Calling this method will result in propagation
           : of the error up the containment hierarchy leading to invalidation 
           : of all containing objects. The containing objects are forced
           : to deal with the exception and call unfatal() or be invalidated.
	   :
           : This method was used for a polling type of exception handling, 
           : where having the error bubble up to the top of a containment
           : hierarchy increases its chance of getting noticed. However, 
           : this behavior is not usually appropriate since the contained
           : object is assuming it knows what is appropriate for the
           : objects in the containment hierarchy of its container. 
           : Because of this, this method is no longer used and is not
	   : recommended.
 Status    : Experimental, Deprecated

See also   : L<valid>(), L<fatal>()

=cut

#------------------'
sub _propagate_err {
#------------------
    my( $self,$err) = @_;

#    print "$ID: propagating err up containment hierarchy.\n";
    my $count = 0;

    my $obj = $self->parent;
    my ($cloned_err);
    while( ref $obj) {

	## Extract the error from other errors in its vector.
	## Each object must have its own copy. Wasteful, I know, but
	## prevents referential mayhem. Also, exceptions are exceptional.
	$cloned_err = $err->clone;

	if( defined $obj->{'_err'}) {
	    $obj->{'_err'}->last->add($cloned_err);
	} else {
	    $obj->{'_err'} = $cloned_err;
	}
	$obj->_set_err_state($cloned_err);
	$obj = $obj->parent;
    }
}



=head2 fatal

 Usage     : fatal([arguments for _set_err()])
 Purpose   : Generate, report, and set a fatal error on the object and
           : to invalidate the object. (fatal = un-recoverable)
 Returns   : Dies unless the global switch $WARN_ON_FATAL is true
           : If true, this method behaves like warn() but sets a fatal err.
 Comments  : Objects that call this method will be invalidated in the sense
           : that calling valid() on the object will return false. This was
           : used for a former polling type of error handling that is now
           : deprecated. Use throw() instead.
 Status    : Experimental, Deprecated

See also   : L<warn>(), L<throw>(), L<throw_strict>()

=cut

#-----------
sub fatal {
#-----------
    my $self = shift;

    $self->_set_err(@_)->last->set('type','FATAL');

    if($WARN_ON_FATAL) {
	#print STDERR "\n* * *   W A R N I N G   * * *\n";
	warn $self->err_string(-CURRENT =>1)
#	print "\nWarned on fatal (dangerous!).\n";<STDIN>;
    } else {
#	print "\nDying on fatal.\n";<STDIN>;
#	print STDERR "\n* * *   F A T A L   * * *\n";
	die $self->err_string(-CURRENT =>1);
    }
}



=head2 unfatal

 Purpose   : 1. Removes the fatal err_state from the object after a
           :    fatal error has occurred.
           : 2. Adds a technical note to the last error about this recovery event.
 Returns   : true (1) if successful, (0) otherwise.
 Comments  : This method is important in situations where an
           : object contains many other objects, one of which
           : has a fatal error. This error would invalidate the
           : container object unless it can clear its err_state.
           : By calling unfatal(), the container object maintains
           : its validity yet still retains the fatal error, which
           : can be useful for error reporting purposes.
           :
           : To recover from a fatal error, unfatal() should be called 
           : immediately after a fatal error occurrs. Under bizarre circumstances, 
           : it is possible that the technical note will be added to the wrong 
           : error in a object with multiple errors. A proper check for this 
           : situation will be added if it becomes a problem. 
 Status    : Experimental, Deprecated

See also   : L<fatal>(), L<valid>() 

=cut

#------------
sub unfatal {
#------------
    my $self = shift;

    return unless $self->err_state() eq 'FATAL';

    $self->{'_errState'} = 'WARNING';
    eval{ 
	my $tnote = "$self->{'_name'} (${\ref($self)}) recovered from fatal error.";
	$self->err->last->set('tech', $tnote );

	## ERRORS ARE NO LONGER PROPAGATED.
	## Notify objects within containment hierarchy about the recovery event.
#	 my $obj = $self->parent;
#	 while( ref $obj) {
#	     if($obj->err_state() =~ /FATAL|EXCEPTION/) {
#		 $obj->{'_errState'} = 'WARNING';
#		 $obj->err->last->set('tech', $tnote );
#	     }
#	     $obj = $obj->parent;
#	 }
    };
    if($@) { warn "\n*** $@\n\n"; return 0; }
    1;
}



=head2 throw_strict

 Usage     : throw_strict([arguments for _set_err()])
 Purpose   : To set a fatal error and activate 'strict' mode.
           : This increases the likelihood that exception will be noticed
           : by forcing the script to terminate.
 Returns   : n/a
 Comments  : Calling this method does not guarantee that the script 
           : will terminate. If the object is deep within a containment hierarchy, 
           : a containing object may trap an exception thrown by the contained object, 
           : ignore the strict mode, and instead attempt to address the condition 
           : causing the error. 
 Status    : Experimental

See also   : L<throw>(), L<fatal>(), L<strict>(), L<handle_init_err>()

=cut

#------------------
sub throw_strict { 
#------------------
    my $self = shift; 

    if(@_) { 
	$self->strict(2);
	$self->throw(@_, -STACK_NUM=>4); 
    }
}



=head2 handle_init_err

 Usage     : handle_init_err($@ [,1|0])
 Purpose   : To provide a consistent procedure for handling exceptions
           : that occur within constructors.
 Arguments : $_[0] = the $@ string describing the cause of the exception.
           : $_[1] = optional strict flag. Default = TRUE.
           :         If TRUE, die() is used.
           :         If FALSE (zero), warn() is used.
 Comments  : The default behavior for this method is to die() with the contents
           : of $@, which are passed in as the first argument. An optional second
           : argument can be used to either warn or die(). If defined, this
           : second argument overrides the objects strict() state. 
           :
           : Typical use: An object could eval {} its own construction process 
           : so that if an exception occurs, the constructor can still
           : return an object reference and let the client decide what to do about 
           : the problem. This would be accomplished by calling handle_init_err($@,0).
           : The danger of this approach is that the error might go unnoticed
           : if the client does not check if the object is valid().
           :
           : In most cases, exceptions within constructors should leave the
           : the object in such a state that it cannot be used at all. 
           : die()ing in a constructor is a sure way to make this so. 
           : Because of this, it is best to eval {} object contruction
           : outside of the constructor method:
           :
           :    my $obj = eval { new Foo(@data) };
           :    if( !ref($obj)) { warn "Can't use object: $@\n"; }    
           :
           : Calling handle_init_err($@,0) might be of use in development situations.
 Status    : Experimental, Deprecated

See also   : L<strict>(), L<throw_strict>()

=cut

#-------------------
sub handle_init_err {
#-------------------
    my ($self, $report, $strict) = @_;
    
#    printf "handle_init_err(): object = %s \"%s\".",ref($self),$self->name;<STDIN>;

    my $trouble_html = qq|<p><b>This is our fault!</b> There is apparently a problem
	with our software. If this problem persists, 
	<A HREF="mailto:$AUTHORITY"><b>please notify us.</b></A> Thanks.<p>|;

    $report ||= sprintf( "<pre>Unknown exception within constructor for %s \"%s\".</pre>\n%s\n",
			 ref $self, $self->name, $trouble_html);

    $self->_set_err($report);

    if( $self->strict > 0) { $strict = 1; }

    if( !defined $strict ) { $strict = 1; }

    $self->set_err_data('tech', "Construction failed for ${\ref $self} \"${\$self->name}.");

#    print "handling init err.";<STDIN>;
    ($strict && !$WARN_ON_FATAL) ? die $report : (!$DONT_WARN && warn $report); 

}


#----------------------
=head2 set_err_data()
#----------------------

 Usage     : $object->set_err_data( field, data);
 Purpose   : Alters data within the last error set by the object.
           : Interface to Bio::Root::Err::set().
 Returns   : Calls Bio::Root::Err::set()
 Argument  : field = string, name of Bio::Root::Err.pm data field to set.
           : data  = string, data to set it to.
 Throws    : Exception if object has no errors.
 Status    : Deprecated

See Also   : B<Bio::Root::Err.pm>set

=cut

#-----------------
sub set_err_data { 
#-----------------
    my( $self, $field, $data) = @_;
    
    $self->throw("Object has no errors.") if !$self->{'_err'};

#    print "$ID: set_err_data($field)  with data = $data\n  in object ${\$self->name}:\n", $self->err->last->string(-CURRENT=>1); <STDIN>;

    $self->{'_err'}->last->set( $field, $data );
}

=head2 set_read

 Usage     : see Bio::Root::IOManager::set_read()
 Purpose   : Sets an input stream for importing data associated with an object.
           : Delegates to Bio::Root::IOManager::set_read().
 Status    : Experimental
 WARNING!  : This method has not been tested.

See also   : B<Bio::Root::IOManager.pm>::set_read

=cut

#--------------
sub set_read { 
#--------------
    my($self,%param) = @_;
    
    $self->_set_io(%param) if !defined $self->{'_io'};

    $self->{'_io'}->set_read(%param);
}



=head2 set_log_err

 Usage     : see Bio::Root::IOManager::set_log_err()
 Purpose   : Sets the output stream for logging information about
           : an object's errors.
           : Delegates to Bio::Root::IOManager::set_log_err().
 Status    : Experimental
 WARNING!  : This method has not been tested.

See also   : B<Bio::Root::IOManager.pm>::set_log_err

=cut

#---------------'
sub set_log_err {
#---------------
    my($self,%param) = @_;
    
    $self->_set_io(%param) if !defined $self->{'_io'};

    $self->{'_io'}->set_log_err(%param);
}


#####################################################################################
#                                  END OF CLASS                                     #
#####################################################################################

=head1 FOR DEVELOPERS ONLY

=head2 Data Members

Information about the various data members of this module is provided for those 
wishing to modify or understand the code. Two things to bear in mind: 

=over 4

=item 1 Do NOT rely on these in any code outside of this module. 

All data members are prefixed with an underscore to signify that they are private.
Always use accessor methods. If the accessor doesn't exist or is inadequate, 
create or modify an accessor (and let me know, too!). 

=item 2 This documentation may be incomplete and out of date.

It is easy for this documentation to become obsolete as this module is still evolving. 
Always double check this info and search for members not described here.

=back

An instance of Bio::Root::Object.pm is a blessed reference to a hash containing
all or some of the following fields:

 FIELD          VALUE
 ------------------------------------------------------------------------
  _name         Common name for an object useful for indexing. 
 	        Should be unique within its class.

  _parent       The object which created and/or is responsible for this object,
 	        also known as the 'parent' object to the current 'child' object.

  _err          Bio::Root::Err.pm object reference. Undefined if the object has no error
                or if the _record_err member is false (which is the default).
 	        If object has multiple errors, err becomes a linked
 	        list of Err objects and the err member always points to latest err.
 	        In theory, an object should care only about whether or not it HAS
 	        an Err not how many it has. I've tried to make the management of 
 	        multiple errors as opaque as possible to Bio::Root::Object.

 _errState      One of @Bio::Root::Err::ERR_TYPES. Allows an object to quickly determine the
 	        the type of error it has (if any) without having to examine 
 	        potentially multiple Err object(s). 

  _xref         Bio::Root::Xref object (Vector) for tracking other object(s) related to the
 	        present object not by inheritance or composition but by some arbitrary
 	        criteria. This is a new, experimental feature and is not fully implemented.

  _make         Used as a switch for custom object initialization. Provides a 
 	        mechanism for alternate constructors. This is somewhat experimental. 
 	        It may be useful for contruction of complex objects and may be of
 	        use for determining how an object was constructed post facto.

  _io           Bio::Root::IOManager.pm object reference. Used primarily for handling the 
	        display of an object's data. 

  _strict       Integer flag to set the sensitivity to exceptions/warnings
 	        for a given object.

  _terse        Boolean indicator for reporting less than the normal amount.

  _verbose      Boolean indicator for reporting more than the normal amount.

  _record_err   Boolean indicator for attaching all thrown exception objects
                to the current object. Default = false (don't attach exceptions).

=cut

1;

