#-----------------------------------------------------------------------------
# PACKAGE : Bio::Root::Utilities.pm
# PURPOSE : Provides general-purpose utilities of potential interest to any Perl script.
# AUTHOR  : Steve A. Chervitz
# CREATED : Feb 1996
# COMMENTS: Most of the methods are AUTOLOADED since a typical script only
#           needs one or a few functions.
# STATUS  : Alpha
#
# TODO    : Configure $GNU_PATH during installation.
#           Improve documentation (POD).
#           Make use of Date::Manip and/or Date::DateCalc as appropriate.
#
# MODIFIED:
#   23 Jun 1998, sac: 
#        * Improved file_date() to be less reliant on the output of ls.
#          (Note the word 'less'; it still relies on ls).
#   15 Jul 1998, sac:
#        * compress() & uncompress() will write files to a temporary location
#          if the first attempt to compress/uncompress fails.
#          This allows users to access compressed files in directories in which they
#          lack write permission.
#
# Copyright (c) 1996-8 Steve A. Chervitz. All Rights Reserved.
#          This module is free software; you can redistribute it and/or 
#          modify it under the same terms as Perl itself.
#
#####################################################################################

package	Bio::Root::Utilities;

use Bio::Root::Global  qw(:data :std);
#use Bio::Root::Err     qw(:std);
use Bio::Root::Object  ();
use Exporter           ();
use POSIX;

*AUTOLOAD = \&AutoLoader::AUTOLOAD;

@ISA         = qw( Bio::Root::Object Exporter);
@EXPORT_OK   = qw($Util);
%EXPORT_TAGS = ( obj => [qw($Util)],
		 std => [qw($Util)],);

use strict;  
use vars qw($ID $VERSION $Util $GNU_PATH);

$ID        = 'Bio::Root::Utilities';
$VERSION   = 0.022;

# $GNU_PATH may be required for executing gzip/gunzip 
# in some situations. Customize for your site or leave
# it as a null string.
$GNU_PATH  = ''; 
#$GNU_PATH  = '/tools/gnu/bin/'; 

## Static UTIL object.
$Util = {};
bless $Util, $ID;
$Util->{'_name'} = 'Static Utilities object';


##########################################################################################
##                               INSTANCE METHODS                                       ##
##########################################################################################


#-----------------------------------------------------------------------
=head1 date_format()
 Title     : date_format
           : 
 Usage     : $Util->date_format( [FMT], [DATE]) 
           :
 Purpose   : -- Get a string containing the formated date or time
           :    taken when this routine is invoked.
           : -- Provides a way to avoid using `date`.
	   : -- Provides an interface to localtime().
           : -- Interconverts some date formats.
           :
           : (For additional functionality, use Date::Manip or
           :  Date::DateCalc available from CPAN).
           :
 Example   : $Util->date_format();
           : $date = $Util->date_format('yyyy-mmm-dd', '11/22/92');
           :
 Returns   : String (unless 'list' is provided as argument, see below)
           :
           :   'yyyy-mm-dd'  = 1996-05-03    # default format. 
           :   'yyyy-dd-mm'  = 1996-03-05   
           :   'yyyy-mmm-dd' = 1996-May-03
           :   'd-m-y'       = 3-May-1996
           :   'd m y'       = 3 May 1996
           :   'dmy'         = 3may96
           :   'mdy'         = May 3, 1996
           :   'ymd'         = 96may3
           :   'md'          = may3
           :   'year'        = 1996
           :   'hms'         = 23:01:59  # 'hms' can be tacked on to any of the above options
           :                             # to add the time stamp: eg 'dmyhms'
           :   'full' | 'unix' = UNIX-style date: Tue May  5 22:00:00 1998
           :   'list'          = the contents of localtime(time) in an array.
           : 
           :
           :
 Argument  : (all are optional)
           : FMT  = yyyy-mm-dd | yyyy-dd-mm | yyyy-mmm-dd |
           :        mdy | ymd | md | d-m-y | hms | hm 
           :        ('hms' may be appended to any of these to 
	   :        add a time stamp)
           :
           : DATE = String containing date to be converted.
           :        Acceptable input formats:
           :           12/1/97 (for 1 December 1997)
           :           1997-12-01 
           :           1997-Dec-01
           :
 Throws    : Exception if the file appears to be empty or non-existent
           :
 Access    : Public 
           :
 Comments  : Relies on the $BASE_YEAR constant exported by Bio:Root::Global.pm.
           : If you don't care about formatting or using backticks, you can
           : always use: $date = `date`;
           :
           : For more features, use Date::Manip.pm, (which I should 
           : probably switch to...)

See Also   : file_date()
=cut
#-----------------------------------------------------------------------'
sub date_format {
    my $self   = shift;
    my $option = shift;
    my $date   = shift;  # optional date to be converted.

    my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);

    $option ||= 'yyyy-mm-dd';

    my ($month_txt, $day_txt, $month_num, $fullYear);
    my (@date);

    # Load a supplied date for conversion:
    if(defined($date)) {
	if( $date =~ /\//) {
	    ($mon,$mday,$year) = split(/\//, $date); 
	} elsif($date =~ /(\d{4})-(\d{1,2})-(\d{1,2})/) {
	    ($year,$mon,$mday) = ($1, $2, $3);
	} elsif($date =~ /(\d{4})-(\w{3,})-(\d{1,2})/) {
	    ($year,$mon,$mday) = ($1, $2, $3);
	    $mon = $self->month2num($2);
	} else {
	    print STDERR "\n*** Unsupported input date format: $date\n";
	}
	if(length($year) == 4) { $year = substr $year, 2; }
	$mon -= 1;
    } else {
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = @date =localtime(time);
	return @date if $option =~ /list/i;
    }
    $month_txt = $MONTHS[$mon];
    $day_txt   = $DAYS[$wday] if defined $wday;
    $month_num = $mon+1;
    $fullYear = $BASE_YEAR+$year; 

#    print "sec: $sec, min: $min, hour: $hour, month: $mon, m-day: $mday, year: $year\nwday: $wday, yday: $yday, dst: $isdst";<STDIN>;

    if( $option =~ /yyyy-mm-dd/i ) {
	$date = sprintf "%4d-%02d-%02d",$fullYear,$month_num,$mday;
    } elsif( $option =~ /yyyy-dd-mm/i ) {
	$date = sprintf "%4d-%02d-%02d",$fullYear,$mday,$month_num;
    } elsif( $option =~ /yyyy-mmm-dd/i ) {
	$date = sprintf "%4d-%3s-%02d",$fullYear,$month_txt,$mday;
    } elsif( $option =~ /full|unix/i ) {
	$date = sprintf "%3s %3s %2d %02d:%02d:%02d %d",$day_txt, $month_txt, $mday, $hour, $min, $sec, $fullYear;
    } elsif( $option =~ /mdy/i ) {
	$date = "$month_txt $mday, $fullYear";
    } elsif( $option =~ /ymd/i ) {
	$date = $year."\l$month_txt$mday";
    } elsif( $option =~ /dmy/i ) {
	$date = $mday."\l$month_txt$year";
    } elsif( $option =~ /md/i ) {
	$date = "\l$month_txt$mday";
    } elsif( $option =~ /d-m-y/i ) {
	$date = "$mday-$month_txt-$fullYear";
    } elsif( $option =~ /d m y/i ) {
	$date = "$mday $month_txt $fullYear";
    } elsif( $option =~ /year/i ) {
	$date = $fullYear;
    } elsif( $option =~ /dmy/i ) {
	$date = $mday.'-'.$month_txt.'-'.$fullYear;
    } elsif($option and $option !~ /hms/i) {
	print STDERR "\n*** Unrecognized date format request: $option\n";
    }
    
    if( $option =~ /hms/i) {
	$date .= " $hour:$min:$sec" if $date;
	$date ||= "$hour:$min:$sec";
    }

    return $date || join(" ", @date);
}

sub month2num {
# Converts a string containing a name of a month
# and returns the number of the month in the year.
# Example: $Util->month2num("march");  # returns 3
# The string argument must contain at least the first
# three characters of the month's name.
# Case insensitive.
# Throws exception if the conversion fails.
    my ($self, $str) = @_;

    # Get string in proper format for conversion.
    $str = substr($str, 0, 3);
    for(0..$#MONTHS) {
	return $_+1 if $str =~ /$MONTHS[$_]/i;
    } 
    $self->throw("Invalid month name: $str");
}

#----------------------------------------------------------------------
=head1 compress()
 Title     : compress
           : 
 Usage     : $Util->compress(filename);
           :
 Purpose   : Compress a file to conserve disk space.
           :
 Example   : $Util->compress("/usr/people/me/data.txt");
           :
 Returns   : String (name of compressed file, full path).
           :
 Argument  : String (name of file to be compressed, full path).
           : If the supplied filename ends with '.gz' or '.Z',
           : that extension will be removed before attempting to
           : compress.
           :
 Throws    : Exception if file cannot be compressed
           :
 Access    : Public
           :
 Comments  : Attempts to compress using gzip (default compression level).
           : If that fails, will attempt to gzip to a temporary file
           : location (where user will have write premission).
           : If that fails, will attempt to use compress.
           : In some situations, the full path to the gzip executable
           : may be required. This can be specified with the $GNU_PATH
           : package global variable. When installed, $GNU_PATH is an
           : empty string.

See Also   : uncompress()
=cut
#----------------------------------------------------------------------'
sub compress {
    my $self = shift;
    my $fileName = shift;  

    my $compressed = "$fileName.gz";
    if($fileName =~ /(\.gz|\.Z)$/) { $fileName =~ s/$1$//; };
    $DEBUG && print STDERR "gzipping file $fileName";
    my @args = ($GNU_PATH.'gzip', '-f', $fileName);

    if(system(@args) != 0) {
	# gzip failed. Could be that user does not have write permission
	# or that gzip was not found. 
	$compressed = POSIX::tmpnam;                                                 
	$compressed .= '.gz';
	@args = ($GNU_PATH."gzip -f < $fileName > $compressed");
	system(@args) == 0 or do{
	    # Try compress.
	    $compressed = "$fileName.Z";
	    @args = ('compress', '-f', $fileName);
	    system(@args) == 0 or $self->throw("Failed to gzip/compress file $fileName: $!",
					       "Either gzip/compress could not be found or you lack\n".
					       "write permission in this directory.\n".
					       "Confirm current \$GNU_PATH: $GNU_PATH",
					       "Edit \$GNU_PATH in Bio::Root::Utilities.pm if nec.");
	};
    }

    $compressed;  # return the name of the compressed file.
}

#----------------------------------------------------------------------
=head1 uncompress()
 Title     : uncompress
           : 
 Usage     : $Util->uncompress(filename);
           :
 Purpose   : Uncompress a file to conserve disk space.
           :
 Example   : $Util->uncompress("/usr/people/me/data.txt.gz");
           :
 Returns   : String (name of uncompressed file, full path).
           :
 Argument  : String (name of file to be uncompressed, full path).
           : If the supplied filename does not end with '.gz' or '.Z'
           : a '.gz' will be appended before attempting to
           : uncompress.
           :
 Throws    : Exception if file cannot be uncompressed
           :
 Access    : Public
           :
 Comments  : Attempts to uncompress using gunzip.
           : If that fails, will attempt to gunzip to a temporary file
           : location (where user will have write premission).
           : If that fails, will use uncompress.
           : In some situations, the full path to the gzip executable
           : may be required. This can be specified with the $GNU_PATH
           : package global variable. When installed, $GNU_PATH is an
           : empty string.

See Also   : compress()
=cut
#----------------------------------------------------------------------'
sub uncompress {
    my $self = shift;
    my $fileName = shift;  
    if(not $fileName =~ /(\.gz|\.Z)$/) { $fileName .= '.gz'; }
    $DEBUG && print STDERR "gunzipping file $fileName";
    my @args = ($GNU_PATH.'gunzip', '-f', $fileName);
    my $unzipped = $fileName;
    $unzipped =~ s/(\.gz|\.Z)$//;

    if(system(@args) != 0) {
	# gzip failed. Could be that user does not have write permission.
	$unzipped = POSIX::tmpnam;                                                 
	@args = ($GNU_PATH."gunzip -f < $fileName > $unzipped");
	system(@args) == 0 or do{
	    # gunzip may not be present. Try uncompress.
	    ($unzipped = $fileName) =~ s/(\.gz|\.Z)$//;
	    @args = ('uncompress', '-f', $fileName);
	    $self->throw("Failed to gunzip/uncompress file $fileName: $!",
			 "Confirm current \$GNU_PATH: $GNU_PATH",
			 "Edit \$GNU_PATH in Bio::Root::Utilities.pm in necessary."); 
	};
    } 
    
    $unzipped;  # return the name of the uncompressed file.
}


#------------------------------------------------------------------------
=head1 file_date()
 Title    : file_date
          :
 Usage    : $Util->file_date( filename [,date_format])
          :
 Purpose  : Obtains the date of a given file.
          : Provides flexible formatting via date_format().
          :
 Returns  : String = date of the file as: yyyy-mm-dd (e.g., 1997-10-15)
          :
 Argument : filename = string, full path name for file
          : date_format = string, desired format for date (see date_format()).
          :               Default = yyyy-mm-dd
          :
 Thows    : Exception if no file is provided or does not exist.
          :
 Comments : Uses `ls -lga` to get file date data.
          :
          : Not so universal and not taint-safe. 
          : Think about using a standard CPAN module for this, like
          : Date::Manip or Date::DateCalc. However, these are pretty heavy-duty.
          : I just want a simple function.
=cut
#------------------------------------------------------------------------
sub file_date {
    my ($self, $file, $fmt) = @_;

    $self->throw("No such file: $file") if not $file or not -e $file;

    $fmt ||= 'yyyy-mm-dd';
    my($fmon,$fday,$fyear);

    my $file_data = `ls -lga $file`;
    # Parsing lines such as:
    # -rwxr-xr-x   1 sac      sac          7028 Nov  5  1997 ./YDR222W.blastp
    # -rwxrwxr-x   1 sac      sac           525 Jun 23 16:20 ./utilities.p
    if($file_data =~ / ([a-z]{3})  ?([\d]{1,2})  ?([\d:]+) /i) {
	($fmon,$fday,$fyear) = ($1, $2, $3);
    } else {
	# If the regexp fails, split().
	my @file_data = split /\s+/, $file_data;
	($fmon,$fday,$fyear) = @file_data[5..7];
    }

    if($fyear =~ /:/) { $fyear = $self->date_format('year'); }
    my $fmon_num = '00';
    foreach(0..11) { if($MONTHS[$_] eq $fmon) {$fmon_num = $_ + 1; last;} }

    my $date = sprintf "%4d-%02d-%02d",$fyear,$fmon_num,$fday;

    if($fmt =~ /yyyy-mmm-dd/i ) {
	$date = sprintf "%4d-%3s-%02d",$fyear,$fmon,$fday;
    } elsif($fmt eq 'd m y' ) {
	$date = sprintf "%s %s %s",$fday,$fmon,$fyear;
    }
    $date;
}


sub untaint {
    # This general untaint() function may not be appropriate for every situation.
    # To allow only a more restricted subset of special characters 
    # (for example, untainting a regular expression), then it would be better to
    # use a custom untainting mechanism.
    # 
    # Purpose: To remove nasty shell characters from untrusted data
    #          and allow a script to run with the -T switch.
    # Potentially dangerous shell meta characters:  &;`'\"|*?~<>^()[]{}$\n\r
    # Accept only the first block of contiguous
    # words, hyphens, periods, underscores, single quotes, commas, spaces, and parens.
    # If $relax is set, a less restrictive version is used which allows: /%=:^<>* 

    # This is for special trusted vars (like $0) which cause problems when running with -T.
	
    my($self,$value,$relax) = @_;
    $relax ||= 0;
    my $untainted;

    $DEBUG and print STDERR "\nUNTAINT: $value\n";
    
    defined $value || return;

    if( $relax ) {
	$value =~ /([-\w.\', ()\/=%:^<>*]+)/;
	$untainted = $1
#    } elsif( $relax == 2 ) {  # Could have several levels of relax.
#	$value =~ /([-\w.\', ()\/=%:^<>*]+)/;
#	$untainted = $1
    } else {
	$value =~ /([-\w.\', ()]+)/;
	$untainted = $1
    }

    $DEBUG and print STDERR "UNTAINTED: $untainted\n";

    $untainted;
}


sub is_tainted {
## Used for testing variables for taintedness. 
## Development use only. It always generates a warning,
## which is why it is commented out.

#    my $self = shift;
#    local($^W) = 0;    ## disable warnings with -w (run-time only)
#    # Test for tained data. See the Camel book 2ed, p.358.
#    not eval {
#	  join("",@_), kill 0;
#	  1;
#    };
}



#------------------------------------------------------------------------
=head1 mean_stdev
 Title    : mean_stdev
          :
 Usage    : ($mean, $stdev) = $Util->mean_stdev( @data )
          :
 Purpose  : Calculates the mean and standard deviation given a list of numbers.
          :
 Returns  : 2-element list (mean, stdev)
          :
 Argument : list of numbers (ints or floats)
          :
 Thows    : n/a

=cut
#------------------------------------------------------------------------
sub mean_stdev {
    my ($self, @data) = @_;
    my $mean = 0;
    foreach (@data) { $mean += $_; }
    $mean /= scalar @data;
    my $sum_diff_sqd = 0;
    foreach (@data) { $sum_diff_sqd += ($mean - $_) * ($mean - $_); }
    my $stdev = sqrt(abs($sum_diff_sqd/(scalar @data)-1));
    return ($mean, $stdev);
}

# Stubs for AUTOLOADED methods:
sub count_files;
sub file_info;
sub gunzip;
sub gzip;
sub delete;
sub include_path;
sub array2string;
sub parse_argv;
sub print_hol;
sub print_welcome;
sub print_menu_header;
sub report_error;
sub retrieve_scalar;
sub request_data;
sub mail_authority;
sub send_mail;
sub verify_version;
sub yes_reply;
sub quit_reply;

1;
#__END__

#######################################################################
#                        AUTOLOADED METHODS
#######################################################################

#-----------------------------------------------------------------------
=head1 count_files()
 Title    : count_files
          :
 Purpose  : Counts the number of files/directories within a given directory.
          : Also reports the number of text and binary files in the dir
          : as well as names of these files and directories.
     	  :                            
 Usage    : count_files(\%data)
          :   $data{-DIR} is the directory to be analyzed. Default is ./
          :   $data{-PRINT} = 0|1; if 1, prints results to STDOUT, (default=0).
          :
 Argument : Hash reference
          :
 Returns  : n/a;
          : Modifies the hash ref passed in as the sole argument.
          :  $$href{-TOTAL}            scalar
          :  $$href{-NUM_TEXT_FILES}   scalar
          :  $$href{-NUM_BINARY_FILES} scalar
          :  $$href{-NUM_DIRS}         scalar
          :  $$href{-T_FILE_NAMES}     array ref
          :  $$href{-B_FILE_NAMES}     array ref
          :  $$href{-DIRNAMES}         array ref
=cut
#-----------------------------------------------------------------------
sub count_files {
    my $self = shift;
    my $href = shift;   # Reference to a hash.
    my( $name, @fileLine);
    my $dir = $$href{-DIR} || './';
    my $print = $$href{-PRINT} || 0;
    
    ### Make sure $dir ends with /
    $dir !~ /\/$/ and do{ $dir .=  '/'; $$href{-DIR} = $dir; };
    
    open ( PIPE, "ls -1 $dir |" ) || $self->throw("Can't open input pipe: $!");

    ### Initialize the hash data.
    $$href{-TOTAL} = 0;
    $$href{-NUM_TEXT_FILES} = $$href{-NUM_BINARY_FILES} = $$href{-NUM_DIRS} = 0;
    $$href{-T_FILE_NAMES} = [];
    $$href{-B_FILE_NAMES} = [];
    $$href{-DIR_NAMES} = [];
    while( <PIPE> ) {
	chomp();
	$$href{-TOTAL}++;
	if( -T $dir.$_ ) {
	    $$href{-NUM_TEXT_FILES}++; push @{$$href{-T_FILE_NAMES}}, $_; }
	if( -B $dir.$_ and not -d $dir.$_) {
	    $$href{-NUM_BINARY_FILES}++; push @{$$href{-B_FILE_NAMES}}, $_; }
	if( -d $dir.$_ ) {
	    $$href{-NUM_DIRS}++; push @{$$href{-DIR_NAMES}}, $_; }
    }
    close PIPE;
    
    if( $print) {
	printf( "\n%4d %s\n", $$href{-TOTAL}, "total files+dirs in $dir");
	printf( "%4d %s\n", $$href{-NUM_TEXT_FILES}, "text files");
	printf( "%4d %s\n", $$href{-NUM_BINARY_FILES}, "binary files");
	printf( "%4d %s\n", $$href{-NUM_DIRS}, "directories");
    }
}


sub num2month {
# Does the opposite of month2num.
# Converts a number into a string containing a name of a month.
# Example: $Util->num2month(3);  # returns 'Mar'
# Throws exception if supplied number is out of range.
    my ($self, $num) = @_;

    $self->throw("Month out of range: $num") if $num < 1 or $num > 12;
    return $MONTHS[$num];
}

########################################
#                                      #
#    Begin File Manipulation Methods   #
#                                      #
########################################

sub file_info {
## Obtains a variety of date for a given file from by calling 
## Perl's stat() on the file.
## Status : Under development. Not ready. Don't use!

    my ($self, %param) = @_;
    my ($file, $get, $fmt) = $self->_rearrange([qw(FILE GET FMT)], %param);
    $get ||= 'all';
    $fmt ||= 'yyyy-mm-dd';

    my($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
       $atime, $mtime, $ctime, $blksize, $blocks) = stat $file;

    if($get =~ /date/i) {
	## I can  get the elapsed time since the file was modified but
	## it's not so straightforward to get the date in a nice format...
        ## Think about using a standard CPAN module for this, like
        ## Date::Manip or Date::DateCalc.

	my $date = $mtime;
	my $elsec = time - $mtime;
	printf "\nFile age: %.0f sec %.0f hrs %.0f days", $elsec, $elsec/3600, $elsec/(3600*24);<STDIN>;
	my $days = sprintf "%.0f", $elsec/(3600*24);
    } elsif($get eq 'all') {
	return stat $file;
    }
}

# Deprecated
sub gunzip { 
    my $self = shift; 
    $self->warn("Deprecated method: gunzip(). Use compress() instead.");
    $self->uncompress(@_);
}

# Deprecated
sub gzip { 
    my $self = shift; 
    $self->warn("Deprecated method: gzip(). Use compress() instead.");
    $self->compress(@_); 
} 

sub delete { 
    my $self = shift; 
    my $fileName = shift;
    return unless -e $fileName;
    $DEBUG && print STDERR "deleting file $fileName";
    my @args = ('/usr/bin/rm', $fileName);
    system(@args) == 0 or $self->throw("Failed to delete file $fileName: $!","Tried to execute /usr/bin/rm"); 
}

######################################
#####     Mail Functions      ########
######################################

sub mail_authority {
    
    my( $self, $message ) = @_;
    my $script = $self->untaint($0,1);

    send_mail( -TO=>$AUTHORITY, -SUBJ=>$script, -MSG=>$message);

}


#------------------------------------------------------------------------
=head1 send_mail()
 Title    : send_mail
          :
 Usage    : $Util->send_mail( named_parameters )
          :
 Purpose  : Provides an interface to /usr/lib/sendmail  
          :
 Returns  : n/a
          :
 Argument : Named parameters:  (case-insensitive)
          :  -TO   => e-mail address to send to
          :  -SUBJ => subject for message  (optional)
          :  -MSG  => message to be sent   (optional)
          :  -CC   => cc: e-mail address   (optional)
          :
 Thows    : Exception TO: address appears bad or is missing
          :
 Comments : Based on  TomC's tip at:
          :   http://www.perl.com/CPAN-local/doc/FMTEYEWTK/safe_shellings
          :
          : Using default 'From:' information.
          :   sendmail options used:
          :      -t: ignore the address given on the command line and 
          :          get To:address from the e-mail header.
          :     -oi: prevents send_mail from ending the message if it 
          :          finds a period at the start of a line.

=cut
#------------------------------------------------------------------------'
sub send_mail {
    my( $self, @param) = @_;
    my($recipient,$subj,$message,$cc) = $self->_rearrange([qw(TO SUBJ MSG CC)],@param);

    $self->throw("Invalid or missing e-mail address: $recipient") 
	if not $recipient =~ /\S+\@\S+/;

    $cc ||= ''; $subj ||= ''; $message ||= '';

    open (SENDMAIL, "|/usr/lib/sendmail -oi -t") || 
	$self->throw("Can't send mail: sendmail cannot fork: $!");

print SENDMAIL <<QQ_EOF_QQ;
To: $recipient
Subject: $subj
Cc: $cc

$message

QQ_EOF_QQ

    close(SENDMAIL);
    if ($?) { warn "sendmail didn't exit nicely: $?" }
}


# An older send_mail() routine that I wrote

#sub send_mail {
#    
#    my( @param) = @_;
#    my($recipient,$subj,$message) = $self->rearrange([qw(TO SUBJ MSG)],@param);
#
#    print "Sending mail:\nTO: $recipient\nSUBJ:$subj\nMSG:$message";<STDIN>;
#
#    open( MAIL, qq/| mailx -s "$subj" $recipient/) || 
#	 return $self->warn("Can't open mail pipe.","Cause: ".$self->untaint($!,1),"$ID sendMail");
#    
#    print MAIL $message;
#    
#    close MAIL;
#}


######################################
###   Interactive Functions      #####
######################################

#-----------------------------------------------------------------------
=head1 yes_reply()
 Title   : yes_reply()
         :
 Usage   : $Util->yes_reply( [query_string]);
         :
 Purpose : To test an STDIN input value for affirmation.
         :
 Example : print +( $Util->yes_reply('Are you ok') ? "great!\n" : "sorry.\n" );
         : $Util->yes_reply('Continue') || die;
         :
 Returns : Boolean, true (1) if input string begins with 'y' or 'Y' 
         :
 Argument: query_string = string to be used to prompt user (optional)
         : If not provided, 'Yes or no' will be used.
         : Question mark is automatically appended.
=cut
#-----------------------------------------------------------------------
sub yes_reply {
    my $self = shift;
    my $query = shift;
    my $reply;
    $query ||= 'Yes or no';
    print "\n$query? (y/n) [n] ";
    chomp( $reply = <STDIN> );
    $reply =~ /^y/i;
}


#-----------------------------------------------------------------------
=head1 request_data()
 Title   : request_data()
         :
 Usage   : $Util->request_data( [value_name]);
         :
 Purpose : To request data from a user to be entered via keyboard (STDIN).
         :
 Example : $name = $Util->request_data('Name');
         : # User will see: % Enter Name: 
         :
 Returns : String, (data entered from keyboard, sans terminal newline.)
         :
 Argument: value_name = string to be used to prompt user.
         : If not provided, 'data' will be used, (not very helpful).
         : Question mark is automatically appended.
=cut
#-----------------------------------------------------------------------
sub request_data {
    my $self = shift;
    my $data = shift || 'data';
    print "Enter $data: ";
    # Remove the terminal newline char.
    chomp($data = <STDIN>);
    $data;
}

sub quit_reply {
# Not much used since you can use request_data() 
# and test for an empty string.
    my $self = shift;
    my $reply;
    chop( $reply = <STDIN> );
    $reply =~ /^q.*/i;
}



###############################################
#                                             #
#  Begin Miscellaneous Defunct Methods        #
#                                             #
###############################################


sub parse_argv {
    
    # WARNING: This functionality (and more) is already provided by Getopts::Long
    #          that comes with the standard Perl distribution,
    #          which is highly recommended!
    #          I created this method before I knew about Getopts.
    #          It was a useful exercise, though.

    # Currently this function handles arguments specified like "get.pl -i=myFileName -tup=2"
    # where the argument type is specified by a hyphen+char(s)=value.
    # All arguments that fit this format are removed from @ARGV.
    # All arguments not fitting this format are left in @ARGV.
    #
    # Arguments  : $_[0] is a reference to a hash, 
    #            : $_[1] is an array (if not defined, @ARGV is used).
    # Returns    : Modifies the hash reference directly, so nothing special is returned.
    #            :
    # Status     : Deprecated. Use Getopt::Long in the standard Perl distribution.

    my( $self, $href) = @_;
    my @args = @ARGV;
    my @unprocessed = ();
#    print STDERR "\nParsing ARGV:\n@ARGV";
    foreach( @args ) {
	if( /^(-\w+)=(.*)/ ) {
	    $href->{$1} = $2;
	} elsif( /^(-\w+)/ ) {
	    $href->{$1}++;
	} else {
	    push @unprocessed, $_;
	}
    }
    @ARGV = @unprocessed;
}


##########################################
sub include_path {
## This is obsolete, but was an interesting learning experience.

    my $self = shift;
    my $path = shift;
    
    not( grep $_ eq $path, @INC) and unshift( @INC, $path );
    
#	print "INC:\n@INC\n"; <STDIN>;
}

sub array2string {
# PURPOSE: convert an array into a multiline string
# NOTE   : Just use join("\n",@_), then you can save a method call.
    my $self = shift;
    my $str="";
    foreach (@_) { $str.="$_\n";  }
    $str;
}



sub print_hol {
# Title    : print_hol
# Purpose : Prints the contents of a hash of lists.
# Usage   : print_hol( \%hashRef, 'filename', 'Hash Title' );
#           If 'filename' is '0' (zero), output will be sent to STDOUT.
# Modified: 1996-Mar-14
    my $self = shift;
    my $HoLref    = $_[0];
    my $file      = $_[1];
    my $title     = $_[2];
    my $where     = '';
    
    if( $file ne '0' ) {
	$where = 'OUTFILE';
	open( $where, ">$file" ) 
	    || die "Can't open file $file for output: $!\n";
    } 
    
    print $where "HASH OF LIST PRINTOUT\n\n";
    print $where "HASH NAME: $title\n";
    print $where "\n------------------------------------\n";
    
    printf( $where "\n%-15s\t%-25s\n", 'HASH KEY', 'LIST CONTENTS' );
    printf( $where "%-15s\t%-25s\n", '--------', '-------------' );
    
    my($hashKey,$listRef);
    for $hashKey ( sort keys %$HoLref ) {
	printf( $where "%-15s\n", $hashKey );
	$listRef = ( $HoLref->{ $hashKey } );
	for( @$listRef ) {
	    printf( $where "%-15s\t%-25s\n", '', $_ );
	}
	print $where "\n------------------------------------\n";
    }
}


############################################
#           Printing Functions             #
############################################

sub print_welcome {
    
    my $self = shift;
    my $version  = shift;
    my $modified = shift;
    my $system   = shift;
    my $os       = 'undef';
    
    if( exists $ENV{ 'OSNAME' } ) {
	$os = $ENV{ 'OSNAME' };
    }elsif( exists $ENV{ 'OSTYPE' }) {
	$os = $ENV{ 'OSTYPE' };
    }		
    
    print "-------------------------------------------------------\n";;
    printf ( "%s: %s\n", "PROGRAM", $0 );
    printf ( "%s: %3.1f %s (%s), %s %0.3f\n", "VERSION", $version, "for $system", $os, "PERL version", $] );
    printf ( "%s: %s\n\n", "MODIFIED", $modified );
    $system eq 'UNIX' and print "EXECUTION STARTED: ".scalar localtime;
    print "-------------------------------------------------------\n";;
}


sub print_menu_header {
    
    my $self = shift;
    my $title = shift;
    
    print "\n\n\n----------------< $title >------------------\n\n";
    
}		


######################################
#         retrieve_scalar()           #
######################################
#
# Purpose: To extract a specific substring from a larger string containing special identifiers.
#          An early attempt for parsing SGML-like key-value data.
#
# E.g. $$data_ref: "blah blah blah blah <ORF>YER020w<tab> blah blah blah blah" 
#                   Using a reference since this could be an enormous string.
#            $tag: '<ORF>'
#         returns: YER020w
#
# NOTE: The '<tab>' included in the above $$data_ref is actually just a \t character immediately 
#       following the datum to be extracted.  
#
# Status : defunct 
#
sub retrieve_scalar {
    my $self = shift;
    my $tag       = shift;
    my $data_ref  = shift;
    
    not defined $$data_ref and return 'undef';   # NEEDS TO BE PRINTABLE, CAN'T JUST RETURN undef.
    
    my @firstSplit = split( /$tag/, $$data_ref );
    
    not defined $firstSplit[1] and return 'undef';
    
    my @secondSplit = split( /\t/, $firstSplit[1] );
    
    not defined $secondSplit[0] and return 'undef';
    
    my $data = $secondSplit[0];
    
    if( $DEBUG ) {
	print STDERR "\n\nSPLIT: $$data_ref \t ON---> $tag";  
	print STDERR "\nSPLIT: $firstSplit[1] \t ON---> tabs";
	print STDERR "\nRETURN: --->$data<---\n\n";
    }
    
    $data; 
}



######################################
####     verify_version()      ########
######################################
##
## DESCRIPTION: Checks the version of Perl used to invoke the script.
##              Aborts program if version is less than the given argument.
##		                               
## USAGE: verify_version( '5.000' )
##
## MODIFIED: sac --- Wed May 15 10:34:00 1996

sub verify_version {
    
    my $self = shift;
    my $reqVersion  = shift;
    
    $] < $reqVersion and do { 
	printf STDERR ( "\a\n%s %0.3f.\n", "** Sorry. This Perl script requires at least version", $reqVersion);
	printf STDERR ( "%s %0.3f %s\n\n", "You are running Perl version", $], "Please update your Perl!" );
	print STDERR "Exiting.\n\n";
	exit(1);
    }
}


#####################################################################################
#                            END OF PACKAGE UTILITIES                               #
#####################################################################################

1;
